@@ -57,7 +57,7 @@ ReferenceCloud* ManualSegmentationTools::segment(GenericIndexedCloudPersist* aCl
 		{
 			if (!Y->addPointIndex(i))
 			{
-				//not engouh memory
+				//not enough memory
 				delete Y;
 				Y = nullptr;
 				break;
@@ -133,6 +133,39 @@ bool ManualSegmentationTools::isPointInsidePoly(const CCVector2& P,
 	return inside;
 }
 
+ReferenceCloud* ManualSegmentationTools::segmentReferenceCloud(ReferenceCloud* cloud,
+	ScalarType minDist,
+	ScalarType maxDist,
+	bool outside/*=false*/)
+{
+	if (!cloud)
+	{
+		assert(false);
+		return nullptr;
+	}
+	ReferenceCloud* Y = new ReferenceCloud(cloud->getAssociatedCloud());
+
+	//for each point
+	for (unsigned i = 0; i < cloud->size(); ++i)
+	{
+		const ScalarType dist = cloud->getPointScalarValue(i);
+		//we test if its associated scalar value falls inside the specified interval
+		if ((dist >= minDist && dist <= maxDist) ^ outside)
+		{
+			if (!Y->addPointIndex(cloud->getPointGlobalIndex(i)))
+			{
+				//not enough memory
+				delete Y;
+				Y = nullptr;
+				break;
+			}
+		}
+	}
+
+	return Y;
+
+}
+
 ReferenceCloud* ManualSegmentationTools::segment(	GenericIndexedCloudPersist* cloud,
 													ScalarType minDist,
 													ScalarType maxDist,
@@ -144,6 +177,10 @@ ReferenceCloud* ManualSegmentationTools::segment(	GenericIndexedCloudPersist* cl
 		return nullptr;
 	}
 
+	ReferenceCloud* cloudREFTest = dynamic_cast<ReferenceCloud*>(cloud);
+	if (cloudREFTest)
+		return segmentReferenceCloud(cloudREFTest, minDist, maxDist, outside);
+
 	ReferenceCloud* Y = new ReferenceCloud(cloud);
 
 	//for each point
@@ -155,7 +192,7 @@ ReferenceCloud* ManualSegmentationTools::segment(	GenericIndexedCloudPersist* cl
 		{
 			if (!Y->addPointIndex(i))
 			{
-				//not engouh memory
+				//not enough memory
 				delete Y;
 				Y = nullptr;
 				break;
@@ -608,7 +645,7 @@ bool ImportSourceVertices(GenericIndexedCloudPersist* srcVertices,
 
 	return true;
 }
-bool ManualSegmentationTools::segmentMeshWitAAPlane(GenericIndexedMesh* mesh,
+bool ManualSegmentationTools::segmentMeshWithAAPlane(GenericIndexedMesh* mesh,
 	GenericIndexedCloudPersist* vertices,
 	MeshCutterParams& ioParams,
 	GenericProgressCallback* progressCb/*=0*/)
@@ -670,7 +707,8 @@ bool ManualSegmentationTools::segmentMeshWitAAPlane(GenericIndexedMesh* mesh,
 
 			//test each vertex
 			//char relativePos[3] = { 1, 1, 1 };
-			std::vector<unsigned char> minusVertIndexes, plusVertIndexes;
+			std::vector<unsigned char> minusVertIndexes;
+			std::vector<unsigned char> plusVertIndexes;
 			for (unsigned char j = 0; j < 3; ++j)
 			{
 				const CCVector3d& v = V[j];
@@ -743,7 +781,8 @@ bool ManualSegmentationTools::segmentMeshWitAAPlane(GenericIndexedMesh* mesh,
 					unsigned char iMinus = minusVertIndexes.front();
 					unsigned char iPlus = plusVertIndexes.front();
 
-					unsigned iCoutside,iCinside;
+					unsigned iCoutside;
+					unsigned iCinside;
 					if (!ComputeEdgePoint(V[iMinus], origVertIndexes[iMinus],
 						V[iPlus], origVertIndexes[iPlus],
 						iCoutside, iCinside,
@@ -796,7 +835,9 @@ bool ManualSegmentationTools::segmentMeshWitAAPlane(GenericIndexedMesh* mesh,
 				else
 				{
 					//we have one vertex on one side and two on the other side
-					unsigned char iLeft, iRight1, iRight2;
+					unsigned char iLeft;
+					unsigned char iRight1;
+					unsigned char iRight2;
 					bool leftIsMinus = true;
 					if (minusVertIndexes.size() == 1)
 					{
@@ -816,8 +857,10 @@ bool ManualSegmentationTools::segmentMeshWitAAPlane(GenericIndexedMesh* mesh,
 					}
 
 					//the plane cuts through the two edges having the 'single' vertex in common
-					unsigned i1outside, i1inside;
-					unsigned i2outside, i2inside;
+					unsigned i1outside;
+					unsigned i1inside;
+					unsigned i2outside;
+					unsigned i2inside;
 					if (!ComputeEdgePoint(V[iRight1], origVertIndexes[iRight1], V[iLeft], origVertIndexes[iLeft], i1outside, i1inside, planeZ, Z, outsideVertices, insideVertices)
 					||	!ComputeEdgePoint(V[iRight2], origVertIndexes[iRight2], V[iLeft], origVertIndexes[iLeft], i2outside, i2inside, planeZ, Z, outsideVertices, insideVertices))
 					{
@@ -895,7 +938,7 @@ bool ManualSegmentationTools::segmentMeshWitAAPlane(GenericIndexedMesh* mesh,
 	return true;
 }
 
-bool ManualSegmentationTools::segmentMeshWitAABox(GenericIndexedMesh* origMesh,
+bool ManualSegmentationTools::segmentMeshWithAABox(GenericIndexedMesh* origMesh,
 	GenericIndexedCloudPersist* origVertices,
 	MeshCutterParams& ioParams,
 	GenericProgressCallback* progressCb/*=0*/)
@@ -1044,7 +1087,8 @@ bool ManualSegmentationTools::segmentMeshWitAABox(GenericIndexedMesh* origMesh,
 				//test the position of each vertex relatively to the current plane
 				//char relativePos[3] = { 1, 1, 1 };
 				//bool insideXY[3] = { false, false, false };
-				std::vector<unsigned char> insideLocalVertIndexes, outsideLocalVertIndexes;
+				std::vector<unsigned char> insideLocalVertIndexes;
+				std::vector<unsigned char> outsideLocalVertIndexes;
 				for (unsigned char j = 0; j < 3; ++j)
 				{
 					const CCVector3d& v = V[j];
@@ -1118,7 +1162,8 @@ bool ManualSegmentationTools::segmentMeshWitAABox(GenericIndexedMesh* origMesh,
 						unsigned char iOuside = outsideLocalVertIndexes.front();
 
 						unsigned char iCenter = 3 - iInside - iOuside;
-						unsigned iCoutside, iCinside;
+						unsigned iCoutside;
+						unsigned iCinside;
 						//we can now create one vertex and two new triangles
 						if (!ComputeEdgePoint(
 							V[iInside], vertIndexes[iInside],
@@ -1176,7 +1221,9 @@ bool ManualSegmentationTools::segmentMeshWitAABox(GenericIndexedMesh* origMesh,
 					else
 					{
 						//we have one vertex on one side and two on the other side
-						unsigned char iLeft, iRight1, iRight2;
+						unsigned char iLeft;
+						unsigned char iRight1;
+						unsigned char iRight2;
 						bool leftIsInside = true;
 						if (insideLocalVertIndexes.size() == 1)
 						{
@@ -1197,8 +1244,10 @@ bool ManualSegmentationTools::segmentMeshWitAABox(GenericIndexedMesh* origMesh,
 
 						//the plane cuts through the two edges having the 'single' vertex in common
 						//we are going to create 3 triangles
-						unsigned i1outside, i1inside;
-						unsigned i2outside, i2inside;
+						unsigned i1outside;
+						unsigned i1inside;
+						unsigned i2outside;
+						unsigned i2inside;
 						if (  !ComputeEdgePoint(	V[iRight1], vertIndexes[iRight1],
 													V[iLeft], vertIndexes[iLeft],
 													i1outside, i1inside,
@@ -1360,12 +1409,12 @@ bool ManualSegmentationTools::segmentMeshWitAABox(GenericIndexedMesh* origMesh,
 		if (	!MergeOldTriangles(	origMesh, origVertices,
 									insideMesh, insideVertices,
 									*preservedTrianglesInside,
-									ioParams.trackOrigIndexes ? &ioParams.origTriIndexesMapInside : 0)
+									ioParams.trackOrigIndexes ? &ioParams.origTriIndexesMapInside : nullptr)
 			||	(	ioParams.generateOutsideMesh
 				&&	!MergeOldTriangles(	origMesh, origVertices,
 										outsideMesh, outsideVertices,
 										preservedTrianglesOutside,
-										ioParams.trackOrigIndexes ? &ioParams.origTriIndexesMapOutside : 0))
+										ioParams.trackOrigIndexes ? &ioParams.origTriIndexesMapOutside : nullptr))
 			)
 		{
 			error = true;
