@@ -247,7 +247,7 @@ bool Neighbourhood::computeLeastSquareBestFittingPlane()
 	unsigned pointCount = (m_associatedCloud ? m_associatedCloud->size() : 0);
 
 	//we need at least 3 points to compute a plane
-	assert(CC_LOCAL_MODEL_MIN_SIZE[LS] >= 3);
+	static_assert(CC_LOCAL_MODEL_MIN_SIZE[LS] >= 3, "Invalid CC_LOCAL_MODEL_MIN_SIZE size");
 	if (pointCount < CC_LOCAL_MODEL_MIN_SIZE[LS])
 	{
 		//not enough points!
@@ -359,7 +359,7 @@ bool Neighbourhood::computeQuadric()
 
 	unsigned count = m_associatedCloud->size();
 	
-	assert(CC_LOCAL_MODEL_MIN_SIZE[QUADRIC] >= 5);
+	static_assert(CC_LOCAL_MODEL_MIN_SIZE[QUADRIC] >= 5, "Invalid CC_LOCAL_MODEL_MIN_SIZE size");
 	if (count < CC_LOCAL_MODEL_MIN_SIZE[QUADRIC])
 		return false;
 
@@ -394,7 +394,8 @@ bool Neighbourhood::computeQuadric()
 	}
 
 	//compute the A matrix and b vector
-	std::vector<float> A, b;
+	std::vector<float> A;
+	std::vector<float> b;
 	try
 	{
 		A.resize(6 * count, 0);
@@ -778,7 +779,8 @@ GenericIndexedMesh* Neighbourhood::triangulateFromQuadric(unsigned nStepX, unsig
 	assert(G);
 
 	//bounding box
-	CCVector3 bbMin, bbMax;
+	CCVector3 bbMin;
+	CCVector3 bbMax;
 	m_associatedCloud->getBoundingBox(bbMin, bbMax);
 	CCVector3 bboxDiag = bbMax - bbMin;
 
@@ -853,7 +855,8 @@ ScalarType Neighbourhood::computeMomentOrder1(const CCVector3& P)
 
 	Jacobi<double>::SortEigenValuesAndVectors(eigVectors, eigValues); //sort the eigenvectors in decreasing order of their associated eigenvalues
 
-	double m1 = 0.0, m2 = 0.0;
+	double m1 = 0.0;
+	double m2 = 0.0;
 	CCVector3d e2;
 	Jacobi<double>::GetEigenVector(eigVectors, 1, e2.u);
 
@@ -950,6 +953,15 @@ double Neighbourhood::computeFeature(GeomFeature feature)
 			value = 1.0 - std::abs(Z.dot(e3));
 		}
 		break;
+	case EigenValue1:
+		value = l1;
+		break;
+	case EigenValue2:
+		value = l2;
+		break;
+	case EigenValue3:
+		value = l3;
+		break;
 	default:
 		assert(false);
 		break;
