@@ -54,7 +54,8 @@ PointCloud* PointProjectionTools::developCloudOnCylinder(	GenericCloud* cloud,
 	CCVector3 C;
 	if (!center)
 	{
-		CCVector3 bbMin,bbMax;
+		CCVector3 bbMin;
+		CCVector3 bbMax;
 		cloud->getBoundingBox(bbMin,bbMax);
 		C = (bbMin+bbMax)/2;
 		center = &C;
@@ -135,7 +136,7 @@ PointCloud* PointProjectionTools::developCloudOnCone(GenericCloud* cloud, unsign
 		progressCb->start();
 	}
 
-	for (unsigned i=0; i<count; i++)
+	for (unsigned i = 0; i < count; i++)
 	{
 		const CCVector3 *Q = cloud->getNextPoint();
 		CCVector3 P = *Q-center;
@@ -249,7 +250,7 @@ GenericIndexedMesh* PointProjectionTools::computeTriangulation(	GenericIndexedCl
 																CC_TRIANGULATION_TYPES type/*=DELAUNAY_2D_AXIS_ALIGNED*/,
 																PointCoordinateType maxEdgeLength/*=0*/,
 																unsigned char dim/*=0*/,
-																char* errorStr/*=0*/)
+																char* errorStr/*=nullptr*/)
 {
 	if (!cloud)
 	{
@@ -259,7 +260,7 @@ GenericIndexedMesh* PointProjectionTools::computeTriangulation(	GenericIndexedCl
 	}
 
 
-	switch(type)
+	switch (type)
 	{
 	case DELAUNAY_2D_AXIS_ALIGNED:
 		{
@@ -270,8 +271,8 @@ GenericIndexedMesh* PointProjectionTools::computeTriangulation(	GenericIndexedCl
 				return nullptr;
 			}
 			const unsigned char Z = static_cast<unsigned char>(dim);
-			const unsigned char X = Z == 2 ? 0 : Z+1;
-			const unsigned char Y = X == 2 ? 0 : X+1;
+			const unsigned char X = (Z == 2 ? 0 : Z + 1);
+			const unsigned char Y = (X == 2 ? 0 : X + 1);
 
 			unsigned count = cloud->size();
 			std::vector<CCVector2> the2DPoints;
@@ -287,7 +288,7 @@ GenericIndexedMesh* PointProjectionTools::computeTriangulation(	GenericIndexedCl
 			}
 
 			cloud->placeIteratorAtBeginning();
-			for (unsigned i=0; i<count; ++i)
+			for (unsigned i = 0; i < count; ++i)
 			{
 				const CCVector3* P = cloud->getPoint(i);
 				the2DPoints[i].x = P->u[X];
@@ -296,14 +297,14 @@ GenericIndexedMesh* PointProjectionTools::computeTriangulation(	GenericIndexedCl
 
 			Delaunay2dMesh* dm = new Delaunay2dMesh();
 			char triLibErrorStr[1024];
-			if (!dm->buildMesh(the2DPoints,0,triLibErrorStr))
+			if (!dm->buildMesh(the2DPoints, 0, triLibErrorStr))
 			{
 				if (errorStr)
 					strcpy(errorStr, triLibErrorStr);
 				delete dm;
 				return nullptr;
 			}
-			dm->linkMeshWith(cloud,false);
+			dm->linkMeshWith(cloud, false);
 
 			//remove triangles with too long edges
 			if (maxEdgeLength > 0)
@@ -321,14 +322,14 @@ GenericIndexedMesh* PointProjectionTools::computeTriangulation(	GenericIndexedCl
 
 			return static_cast<GenericIndexedMesh*>(dm);
 		}
-		break;
+
 	case DELAUNAY_2D_BEST_LS_PLANE:
 		{
 			Neighbourhood Yk(cloud);
-			GenericIndexedMesh* mesh = Yk.triangulateOnPlane(false,maxEdgeLength,errorStr);
+			GenericIndexedMesh* mesh = Yk.triangulateOnPlane(false, maxEdgeLength, errorStr);
 			return mesh;
 		}
-		break;
+
 	default:
 		//shouldn't happen
 		assert(false);
@@ -494,23 +495,26 @@ using Vertex2D = CCLib::PointProjectionTools::IndexedCCVector2;
 using VertexIterator = std::list<Vertex2D *>::iterator;
 using ConstVertexIterator = std::list<Vertex2D *>::const_iterator;
 
-struct Edge
+namespace 
 {
-	Edge() : nearestPointIndex(0), nearestPointSquareDist(-1.0f) {}
-	
-	Edge(const VertexIterator& A, unsigned _nearestPointIndex, float _nearestPointSquareDist)
-		: itA(A)
-		, nearestPointIndex(_nearestPointIndex)
-		, nearestPointSquareDist(_nearestPointSquareDist)
-	{}
-
-	//operator
-	inline bool operator< (const Edge& e) const { return nearestPointSquareDist < e.nearestPointSquareDist; }
-
-	VertexIterator itA;
-	unsigned nearestPointIndex;
-	float nearestPointSquareDist;
-};
+	struct Edge
+	{
+		Edge() : nearestPointIndex(0), nearestPointSquareDist(-1.0f) {}
+		
+		Edge(const VertexIterator& A, unsigned _nearestPointIndex, float _nearestPointSquareDist)
+			: itA(A)
+			, nearestPointIndex(_nearestPointIndex)
+			, nearestPointSquareDist(_nearestPointSquareDist)
+		{}
+		
+		//operator
+		inline bool operator< (const Edge& e) const { return nearestPointSquareDist < e.nearestPointSquareDist; }
+		
+		VertexIterator itA;
+		unsigned nearestPointIndex;
+		float nearestPointSquareDist;
+	};
+}
 
 //! Finds the nearest (available) point to an edge
 /** \return The nearest point distance (or -1 if no point was found!)
@@ -600,7 +604,8 @@ bool PointProjectionTools::extractConcaveHull2D(std::vector<IndexedCCVector2>& p
 	//hack: compute the theoretical 'minimal' edge length
 	PointCoordinateType minSquareEdgeLength = 0;
 	{
-		CCVector2 minP,maxP;
+		CCVector2 minP;
+		CCVector2 maxP;
 		for (std::size_t i = 0; i < pointCount; ++i)
 		{
 			const IndexedCCVector2& P = points[i];
