@@ -129,7 +129,7 @@ void RegistrationTools::FilterTransformation(	const ScaledTransformation& inTran
 struct ModelCloud
 {
 	ModelCloud() : cloud(nullptr), weights(nullptr) {}
-	ModelCloud(const ModelCloud& m) : cloud(m.cloud), weights(m.weights) {}
+	ModelCloud(const ModelCloud& m) = default;
 	GenericIndexedCloudPersist* cloud;
 	ScalarField* weights;
 };
@@ -373,7 +373,8 @@ ICPRegistrationTools::RESULT_TYPE ICPRegistrationTools::Register(	GenericIndexed
 		fprintf(fTraceFile,"Iteration; RMS; Point count;\n");
 #endif
 
-	double lastStepRMS = -1.0, initialDeltaRMS = -1.0;
+	double lastStepRMS = -1.0;
+	double initialDeltaRMS = -1.0;
 	ScaledTransformation currentTrans;
 	RESULT_TYPE result = ICP_ERROR;
 
@@ -393,7 +394,8 @@ ICPRegistrationTools::RESULT_TYPE ICPRegistrationTools::Register(	GenericIndexed
 			N.computeParameters(data.cloud);
 			if (N.isValid())
 			{
-				ScalarType mu,sigma2;
+				ScalarType mu;
+				ScalarType sigma2;
 				N.getParameters(mu,sigma2);
 				ScalarType maxDistance = static_cast<ScalarType>(mu + 2.5*sqrt(sigma2));
 
@@ -864,7 +866,7 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 {
 	//resulting transformation (R is invalid on initialization, T is (0,0,0) and s==1)
 	trans.R.invalidate();
-	trans.T = CCVector3(0,0,0);
+	trans.T = CCVector3(0, 0, 0);
 	trans.s = PC_ONE;
 
 	if (P == nullptr || X == nullptr || P->size() != X->size() || P->size() < 3)
@@ -883,9 +885,9 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 		const CCVector3* Ap = P->getNextPoint();
 		const CCVector3* Bp = P->getNextPoint();
 		const CCVector3* Cp = P->getNextPoint();
-		CCVector3 Np(0,0,1);
+		CCVector3 Np(0, 0, 1);
 		{
-			Np = (*Bp-*Ap).cross(*Cp-*Ap);
+			Np = (*Bp - *Ap).cross(*Cp - *Ap);
 			double norm = Np.normd();
 			if (norm < ZERO_TOLERANCE)
 				return false;
@@ -896,9 +898,9 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 		const CCVector3* Ax = X->getNextPoint();
 		const CCVector3* Bx = X->getNextPoint();
 		const CCVector3* Cx = X->getNextPoint();
-		CCVector3 Nx(0,0,1);
+		CCVector3 Nx(0, 0, 1);
 		{
-			Nx = (*Bx-*Ax).cross(*Cx-*Ax);
+			Nx = (*Bx - *Ax).cross(*Cx - *Ax);
 			double norm = Nx.normd();
 			if (norm < ZERO_TOLERANCE)
 				return false;
@@ -912,17 +914,16 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 			trans.R.toIdentity();
 			if (Np.dot(Nx) < 0)
 			{
-				trans.R.scale(-1);
+				trans.R.scale(-PC_ONE);
 			}
 		}
 		else
 		{
 			double cos_t = Np.dot(Nx);
 			assert(cos_t > -1.0 && cos_t < 1.0); //see above
-			double s = sqrt((1+cos_t)*2);
-			double q[4] = { s/2, a.x/s, a.y/s, a.z/s };
-			//don't forget to normalize the quaternion
-			double qnorm = q[0]*q[0] + q[1]*q[1] + q[2]*q[2] + q[3]*q[3];
+			double s = sqrt((1 + cos_t) * 2);
+			double q[4] = { s / 2, a.x / s, a.y / s, a.z / s }; //don't forget to normalize the quaternion
+			double qnorm = q[0] * q[0] + q[1] * q[1] + q[2] * q[2] + q[3] * q[3];
 			assert(qnorm >= ZERO_TOLERANCE);
 			qnorm = sqrt(qnorm);
 			q[0] /= qnorm;
@@ -934,16 +935,16 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 
 		if (adjustScale)
 		{
-			double sumNormP = (*Bp-*Ap).norm() + (*Cp-*Bp).norm() + (*Ap-*Cp).norm();
+			double sumNormP = (*Bp - *Ap).norm() + (*Cp - *Bp).norm() + (*Ap - *Cp).norm();
 			sumNormP *= aPrioriScale;
 			if (sumNormP < ZERO_TOLERANCE)
 				return false;
-			double sumNormX = (*Bx-*Ax).norm() + (*Cx-*Bx).norm() + (*Ax-*Cx).norm();
+			double sumNormX = (*Bx - *Ax).norm() + (*Cx - *Bx).norm() + (*Ax - *Cx).norm();
 			trans.s = static_cast<PointCoordinateType>(sumNormX / sumNormP); //sumNormX / (sumNormP * Sa) in fact
 		}
 
 		//we deduce the first translation
-		trans.T = Gx - (trans.R*Gp) * (aPrioriScale*trans.s); //#26 in besl paper, modified with the scale as in jschmidt
+		trans.T = Gx - (trans.R*Gp) * (aPrioriScale*trans.s); //#26 in Besl paper, modified with the scale as in jschmidt
 
 		//we need to find the rotation in the (X) plane now
 		{
@@ -953,9 +954,10 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 
 			double C = 0;
 			double S = 0;
-			CCVector3 Ssum(0,0,0);
-			CCVector3 rx,rp;
-			
+			CCVector3 Ssum(0, 0, 0);
+			CCVector3 rx;
+			CCVector3 rp;
+
 			rx = *Ax - Gx;
 			rp = App - Gx;
 			C = rx.dot(rp);
@@ -972,10 +974,10 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 			Ssum += rx.cross(rp);
 
 			S = Ssum.dot(Nx);
-			double Q = sqrt(S*S + C*C);
+			double Q = sqrt(S*S + C * C);
 			if (Q < ZERO_TOLERANCE)
 				return false;
-			
+
 			PointCoordinateType sin_t = static_cast<PointCoordinateType>(S / Q);
 			PointCoordinateType cos_t = static_cast<PointCoordinateType>(C / Q);
 			PointCoordinateType inv_cos_t = 1 - cos_t;
@@ -984,24 +986,24 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 			const PointCoordinateType& l2 = Nx.y;
 			const PointCoordinateType& l3 = Nx.z;
 
-			PointCoordinateType l1_inv_cos_t = l1*inv_cos_t;
-			PointCoordinateType l3_inv_cos_t = l3*inv_cos_t;
+			PointCoordinateType l1_inv_cos_t = l1 * inv_cos_t;
+			PointCoordinateType l3_inv_cos_t = l3 * inv_cos_t;
 
 			SquareMatrix R(3);
 			//1st column
-			R.m_values[0][0] = cos_t + l1*l1_inv_cos_t;
-			R.m_values[0][1] = l2*l1_inv_cos_t+l3*sin_t;
-			R.m_values[0][2] = l3*l1_inv_cos_t-l2*sin_t;
+			R.m_values[0][0] = cos_t + l1 * l1_inv_cos_t;
+			R.m_values[0][1] = l2 * l1_inv_cos_t + l3 * sin_t;
+			R.m_values[0][2] = l3 * l1_inv_cos_t - l2 * sin_t;
 
 			//2nd column
-			R.m_values[1][0] = l2*l1_inv_cos_t-l3*sin_t;
-			R.m_values[1][1] = cos_t+l2*l2*inv_cos_t;
-			R.m_values[1][2] = l2*l3_inv_cos_t+l1*sin_t;
+			R.m_values[1][0] = l2 * l1_inv_cos_t - l3 * sin_t;
+			R.m_values[1][1] = cos_t + l2 * l2*inv_cos_t;
+			R.m_values[1][2] = l2 * l3_inv_cos_t + l1 * sin_t;
 
 			//3rd column
-			R.m_values[2][0] = l3*l1_inv_cos_t+l2*sin_t;
-			R.m_values[2][1] = l2*l3_inv_cos_t-l1*sin_t;
-			R.m_values[2][2] = cos_t+l3*l3_inv_cos_t;
+			R.m_values[2][0] = l3 * l1_inv_cos_t + l2 * sin_t;
+			R.m_values[2][1] = l2 * l3_inv_cos_t - l1 * sin_t;
+			R.m_values[2][2] = cos_t + l3 * l3_inv_cos_t;
 
 			trans.R = R * trans.R;
 			trans.T = Gx - (trans.R*Gp) * (aPrioriScale*trans.s); //update T as well
@@ -1009,22 +1011,23 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 	}
 	else
 	{
-		CCVector3 bbMin,bbMax;
-		X->getBoundingBox(bbMin,bbMax);
+		CCVector3 bbMin;
+		CCVector3 bbMax;
+		X->getBoundingBox(bbMin, bbMax);
 
 		//if the data cloud is equivalent to a single point (for instance
 		//it's the case when the two clouds are very far away from
 		//each other in the ICP process) we try to get the two clouds closer
-		CCVector3 diag = bbMax-bbMin;
+		CCVector3 diag = bbMax - bbMin;
 		if (std::abs(diag.x) + std::abs(diag.y) + std::abs(diag.z) < ZERO_TOLERANCE)
 		{
-			trans.T = Gx - Gp*aPrioriScale;
+			trans.T = Gx - Gp * aPrioriScale;
 			return true;
 		}
 
 		//Cross covariance matrix, eq #24 in Besl92 (but with weights, if any)
 		SquareMatrixd Sigma_px = (coupleWeights ? GeometricalAnalysisTools::ComputeWeightedCrossCovarianceMatrix(P, X, Gp, Gx, coupleWeights)
-												: GeometricalAnalysisTools::ComputeCrossCovarianceMatrix(P,X,Gp,Gx) );
+			: GeometricalAnalysisTools::ComputeCrossCovarianceMatrix(P, X, Gp, Gx));
 		if (!Sigma_px.isValid())
 			return false;
 
@@ -1092,7 +1095,7 @@ bool RegistrationTools::RegistrationProcedure(	GenericCloud* P, //data
 
 			unsigned count = X->size();
 			assert(P->size() == count);
-			for (unsigned i=0; i<count; ++i)
+			for (unsigned i = 0; i < count; ++i)
 			{
 				//'a' refers to the data 'A' (moving) = P
 				//'b' refers to the model 'B' (not moving) = X
@@ -1141,13 +1144,15 @@ bool FPCSRegistrationTools::RegisterClouds(	GenericIndexedCloud* modelCloud,
 	//Initialize random seed with current time
 	srand(static_cast<unsigned>(time(nullptr)));
 
-	unsigned bestScore = 0, score = 0;
+	unsigned bestScore = 0;
+	unsigned score = 0;
 	transform.R.invalidate();
 	transform.T = CCVector3(0,0,0);
 
 	//Adapt overlap to the model cloud size
 	{
-		CCVector3 bbMin, bbMax;
+		CCVector3 bbMin;
+		CCVector3 bbMax;
 		modelCloud->getBoundingBox(bbMin, bbMax);
 		CCVector3 diff = bbMax - bbMin;
 		overlap *= diff.norm() / 2;
@@ -1296,11 +1301,28 @@ bool FPCSRegistrationTools::FindBase(	GenericIndexedCloud* cloud,
 										unsigned nbTries,
 										Base &base)
 {
-	unsigned a, b, c, d;
-	unsigned i, size;
-	PointCoordinateType f, best, d0, d1, d2, x, y, z, w;
-	const CCVector3 *p0, *p1, *p2, *p3;
-	CCVector3 normal, u, v;
+	unsigned a;
+	unsigned b;
+	unsigned c;
+	unsigned d;
+	unsigned i;
+	unsigned size;
+	PointCoordinateType f;
+	PointCoordinateType best;
+	PointCoordinateType d0;
+	PointCoordinateType d1;
+	PointCoordinateType d2;
+	PointCoordinateType x;
+	PointCoordinateType y;
+	PointCoordinateType z;
+	PointCoordinateType w;
+	const CCVector3 *p0 = nullptr;
+	const CCVector3 *p1 = nullptr;
+	const CCVector3 *p2 = nullptr;
+	const CCVector3 *p3 = nullptr;
+	CCVector3 normal;
+	CCVector3 u;
+	CCVector3 v;
 
 	overlap *= overlap;
 	size = cloud->size();
@@ -1425,7 +1447,10 @@ int FPCSRegistrationTools::FindCongruentBases(KDTree* tree,
 												std::vector<Base>& results)
 {
 	//Compute reference base invariants (r1, r2)
-	PointCoordinateType r1, r2, d1, d2;
+	PointCoordinateType r1;
+	PointCoordinateType r2;
+	PointCoordinateType d1;
+	PointCoordinateType d2;
 	{
 		const CCVector3* p0 = base[0];
 		const CCVector3* p1 = base[1];
@@ -1443,7 +1468,8 @@ int FPCSRegistrationTools::FindCongruentBases(KDTree* tree,
 	GenericIndexedCloud* cloud = tree->getAssociatedCloud();
 
 	//Find all pairs which are d1-appart and d2-appart
-	std::vector<IndexPair> pairs1, pairs2;
+	std::vector<IndexPair> pairs1;
+	std::vector<IndexPair> pairs2;
 	{
 		unsigned count = static_cast<unsigned>(cloud->size());
 		std::vector<unsigned> pointsIndexes;
@@ -1495,7 +1521,8 @@ int FPCSRegistrationTools::FindCongruentBases(KDTree* tree,
 	//Select among the pairs the ones that can be congruent to the base "base"
 	std::vector<IndexPair> match;
 	{
-		PointCloud tmpCloud1, tmpCloud2;
+		PointCloud tmpCloud1;
+		PointCloud tmpCloud2;
 		{
 			unsigned count = static_cast<unsigned>(pairs1.size());
 			if (!tmpCloud1.reserve(count * 2)) //not enough memory
@@ -1613,8 +1640,13 @@ bool FPCSRegistrationTools::LinesIntersections(	const CCVector3 &p0,
 												PointCoordinateType& lambda,
 												PointCoordinateType& mu)
 {
-	CCVector3 p02, p32, p10, A, B;
-	PointCoordinateType num, denom;
+	CCVector3 p02;
+	CCVector3 p32;
+	CCVector3 p10;
+	CCVector3 A;
+	CCVector3 B;
+	PointCoordinateType num;
+	PointCoordinateType denom;
 
 	//Find lambda and mu such that :
 	//A = p0+lambda(p1-p0)
@@ -1654,11 +1686,13 @@ bool FPCSRegistrationTools::FilterCandidates(	GenericIndexedCloud *modelCloud,
 												std::vector<ScaledTransformation>& transforms)
 {
 	std::vector<Base> table;
-	std::vector<float> scores, sortedscores;
+	std::vector<float> scores;
+	std::vector<float> sortedscores;
 	const CCVector3* p[4];
 	ScaledTransformation t;
 	std::vector<ScaledTransformation> tarray;
-	PointCloud referenceBaseCloud, dataBaseCloud;
+	PointCloud referenceBaseCloud;
+	PointCloud dataBaseCloud;
 
 	unsigned candidatesCount = static_cast<unsigned>(candidates.size());
 	if (candidatesCount == 0)
