@@ -40,6 +40,11 @@ void ScalarFieldTools::SetScalarValueToZero(const CCVector3& P, ScalarType& scal
 	scalarValue = 0;
 }
 
+void ScalarFieldTools::SetScalarValueInverted(const CCVector3& P, ScalarType& scalarValue)
+{
+	scalarValue = -scalarValue;
+}
+
 int ScalarFieldTools::computeScalarFieldGradient(	GenericIndexedCloudPersist* theCloud,
 													PointCoordinateType radius,
 													bool euclideanDistances,
@@ -249,7 +254,7 @@ bool ScalarFieldTools::applyScalarFieldGaussianFilter(PointCoordinateType sigma,
         return false;
 
 	unsigned n = theCloud->size();
-	if (n==0)
+	if (n == 0)
         return false;
 
 	DgmOctree* theOctree = nullptr;
@@ -258,7 +263,7 @@ bool ScalarFieldTools::applyScalarFieldGaussianFilter(PointCoordinateType sigma,
 	else
 	{
 		theOctree = new DgmOctree(theCloud);
-		if (theOctree->build(progressCb)<1)
+		if (theOctree->build(progressCb) < 1)
 		{
 			delete theOctree;
 			return false;
@@ -269,7 +274,12 @@ bool ScalarFieldTools::applyScalarFieldGaussianFilter(PointCoordinateType sigma,
 	unsigned char level = theOctree->findBestLevelForAGivenNeighbourhoodSizeExtraction(3.0f*sigma);
 
 	//output scalar field should be different than input one
-	theCloud->enableScalarField();
+	if (!theCloud->enableScalarField())
+	{
+		if (!theCloudOctree)
+			delete theOctree;
+		return false;
+	}
 
 	if (progressCb)
 	{
@@ -328,9 +338,9 @@ bool ScalarFieldTools::computeCellGaussianFilter(	const DgmOctree::octreeCell& c
 	//structures pour la recherche de voisinages SPECIFIQUES
 	DgmOctree::NearestNeighboursSphericalSearchStruct nNSS;
 	nNSS.level = cell.level;
-	nNSS.prepare(radius,cell.parentOctree->getCellSize(nNSS.level));
-	cell.parentOctree->getCellPos(cell.truncatedCode,cell.level,nNSS.cellPos,true);
-	cell.parentOctree->computeCellCenter(nNSS.cellPos,cell.level,nNSS.cellCenter);
+	nNSS.prepare(radius, cell.parentOctree->getCellSize(nNSS.level));
+	cell.parentOctree->getCellPos(cell.truncatedCode, cell.level, nNSS.cellPos, true);
+	cell.parentOctree->computeCellCenter(nNSS.cellPos, cell.level, nNSS.cellCenter);
 
 	//we already know the points lying in the first cell (this is the one we are treating :)
 	try
@@ -344,7 +354,7 @@ bool ScalarFieldTools::computeCellGaussianFilter(	const DgmOctree::octreeCell& c
 	
 	DgmOctree::NeighboursSet::iterator it = nNSS.pointsInNeighbourhood.begin();
 	{
-		for (unsigned i=0; i<n; ++i,++it)
+		for (unsigned i = 0; i < n; ++i, ++it)
 		{
 			it->point = cell.points->getPointPersistentPtr(i);
 			it->pointIndex = cell.points->getPointGlobalIndex(i);
@@ -357,7 +367,7 @@ bool ScalarFieldTools::computeCellGaussianFilter(	const DgmOctree::octreeCell& c
     //Pure Gaussian Filtering
     if (sigmaSF == -1)
     {
-        for (unsigned i=0; i<n; ++i) //for each point in cell
+		for (unsigned i = 0; i < n; ++i) //for each point in cell
         {
             //we get the points inside a spherical neighbourhood (radius: '3*sigma')
             cell.points->getPoint(i,nNSS.queryPoint);
@@ -368,61 +378,61 @@ bool ScalarFieldTools::computeCellGaussianFilter(	const DgmOctree::octreeCell& c
             it = nNSS.pointsInNeighbourhood.begin();
             double meanValue = 0.0;
             double wSum = 0.0;
-            for (unsigned j=0;j<k;++j,++it)
-            {
-                double weight = exp(-(it->squareDistd)/sigma2); //PDF: -exp(-(x-mu)^2/(2*sigma^2))
-                ScalarType val = cloud->getPointScalarValue(it->pointIndex);
-                //scalar value must be valid
+			for (unsigned j = 0; j < k; ++j, ++it)
+			{
+				double weight = exp(-(it->squareDistd) / sigma2); //PDF: -exp(-(x-mu)^2/(2*sigma^2))
+				ScalarType val = cloud->getPointScalarValue(it->pointIndex);
+				//scalar value must be valid
 				if (ScalarField::ValidValue(val))
-                {
-                    meanValue += static_cast<double>(val) * weight;
-                    wSum += weight;
-                }
-            }
+				{
+					meanValue += static_cast<double>(val) * weight;
+					wSum += weight;
+				}
+			}
 
 			ScalarType newValue = (wSum > 0.0 ? static_cast<ScalarType>(meanValue / wSum) : NAN_VALUE);
 
-            cell.points->setPointScalarValue(i,newValue);
+			cell.points->setPointScalarValue(i, newValue);
 
 			if (nProgress && !nProgress->oneStep())
 				return false;
-        }
+		}
     }
-    //Bilateral Filtering using the second sigma parameters on values (when given)
-    else
-    {
-        for (unsigned i=0;i<n;++i) //for each point in cell
-        {
-            ScalarType queryValue = cell.points->getPointScalarValue(i); //scalar of the query point
+	//Bilateral Filtering using the second sigma parameters on values (when given)
+	else
+	{
+		for (unsigned i = 0; i < n; ++i) //for each point in cell
+		{
+			ScalarType queryValue = cell.points->getPointScalarValue(i); //scalar of the query point
 
-            //we get the points inside a spherical neighbourhood (radius: '3*sigma')
-            cell.points->getPoint(i,nNSS.queryPoint);
+			//we get the points inside a spherical neighbourhood (radius: '3*sigma')
+			cell.points->getPoint(i, nNSS.queryPoint);
 			//warning: there may be more points at the end of nNSS.pointsInNeighbourhood than the actual nearest neighbors (k)!
-            unsigned k = cell.parentOctree->findNeighborsInASphereStartingFromCell(nNSS,radius,false);
+			unsigned k = cell.parentOctree->findNeighborsInASphereStartingFromCell(nNSS, radius, false);
 
-            //each point adds a contribution weighted by its distance to the sphere center
-            it = nNSS.pointsInNeighbourhood.begin();
-            double meanValue = 0.0;
-            double wSum = 0.0;
-            for (unsigned j=0;j<k;++j,++it)
-            {
-                ScalarType val = cloud->getPointScalarValue(it->pointIndex);
+			//each point adds a contribution weighted by its distance to the sphere center
+			it = nNSS.pointsInNeighbourhood.begin();
+			double meanValue = 0.0;
+			double wSum = 0.0;
+			for (unsigned j = 0; j < k; ++j, ++it)
+			{
+				ScalarType val = cloud->getPointScalarValue(it->pointIndex);
 				ScalarType dSF = queryValue - val;
-                double weight = exp(-(it->squareDistd)/sigma2) * exp(-(dSF*dSF)/sigmaSF2);
-                //scalar value must be valid
+				double weight = exp(-(it->squareDistd) / sigma2) * exp(-(dSF*dSF) / sigmaSF2);
+				//scalar value must be valid
 				if (ScalarField::ValidValue(val))
-                {
-                    meanValue += static_cast<double>(val) * weight;
-                    wSum += weight;
-                }
-            }
+				{
+					meanValue += static_cast<double>(val) * weight;
+					wSum += weight;
+				}
+			}
 
-            cell.points->setPointScalarValue(i,wSum > 0.0 ? static_cast<ScalarType>(meanValue / wSum) : NAN_VALUE);
+			cell.points->setPointScalarValue(i, wSum > 0.0 ? static_cast<ScalarType>(meanValue / wSum) : NAN_VALUE);
 
 			if (nProgress && !nProgress->oneStep())
 				return false;
-        }
-    }
+		}
+	}
 
 	return true;
 }
@@ -529,7 +539,8 @@ void ScalarFieldTools::computeScalarFieldHistogram(const GenericCloud* theCloud,
 	}
 
 	//compute the min and max sf values
-	ScalarType minV,maxV;
+	ScalarType minV;
+	ScalarType maxV;
 	{
 		computeScalarFieldExtremas(theCloud, minV, maxV);
 
@@ -601,7 +612,8 @@ bool ScalarFieldTools::computeKmeans(	const GenericCloud* theCloud,
 	}
 
 	//compute min and max SF values
-	ScalarType minV,maxV;
+	ScalarType minV;
+	ScalarType maxV;
 	{
 		computeScalarFieldExtremas(theCloud, minV, maxV);
 		
@@ -709,7 +721,8 @@ bool ScalarFieldTools::computeKmeans(	const GenericCloud* theCloud,
 	} while (meansHaveMoved);
 
 	//update distances
-	std::vector<ScalarType> mins, maxs;
+	std::vector<ScalarType> mins;
+	std::vector<ScalarType> maxs;
 	try
 	{
 		mins.resize(K, maxV);
