@@ -15,6 +15,8 @@
 //#                                                                        #
 //##########################################################################
 
+#include <clocale>
+
 //Qt
 #include <QDir>
 #include <QStandardPaths>
@@ -43,51 +45,54 @@
 #endif
 
 
-void ccApplicationBase::init(bool noOpenGLSupport)
+void ccApplicationBase::initOpenGL()
 {
-	if (!noOpenGLSupport)
+	//See http://doc.qt.io/qt-5/qopenglwidget.html#opengl-function-calls-headers-and-qopenglfunctions
+	/** Calling QSurfaceFormat::setDefaultFormat() before constructing the QApplication instance is mandatory
+		on some platforms (for example, OS X) when an OpenGL core profile context is requested. This is to
+		ensure that resource sharing between contexts stays functional as all internal contexts are created
+		using the correct version and profile.
+	**/
 	{
-		//See http://doc.qt.io/qt-5/qopenglwidget.html#opengl-function-calls-headers-and-qopenglfunctions
-		/** Calling QSurfaceFormat::setDefaultFormat() before constructing the QApplication instance is mandatory
-			on some platforms (for example, OS X) when an OpenGL core profile context is requested. This is to
-			ensure that resource sharing between contexts stays functional as all internal contexts are created
-			using the correct version and profile.
-		**/
-		{
-			QSurfaceFormat format = QSurfaceFormat::defaultFormat();
+		QSurfaceFormat format = QSurfaceFormat::defaultFormat();
 
-			format.setSwapBehavior(QSurfaceFormat::DoubleBuffer);
-			format.setStencilBufferSize(0);
+		format.setSwapBehavior(QSurfaceFormat::DoubleBuffer);
+		format.setStencilBufferSize(0);
 
 #ifdef CC_GL_WINDOW_USE_QWINDOW
-			format.setStereo(true);
+		format.setStereo(true);
 #endif
 
 #ifdef Q_OS_MAC
-			format.setVersion(2, 1);	// must be 2.1 - see ccGLWindow::functions()
-			format.setProfile(QSurfaceFormat::CoreProfile);
+		format.setVersion(2, 1);	// must be 2.1 - see ccGLWindow::functions()
+		format.setProfile(QSurfaceFormat::CoreProfile);
 #endif
 
 #ifdef QT_DEBUG
-			format.setOption(QSurfaceFormat::DebugContext, true);
+		format.setOption(QSurfaceFormat::DebugContext, true);
 #endif
 
-			QSurfaceFormat::setDefaultFormat(format);
-		}
-
-		// The 'AA_ShareOpenGLContexts' attribute must be defined BEFORE the creation of the Q(Gui)Application
-		// DGM: this is mandatory to enable exclusive full screen for ccGLWidget (at least on Windows)
-		QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
+		QSurfaceFormat::setDefaultFormat(format);
 	}
+
+	// The 'AA_ShareOpenGLContexts' attribute must be defined BEFORE the creation of the Q(Gui)Application
+	// DGM: this is mandatory to enable exclusive full screen for ccGLWidget (at least on Windows)
+	QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
 }
 
-ccApplicationBase::ccApplicationBase(int &argc, char **argv, const QString &version)
+ccApplicationBase::ccApplicationBase(int &argc, char **argv, bool isCommandLine, const QString &version)
     : QApplication( argc, argv )
     , c_VersionStr( version )
+	, c_CommandLine( isCommandLine )
 {
 	setOrganizationName( "CCCorp" );
 
 	setupPaths();
+
+#ifdef Q_OS_WIN
+	//enables automatic scaling based on the monitor's pixel density
+	setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
 		
 #ifdef Q_OS_MAC
 	// Mac OS X apps don't show icons in menus
@@ -104,7 +109,7 @@ ccApplicationBase::ccApplicationBase(int &argc, char **argv, const QString &vers
 #endif
 	
 	ccGLWindow::setShaderPath( m_ShaderPath );
-	ccPluginManager::setPaths( m_PluginPaths );
+	ccPluginManager::get().setPaths( m_PluginPaths );
 	
 	ccTranslationManager::get().registerTranslatorFile( QStringLiteral( "qt" ), m_TranslationPath );
 	ccTranslationManager::get().registerTranslatorFile( QStringLiteral( "CloudCompare" ), m_TranslationPath );
