@@ -298,7 +298,8 @@ ccGBLSensor::NormalGrid* ccGBLSensor::projectNormals(	CCLib::GenericCloud* cloud
 			}
 
 			//project in Z-buffer
-			unsigned x, y;
+			unsigned x = 0;
+			unsigned y = 0;
 			if (convertToDepthMapCoords(Q.x, Q.y, x, y))
 			{
 				//add the transformed normal
@@ -365,7 +366,7 @@ ccGBLSensor::ColorGrid* ccGBLSensor::projectColors(	CCLib::GenericCloud* cloud,
 	ColorGrid* colorGrid = new ColorGrid;
 	try
 	{
-		colorGrid->resize(gridSize, ccColor::black);
+		colorGrid->resize(gridSize, ccColor::blackRGB);
 	}
 	catch (const std::bad_alloc&)
 	{
@@ -385,7 +386,8 @@ ccGBLSensor::ColorGrid* ccGBLSensor::projectColors(	CCLib::GenericCloud* cloud,
 				PointCoordinateType depth;
 				projectPoint(*P, Q, depth, m_activeIndex);
 
-				unsigned x, y;
+				unsigned x = 0;
+				unsigned y = 0;
 				if (convertToDepthMapCoords(Q.x, Q.y, x, y))
 				{
 					unsigned index = y*m_depthBuffer.width + x;
@@ -440,7 +442,9 @@ struct Interval
 	template<class T> static Interval FindBiggest(const std::vector<T>& values, T intValue, bool allowLoop = true)
 	{
 		//look for the largest 'empty' part
-		Interval firstEmptyPart, bestEmptyPart, currentEmptyPart;
+		Interval firstEmptyPart;
+		Interval bestEmptyPart;
+		Interval currentEmptyPart;
 
 		for (size_t i = 0; i < values.size(); ++i)
 		{
@@ -498,7 +502,8 @@ bool ccGBLSensor::computeAutoParameters(CCLib::GenericCloud* theCloud)
 		return false;
 	}
 
-	std::vector<bool> nonEmptyAnglesYaw, nonEmptyAnglesPitch;
+	std::vector<bool> nonEmptyAnglesYaw;
+	std::vector<bool> nonEmptyAnglesPitch;
 	try
 	{
 		nonEmptyAnglesYaw.resize(360, false);
@@ -515,7 +520,10 @@ bool ccGBLSensor::computeAutoParameters(CCLib::GenericCloud* theCloud)
 
 	unsigned pointCount = theCloud->size();
 
-	PointCoordinateType minPitch = 0, maxPitch = 0, minYaw = 0, maxYaw = 0;
+	PointCoordinateType minPitch = 0;
+	PointCoordinateType maxPitch = 0;
+	PointCoordinateType minYaw = 0;
+	PointCoordinateType maxYaw = 0;
 	PointCoordinateType maxDepth = 0;
 	{
 		//first project all points to compute the (yaw,ptich) ranges
@@ -705,7 +713,8 @@ bool ccGBLSensor::computeDepthBuffer(CCLib::GenericCloud* theCloud, int& errorCo
 				PointCoordinateType depth;
 				projectPoint(*P, Q, depth, m_activeIndex);
 
-				unsigned x, y;
+				unsigned x = 0;
+				unsigned y = 0;
 				if (convertToDepthMapCoords(Q.x, Q.y, x, y))
 				{
 					PointCoordinateType& zBuf = m_depthBuffer.zBuff[y*m_depthBuffer.width + x];
@@ -754,7 +763,8 @@ unsigned char ccGBLSensor::checkVisibility(const CCVector3& P) const
 		return POINT_OUT_OF_RANGE;
 	}
 
-	unsigned x, y;
+	unsigned x = 0;
+	unsigned y = 0;
 	if (!convertToDepthMapCoords(Q.x, Q.y, x, y))
 	{
 		//out of field of view
@@ -837,18 +847,18 @@ void ccGBLSensor::drawMeOnly(CC_DRAW_CONTEXT& context)
 	//sensor axes
 	{
 		PointCoordinateType axisLength = halfHeadSize * m_scale;
-		ccGL::Color3v(glFunc, ccColor::red.rgb);
+		ccGL::Color4v(glFunc, ccColor::red.rgba);
 		CCVector3 C(0, 0, 0);
 		glFunc->glBegin(GL_LINES);
 		ccGL::Vertex3v(glFunc, C.u);
 		ccGL::Vertex3(glFunc, C.x + axisLength, C.y, C.z);
 		glFunc->glEnd();
-		ccGL::Color3v(glFunc, ccColor::green.rgb);
+		ccGL::Color4v(glFunc, ccColor::green.rgba);
 		glFunc->glBegin(GL_LINES);
 		ccGL::Vertex3v(glFunc, C.u);
 		ccGL::Vertex3(glFunc, C.x, C.y + axisLength, C.z);
 		glFunc->glEnd();
-		ccGL::Color3v(glFunc, ccColor::blue.rgb);
+		ccGL::Color4v(glFunc, ccColor::blue.rgba);
 		glFunc->glBegin(GL_LINES);
 		ccGL::Vertex3v(glFunc, C.u);
 		ccGL::Vertex3(glFunc, C.x, C.y, C.z + axisLength);
@@ -1030,9 +1040,9 @@ bool ccGBLSensor::toFile_MeOnly(QFile& out) const
 	return true;
 }
 
-bool ccGBLSensor::fromFile_MeOnly(QFile& in, short dataVersion, int flags)
+bool ccGBLSensor::fromFile_MeOnly(QFile& in, short dataVersion, int flags, LoadedIDMap& oldToNewIDMap)
 {
-	if (!ccSensor::fromFile_MeOnly(in, dataVersion, flags))
+	if (!ccSensor::fromFile_MeOnly(in, dataVersion, flags, oldToNewIDMap))
 		return false;
 
 	//rotation order (dataVersion>=34)
@@ -1051,8 +1061,12 @@ bool ccGBLSensor::fromFile_MeOnly(QFile& in, short dataVersion, int flags)
 	ccSerializationHelper::CoordsFromDataStream(inStream, flags, &m_deltaTheta, 1);
 	if (dataVersion < 38)
 	{
-		ccSerializationHelper::ScalarsFromDataStream(inStream, flags, &m_sensorRange, 1);
-		ccSerializationHelper::ScalarsFromDataStream(inStream, flags, &m_uncertainty, 1);
+		ScalarType sensorRange{};
+		ScalarType uncertainty{};
+		ccSerializationHelper::ScalarsFromDataStream(inStream, flags, &sensorRange, 1);
+		ccSerializationHelper::ScalarsFromDataStream(inStream, flags, &uncertainty, 1);
+		m_sensorRange = static_cast<PointCoordinateType>(sensorRange);
+		m_uncertainty = static_cast<PointCoordinateType>(uncertainty);
 	}
 	else
 	{
