@@ -41,9 +41,9 @@
 //system
 #include <cassert>
 
-ccGenericMesh::ccGenericMesh(QString name/*=QString()*/)
+ccGenericMesh::ccGenericMesh(QString name/*=QString()*/, unsigned uniqueID/*=ccUniqueIDGenerator::InvalidUniqueID*/)
 	: GenericIndexedMesh()
-	, ccHObject(name)
+	, ccHObject(name, uniqueID)
 	, m_triNormsShown(false)
 	, m_materialsShown(false)
 	, m_showWired(false)
@@ -130,9 +130,9 @@ CCVector3* ccGenericMesh::GetNormalsBuffer()
 }
 
 //Colors buffer
-ccColor::Rgb* ccGenericMesh::GetColorsBuffer()
+ColorCompType* ccGenericMesh::GetColorsBuffer()
 {
-	static ccColor::Rgb s_rgbBuffer[ccChunk::SIZE * 3];
+	static ColorCompType s_rgbBuffer[ccChunk::SIZE * 3 * 4];
 	return s_rgbBuffer;
 }
 
@@ -295,18 +295,18 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 		}
 
 		//in the case we need to display vertex colors
-		ColorsTableType* rgbColorsTable = nullptr;
+		RGBAColorsTableType* rgbaColorsTable = nullptr;
 		if (glParams.showColors)
 		{
 			if (isColorOverriden())
 			{
-				ccGL::Color3v(glFunc, m_tempColor.rgb);
+				ccGL::Color4v(glFunc, m_tempColor.rgba);
 				glParams.showColors = false;
 			}
 			else
 			{
 				assert(vertices->isA(CC_TYPES::POINT_CLOUD));
-				rgbColorsTable = static_cast<ccPointCloud*>(vertices)->rgbColors();
+				rgbaColorsTable = static_cast<ccPointCloud*>(vertices)->rgbaColors();
 			}
 		}
 		else
@@ -350,11 +350,16 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 				glFunc->glEnableClientState(GL_NORMAL_ARRAY);
 				glFunc->glNormalPointer(GL_COORD_TYPE, 0, GetNormalsBuffer());
 			}
-			if (glParams.showSF || glParams.showColors)
+			if (glParams.showSF)
 			{
 				glFunc->glEnableClientState(GL_COLOR_ARRAY);
 				glFunc->glColorPointer(3, GL_UNSIGNED_BYTE, 0, GetColorsBuffer());
 			}
+			else if (glParams.showColors)
+			{
+				glFunc->glEnableClientState(GL_COLOR_ARRAY);
+				glFunc->glColorPointer(4, GL_UNSIGNED_BYTE, 0, GetColorsBuffer());
+			}
 
 			//we can scan and process each chunk separately in an optimized way
 			//we mimic the way ccMesh beahves by using virtual chunks!
@@ -378,7 +383,7 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 				//scalar field
 				if (glParams.showSF)
 				{
-					ccColor::Rgb* _rgbColors = GetColorsBuffer();
+					ccColor::Rgb* _rgbColors = reinterpret_cast<ccColor::Rgb*>(GetColorsBuffer());
 					assert(colorScale);
 					for (unsigned n = 0; n < chunkSize; n += decimStep)
 					{
@@ -391,14 +396,14 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 				//colors
 				else if (glParams.showColors)
 				{
-					ccColor::Rgb* _rgbColors = GetColorsBuffer();
+					ccColor::Rgba* _rgbaColors = reinterpret_cast<ccColor::Rgba*>(GetColorsBuffer());
 
 					for (unsigned n = 0; n < chunkSize; n += decimStep)
 					{
 						const CCLib::VerticesIndexes* ti = getTriangleVertIndexes(static_cast<unsigned>(chunkStart + n));
-						*_rgbColors++ = rgbColorsTable->at(ti->i1);
-						*_rgbColors++ = rgbColorsTable->at(ti->i2);
-						*_rgbColors++ = rgbColorsTable->at(ti->i3);
+						*_rgbaColors++ = rgbaColorsTable->at(ti->i1);
+						*_rgbaColors++ = rgbaColorsTable->at(ti->i2);
+						*_rgbaColors++ = rgbaColorsTable->at(ti->i3);
 					}
 				}
 
@@ -410,7 +415,9 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 					{
 						for (unsigned n = 0; n < chunkSize; n += decimStep)
 						{
-							CCVector3 Na, Nb, Nc;
+							CCVector3 Na;
+							CCVector3 Nb;
+							CCVector3 Nc;
 							getTriangleNormals(static_cast<unsigned>(chunkStart + n), Na, Nb, Nc);
 							*_normals++ = Na;
 							*_normals++ = Nb;
@@ -448,12 +455,22 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 		}
 		else
 		{
-			//current vertex color
-			const ccColor::Rgb *col1 = nullptr, *col2 = nullptr, *col3 = nullptr;
+			//current vertex color (RGB)
+			const ccColor::Rgb *rgb1 = nullptr;
+			const ccColor::Rgb *rgb2 = nullptr;
+			const ccColor::Rgb *rgb3 = nullptr;
+			//current vertex color (RGBA)
+			const ccColor::Rgba *rgba1 = nullptr;
+			const ccColor::Rgba *rgba2 = nullptr;
+			const ccColor::Rgba *rgba3 = nullptr;
 			//current vertex normal
-			const PointCoordinateType *N1 = nullptr, *N2 = nullptr, *N3 = nullptr;
+			const PointCoordinateType *N1 = nullptr;
+			const PointCoordinateType *N2 = nullptr;
+			const PointCoordinateType *N3 = nullptr;
 			//current vertex texture coordinates
-			TexCoords2D *Tx1 = nullptr, *Tx2 = nullptr, *Tx3 = nullptr;
+			TexCoords2D *Tx1 = nullptr;
+			TexCoords2D *Tx2 = nullptr;
+			TexCoords2D *Tx3 = nullptr;
 
 			//loop on all triangles
 			int lasMtlIndex = -1;
@@ -495,21 +512,21 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 				if (glParams.showSF)
 				{
 					assert(colorScale);
-					col1 = currentDisplayedScalarField->getValueColor(tsi->i1);
-					if (!col1)
+					rgb1 = currentDisplayedScalarField->getValueColor(tsi->i1);
+					if (!rgb1)
 						continue;
-					col2 = currentDisplayedScalarField->getValueColor(tsi->i2);
-					if (!col2)
+					rgb2 = currentDisplayedScalarField->getValueColor(tsi->i2);
+					if (!rgb2)
 						continue;
-					col3 = currentDisplayedScalarField->getValueColor(tsi->i3);
-					if (!col3)
+					rgb3 = currentDisplayedScalarField->getValueColor(tsi->i3);
+					if (!rgb3)
 						continue;
 				}
 				else if (glParams.showColors)
 				{
-					col1 = &rgbColorsTable->at(tsi->i1);
-					col2 = &rgbColorsTable->at(tsi->i2);
-					col3 = &rgbColorsTable->at(tsi->i3);
+					rgba1 = &rgbaColorsTable->at(tsi->i1);
+					rgba2 = &rgbaColorsTable->at(tsi->i2);
+					rgba3 = &rgbaColorsTable->at(tsi->i3);
 				}
 
 				if (glParams.showNorms)
@@ -517,7 +534,9 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 					if (showTriNormals)
 					{
 						assert(triNormals);
-						int n1, n2, n3;
+						int n1 = 0;
+						int n2 = 0;
+						int n3 = 0;
 						getTriangleNormalIndexes(n, n1, n2, n3);
 						N1 = (n1 >= 0 ?                 ccNormalVectors::GetNormal(triNormals->at(n1)).u : nullptr);
 						N2 = (n1 == n2 ? N1 : n1 >= 0 ? ccNormalVectors::GetNormal(triNormals->at(n2)).u : nullptr);
@@ -584,8 +603,10 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 				//vertex 1
 				if (N1)
 					ccGL::Normal3v(glFunc, N1);
-				if (col1)
-					glFunc->glColor3ubv(col1->rgb);
+				if (rgb1)
+					glFunc->glColor3ubv(rgb1->rgb);
+				else if (rgba1)
+					glFunc->glColor4ubv(rgba1->rgba);
 				if (Tx1)
 					glFunc->glTexCoord2fv(Tx1->t);
 				ccGL::Vertex3v(glFunc, vertices->getPoint(tsi->i1)->u);
@@ -593,8 +614,10 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 				//vertex 2
 				if (N2)
 					ccGL::Normal3v(glFunc, N2);
-				if (col2)
-					glFunc->glColor3ubv(col2->rgb);
+				if (rgb2)
+					glFunc->glColor3ubv(rgb2->rgb);
+				else if (rgba2)
+					glFunc->glColor4ubv(rgba2->rgba);
 				if (Tx2)
 					glFunc->glTexCoord2fv(Tx2->t);
 				ccGL::Vertex3v(glFunc, vertices->getPoint(tsi->i2)->u);
@@ -602,8 +625,10 @@ void ccGenericMesh::drawMeOnly(CC_DRAW_CONTEXT& context)
 				//vertex 3
 				if (N3)
 					ccGL::Normal3v(glFunc, N3);
-				if (col3)
-					glFunc->glColor3ubv(col3->rgb);
+				if (rgb3)
+					glFunc->glColor3ubv(rgb3->rgb);
+				else if (rgba3)
+					glFunc->glColor4ubv(rgba3->rgba);
 				if (Tx3)
 					glFunc->glTexCoord2fv(Tx3->t);
 				ccGL::Vertex3v(glFunc, vertices->getPoint(tsi->i3)->u);
@@ -669,9 +694,9 @@ bool ccGenericMesh::toFile_MeOnly(QFile& out) const
 	return true;
 }
 
-bool ccGenericMesh::fromFile_MeOnly(QFile& in, short dataVersion, int flags)
+bool ccGenericMesh::fromFile_MeOnly(QFile& in, short dataVersion, int flags, LoadedIDMap& oldToNewIDMap)
 {
-	if (!ccHObject::fromFile_MeOnly(in, dataVersion, flags))
+	if (!ccHObject::fromFile_MeOnly(in, dataVersion, flags, oldToNewIDMap))
 		return false;
 
 	//'show wired' state (dataVersion>=20)
@@ -793,9 +818,9 @@ ccPointCloud* ccGenericMesh::samplePoints(	bool densityBased,
 					unsigned triIndex = triIndices->at(i);
 					const CCVector3* P = cloud->getPoint(i);
 
-					ccColor::Rgb C;
-					getColorFromMaterial(triIndex, *P, C, withRGB);
-					cloud->addRGBColor(C);
+					ccColor::Rgba color;
+					getColorFromMaterial(triIndex, *P, color, withRGB);
+					cloud->addColor(color);
 				}
 
 				cloud->showColors(true);
@@ -816,7 +841,7 @@ ccPointCloud* ccGenericMesh::samplePoints(	bool densityBased,
 
 					ccColor::Rgb C;
 					interpolateColors(triIndex, *P, C);
-					cloud->addRGBColor(C);
+					cloud->addColor(C);
 				}
 
 				cloud->showColors(true);
@@ -885,11 +910,108 @@ void ccGenericMesh::computeInterpolationWeights(unsigned triIndex, const CCVecto
 	weights /= sum;
 }
 
+bool ccGenericMesh::trianglePicking(unsigned triIndex,
+									const CCVector2d& clickPos,
+									const ccGLMatrix& trans,
+									bool noGLTrans,
+									const ccGenericPointCloud& vertices,
+									const ccGLCameraParameters& camera,
+									CCVector3d& point,
+									CCVector3d* barycentricCoords/*=nullptr*/,
+									QPainter* painter/*=nullptr*/) const
+{
+	assert(triIndex < size());
+
+	CCVector3 A3D;
+	CCVector3 B3D;
+	CCVector3 C3D;
+	getTriangleVertices(triIndex, A3D, B3D, C3D);
+
+	CCVector3d A2D;
+	CCVector3d B2D;
+	CCVector3d C2D;
+	bool insideA = false;
+	bool insideB = false;
+	bool insideC = false;
+
+	if (noGLTrans)
+	{
+		camera.project(A3D, A2D, &insideA);
+		camera.project(B3D, B2D, &insideB);
+		camera.project(C3D, C2D, &insideC);
+	}
+	else
+	{
+		CCVector3 A3Dp = trans * A3D;
+		CCVector3 B3Dp = trans * B3D;
+		CCVector3 C3Dp = trans * C3D;
+		camera.project(A3Dp, A2D, &insideA);
+		camera.project(B3Dp, B2D, &insideB);
+		camera.project(C3Dp, C2D, &insideC);
+	}
+
+	// if none of the vertices fall inside the frustum, the triangle is (probably) not visible...
+	// FIXME: if there's one huge triangle or the user zoom in a lot, if's not true!
+	if (!insideA && !insideB && !insideC)
+	{
+		return false;
+	}
+
+	if (painter)
+	{
+		//for debug purpose
+		painter->drawLine(QPointF(A2D.x, A2D.y), QPointF(B2D.x, B2D.y));
+		painter->drawLine(QPointF(B2D.x, B2D.y), QPointF(C2D.x, C2D.y));
+		painter->drawLine(QPointF(C2D.x, C2D.y), QPointF(A2D.x, A2D.y));
+	}
+
+	//barycentric coordinates
+	GLdouble detT = (B2D.y - C2D.y) * (A2D.x - C2D.x) + (C2D.x - B2D.x) * (A2D.y - C2D.y);
+	if (std::abs(detT) < ZERO_TOLERANCE)
+	{
+		return false;
+	}
+	GLdouble l1 = ((B2D.y - C2D.y) * (clickPos.x - C2D.x) + (C2D.x - B2D.x) * (clickPos.y - C2D.y)) / detT;
+	GLdouble l2 = ((C2D.y - A2D.y) * (clickPos.x - C2D.x) + (A2D.x - C2D.x) * (clickPos.y - C2D.y)) / detT;
+
+	//does the point falls inside the triangle?
+	if (l1 >= 0 && l1 <= 1.0 && l2 >= 0.0 && l2 <= 1.0)
+	{
+		double l1l2 = l1 + l2;
+		assert(l1l2 >= -1.0e-12);
+		if (l1l2 > 1.0)
+		{
+			//we fall outside of the triangle!
+			return false;
+		}
+
+		GLdouble l3 = 1.0 - l1 - l2;
+		assert(l3 >= -1.0e-12);
+
+		//now deduce the 3D position
+		point = CCVector3d(	l1 * A3D.x + l2 * B3D.x + l3 * C3D.x,
+							l1 * A3D.y + l2 * B3D.y + l3 * C3D.y,
+							l1 * A3D.z + l2 * B3D.z + l3 * C3D.z);
+
+		if (barycentricCoords)
+		{
+			*barycentricCoords = CCVector3d(l1, l2, l3);
+		}
+
+		return true;
+	}
+	else
+	{
+		return false;
+	}
+}
+
 bool ccGenericMesh::trianglePicking(const CCVector2d& clickPos,
 									const ccGLCameraParameters& camera,
 									int& nearestTriIndex,
 									double& nearestSquareDist,
-									CCVector3d& nearestPoint)
+									CCVector3d& nearestPoint,
+									CCVector3d* barycentricCoords/*=nullptr*/) const
 {
 	ccGLMatrix trans;
 	bool noGLTrans = !getAbsoluteGLTransformation(trans);
@@ -905,6 +1027,8 @@ bool ccGenericMesh::trianglePicking(const CCVector2d& clickPos,
 	nearestTriIndex = -1;
 	nearestSquareDist = -1.0;
 	nearestPoint = CCVector3d(0, 0, 0);
+	if (barycentricCoords)
+		*barycentricCoords = CCVector3d(0, 0, 0);
 
 	ccGenericPointCloud* vertices = getAssociatedCloud();
 	if (!vertices)
@@ -913,68 +1037,108 @@ bool ccGenericMesh::trianglePicking(const CCVector2d& clickPos,
 		return false;
 	}
 
-#if defined(_OPENMP)
+//#define TEST_PICKING
+#ifdef TEST_PICKING
+	QImage testImage(camera.viewport[2], camera.viewport[3], QImage::Format::Format_ARGB32);
+	testImage.fill(Qt::white);
+	QPainter painter;
+	painter.begin(&testImage);
+	painter.setBrush(Qt::red);
+	painter.setPen(Qt::NoPen);
+	static const double brushRadius_pix = 2.5;
+	painter.drawEllipse(QPointF(clickPosd.x - brushRadius_pix, clickPosd.y - brushRadius_pix), 2 * brushRadius_pix, 2 * brushRadius_pix);
+	QPen pen(Qt::black);
+	pen.setWidth(3);
+	painter.setPen(pen);
+#endif
+
+#if defined(_OPENMP) && !defined(_DEBUG) && !defined(TEST_PICKING)
 	#pragma omp parallel for
 #endif
 	for (int i = 0; i < static_cast<int>(size()); ++i)
 	{
-		CCLib::VerticesIndexes* tsi = getTriangleVertIndexes(i);
-		const CCVector3* A3D = vertices->getPoint(tsi->i1);
-		const CCVector3* B3D = vertices->getPoint(tsi->i2);
-		const CCVector3* C3D = vertices->getPoint(tsi->i3);
+		CCVector3d P;
+		CCVector3d BC;
+		if (!trianglePicking(	i,	
+								clickPos,
+								trans,
+								noGLTrans,
+								*vertices,
+								camera,
+								P,
+								barycentricCoords ? &BC : nullptr
+#ifdef TEST_PICKING
+			, &painter
+#endif
+		))
+			continue;
 
-		CCVector3d A2D, B2D, C2D;
-		if (noGLTrans)
+		double squareDist = (X - P).norm2d();
+		if (nearestTriIndex < 0 || squareDist < nearestSquareDist)
 		{
-			camera.project(*A3D, A2D);
-			camera.project(*B3D, B2D);
-			camera.project(*C3D, C2D);
-		}
-		else
-		{
-			CCVector3 A3Dp = *A3D;
-			CCVector3 B3Dp = *B3D;
-			CCVector3 C3Dp = *C3D;
-			trans.apply(A3Dp);
-			trans.apply(B3Dp);
-			trans.apply(C3Dp);
-			camera.project(A3Dp, A2D);
-			camera.project(B3Dp, B2D);
-			camera.project(C3Dp, C2D);
-		}
-
-		//barycentric coordinates
-		GLdouble detT =  (B2D.y - C2D.y) *      (A2D.x - C2D.x) + (C2D.x - B2D.x) *      (A2D.y - C2D.y);
-		GLdouble l1   = ((B2D.y - C2D.y) * (clickPos.x - C2D.x) + (C2D.x - B2D.x) * (clickPos.y - C2D.y)) / detT;
-		GLdouble l2   = ((C2D.y - A2D.y) * (clickPos.x - C2D.x) + (A2D.x - C2D.x) * (clickPos.y - C2D.y)) / detT;
-
-		//does the point falls inside the triangle?
-		if (l1 >= 0 && l1 <= 1.0 && l2 >= 0.0 && l2 <= 1.0)
-		{
-			double l1l2 = l1+l2;
-			assert(l1l2 >= 0);
-			if (l1l2 > 1.0)
-			{
-				//we fall outside of the triangle!
-				continue;
-			}
-
-			GLdouble l3 = 1.0 - l1 - l2;
-			assert(l3 >= -1.0e-12);
-
-			//now deduce the 3D position
-			CCVector3d P(	l1 * A3D->x + l2 * B3D->x + l3 * C3D->x,
-							l1 * A3D->y + l2 * B3D->y + l3 * C3D->y,
-							l1 * A3D->z + l2 * B3D->z + l3 * C3D->z);
-			double squareDist = (X - P).norm2d();
-			if (nearestTriIndex < 0 || squareDist < nearestSquareDist)
-			{
-				nearestSquareDist = squareDist;
-				nearestTriIndex = static_cast<int>(i);
-				nearestPoint = P;
-			}
+			nearestSquareDist = squareDist;
+			nearestTriIndex = static_cast<int>(i);
+			nearestPoint = P;
+			if (barycentricCoords)
+				*barycentricCoords = BC;
 		}
 	}
 
+#ifdef TEST_PICKING
+	testImage.save("lastPickingSession.png");
+#endif
+
 	return (nearestTriIndex >= 0);
-}
\ No newline at end of file
+}
+
+bool ccGenericMesh::trianglePicking(unsigned triIndex,
+									const CCVector2d& clickPos,
+									const ccGLCameraParameters& camera,
+									CCVector3d& point,
+									CCVector3d* barycentricCoords/*=nullptr*/) const
+{
+	if (triIndex >= size())
+	{
+		assert(false);
+		return false;
+	}
+
+	ccGLMatrix trans;
+	bool noGLTrans = !getAbsoluteGLTransformation(trans);
+
+	ccGenericPointCloud* vertices = getAssociatedCloud();
+	if (!vertices)
+	{
+		assert(false);
+		return false;
+	}
+
+	return trianglePicking(triIndex, clickPos, trans, noGLTrans, *vertices, camera, point, barycentricCoords);
+}
+
+bool ccGenericMesh::computePointPosition(unsigned triIndex, const CCVector2d& uv, CCVector3& P, bool warningIfOutside/*=true*/) const
+{
+	if (triIndex >= size())
+	{
+		assert(false);
+		ccLog::Warning("Index out of range");
+		return true;
+	}
+	
+	CCVector3 A;
+	CCVector3 B;
+	CCVector3 C;
+	getTriangleVertices(triIndex, A, B, C);
+
+	double z = 1.0 - uv.x - uv.y;
+	if (warningIfOutside && ((z < -1.0e-6) || (z > 1.0 + 1.0e-6)))
+	{
+		ccLog::Warning("Point falls outside of the triangle");
+	}
+	
+	P = CCVector3(	static_cast<PointCoordinateType>(uv.x * A.x + uv.y * B.x + z * C.x),
+					static_cast<PointCoordinateType>(uv.x * A.y + uv.y * B.y + z * C.y),
+					static_cast<PointCoordinateType>(uv.x * A.z + uv.y * B.z + z * C.z));
+
+	return true;
+}
