@@ -31,13 +31,14 @@ public: //construction
 
 	//! Default constructor
 	/** \param name object name (optional)
+		\param uniqueID unique ID (handle with care)
 	**/
-	ccHObject(QString name = QString());
+	ccHObject(const QString& name = QString(), unsigned uniqueID = ccUniqueIDGenerator::InvalidUniqueID);
 	//! Copy constructor
 	ccHObject(const ccHObject& object);
 
 	//! Default destructor
-	virtual ~ccHObject() override;
+	~ccHObject() override;
 
 	//! Static factory
 	/** Warning: objects depending on other structures (such as meshes 
@@ -63,7 +64,7 @@ public: //base members access
 	inline CC_CLASS_ENUM getClassID() const override { return CC_TYPES::HIERARCHY_OBJECT; }
 
 	//! Returns whether the instance is a group
-	inline bool isGroup() const { return getClassID() == CC_TYPES::HIERARCHY_OBJECT; }
+	inline bool isGroup() const { return getClassID() == static_cast<CC_CLASS_ENUM>(CC_TYPES::HIERARCHY_OBJECT); }
 
 	//! Returns parent object
 	/** \return parent object (nullptr if no parent)
@@ -98,7 +99,7 @@ public: //dependencies management
 	//! Returns the dependency flags with a given object
 	/** \param otherObject other object
 	**/
-	int getDependencyFlagsWith(const ccHObject* otherObject);
+	int getDependencyFlagsWith(const ccHObject* otherObject) const;
 
 	//! Removes any dependency flags with a given object
 	/** \param otherObject other object
@@ -144,7 +145,7 @@ public: //children management
 	/** \param uniqueID child unique ID
 		\return child (or nullptr if not found)
 	**/
-	ccHObject* find(unsigned uniqueID);
+	ccHObject* find(unsigned uniqueID) const;
 
 	//! Standard instances container (for children, etc.)
 	using Container = std::vector<ccHObject *>;
@@ -279,7 +280,7 @@ public: //display
 	virtual bool isDisplayed() const;
 
 	//! Returns whether the object is actually displayed (visible) in a given display or not
-	virtual bool isDisplayedIn(ccGenericGLDisplay* display) const;
+	virtual bool isDisplayedIn(const ccGenericGLDisplay* display) const;
 
 	//! Returns whether the object and all its ancestors are enabled
 	virtual bool isBranchEnabled() const;
@@ -349,16 +350,17 @@ public: //display
 	//inherited from ccSerializableObject
 	bool isSerializable() const override;
 	bool toFile(QFile& out) const override;
-	bool fromFile(QFile& in, short dataVersion, int flags) override;
+	bool fromFile(QFile& in, short dataVersion, int flags, LoadedIDMap& oldToNewIDMap) override;
 
 	//! Custom version of ccSerializableObject::fromFile
 	/** This is used to load only the object's part of a stream (and not its children)
 		\param in input file (already opened)
 		\param dataVersion file version
 		\param flags deserialization flags (see ccSerializableObject::DeserializationFlags)
+		\param oldToNewIDMap map to convert old IDs to new ones
 		\return success
 	**/
-	bool fromFileNoChildren(QFile& in, short dataVersion, int flags);
+	bool fromFileNoChildren(QFile& in, short dataVersion, int flags, LoadedIDMap& oldToNewIDMap);
 
 	//! Returns whether object is shareable or not
 	/** If object is father dependent and 'shared', it won't
@@ -418,7 +420,7 @@ protected:
 		\param dataVersion file version
 		\param flags deserialization flags (see ccSerializableObject::DeserializationFlags)
 	**/
-	virtual bool fromFile_MeOnly(QFile& in, short dataVersion, int flags);
+	virtual bool fromFile_MeOnly(QFile& in, short dataVersion, int flags, LoadedIDMap& oldToNewIDMap);
 
 	//! Draws the entity name in 3D
 	/** Names is displayed at the center of the bounding box by default.
