@@ -35,8 +35,9 @@ public:
 
 	//! Default ccMesh constructor
 	/** \param vertices the vertices cloud
+		\param uniqueID unique ID (handle with care)
 	**/
-	explicit ccMesh(ccGenericPointCloud* vertices);
+	explicit ccMesh(ccGenericPointCloud* vertices, unsigned uniqueID = ccUniqueIDGenerator::InvalidUniqueID);
 
 	//! ccMesh constructor (from a CCLib::GenericIndexedMesh)
 	/** The GenericIndexedMesh should refer to a known ccGenericPointCloud.
@@ -49,23 +50,23 @@ public:
 	~ccMesh() override;
 	
 	//! Returns class ID
-	CC_CLASS_ENUM getClassID() const override { return CC_TYPES::MESH; }
+	inline CC_CLASS_ENUM getClassID() const override { return CC_TYPES::MESH; }
 
 	//! Sets the associated vertices cloud (warning)
 	void setAssociatedCloud(ccGenericPointCloud* cloud);
 
 	//! Clones this entity
 	/** All the main features of the entity are cloned, except from the octree
-		\param vertices vertices set to use (will be automatically - AND OPTIMALLY - cloned if 0)
+		\param vertices vertices set to use (will be automatically - AND OPTIMALLY - cloned if nullptr)
 		\param clonedMaterials for internal use
 		\param clonedNormsTable for internal use
 		\param cloneTexCoords for internal use
 		\return a copy of this entity
 	**/
 	ccMesh* cloneMesh(	ccGenericPointCloud* vertices = nullptr,
-								ccMaterialSet* clonedMaterials = nullptr,
-								NormsIndexesTableType* clonedNormsTable = nullptr,
-								TextureCoordsContainer* cloneTexCoords = nullptr);
+						ccMaterialSet* clonedMaterials = nullptr,
+						NormsIndexesTableType* clonedNormsTable = nullptr,
+						TextureCoordsContainer* cloneTexCoords = nullptr);
 
 	//! Creates a Delaunay 2.5D mesh from a point cloud
 	/** See CCLib::PointProjectionTools::computeTriangulation.
@@ -96,10 +97,14 @@ public:
 	inline ccGenericPointCloud* getAssociatedCloud() const override { return m_associatedCloud; }
 	void refreshBB() override;
 	bool interpolateNormals(unsigned triIndex, const CCVector3& P, CCVector3& N) override;
+	bool interpolateNormalsBC(unsigned triIndex, const CCVector3d& w, CCVector3& N) override;
 	bool interpolateColors(unsigned triIndex, const CCVector3& P, ccColor::Rgb& C) override;
+	bool interpolateColorsBC(unsigned triIndex, const CCVector3d& w, ccColor::Rgb& C) override;
+	bool interpolateColors(unsigned triIndex, const CCVector3& P, ccColor::Rgba& C) override;
+	bool interpolateColorsBC(unsigned triIndex, const CCVector3d& w, ccColor::Rgba& C) override;
 	void computeInterpolationWeights(unsigned triIndex, const CCVector3& P, CCVector3d& weights) const override;
-	bool getColorFromMaterial(unsigned triIndex, const CCVector3& P, ccColor::Rgb& C, bool interpolateColorIfNoTexture) override;
-	bool getVertexColorFromMaterial(unsigned triIndex, unsigned char vertIndex, ccColor::Rgb& C, bool returnColorIfNoTexture) override;
+	bool getColorFromMaterial(unsigned triIndex, const CCVector3& P, ccColor::Rgba& C, bool interpolateColorIfNoTexture) override;
+	bool getVertexColorFromMaterial(unsigned triIndex, unsigned char vertIndex, ccColor::Rgba& color, bool returnColorIfNoTexture) override;
 	unsigned capacity() const override;
 
 	//inherited methods (GenericIndexedMesh)
@@ -129,6 +134,11 @@ public:
 	**/
 	void shiftTriangleIndexes(unsigned shift);
 
+	//! Flips the triangle
+	/** Swaps the second and third vertices indexes
+	**/
+	void flipTriangles();
+
 	//! Adds a triangle to the mesh
 	/** \warning Bounding-box validity is broken after a call to this method.
 		However, for the sake of performance, no call to notifyGeometryUpdate
@@ -378,12 +388,12 @@ public:
 	//! Transforms the mesh per-triangle normals
 	void transformTriNormals(const ccGLMatrix& trans);
 
-protected:
+protected: //methods
 
 	//inherited from ccHObject
 	void drawMeOnly(CC_DRAW_CONTEXT& context) override;
 	bool toFile_MeOnly(QFile& out) const override;
-	bool fromFile_MeOnly(QFile& in, short dataVersion, int flags) override;
+	bool fromFile_MeOnly(QFile& in, short dataVersion, int flags, LoadedIDMap& oldToNewIDMap) override;
 	void applyGLTransformation(const ccGLMatrix& trans) override;
 	void onUpdateOf(ccHObject* obj) override;
 	void onDeletionOf(const ccHObject* obj) override;
@@ -391,9 +401,11 @@ protected:
 	//! Same as other 'computeInterpolationWeights' method with a set of 3 vertices indexes
 	void computeInterpolationWeights(const CCLib::VerticesIndexes& vertIndexes, const CCVector3& P, CCVector3d& weights) const;
 	//! Same as other 'interpolateNormals' method with a set of 3 vertices indexes
-	bool interpolateNormals(const CCLib::VerticesIndexes& vertIndexes, const CCVector3& P, CCVector3& N, const Tuple3i* triNormIndexes = nullptr);
+	bool interpolateNormals(const CCLib::VerticesIndexes& vertIndexes, const CCVector3d& w, CCVector3& N, const Tuple3i* triNormIndexes = nullptr);
 	//! Same as other 'interpolateColors' method with a set of 3 vertices indexes
-	bool interpolateColors(const CCLib::VerticesIndexes& vertIndexes, const CCVector3& P, ccColor::Rgb& C);
+	bool interpolateColors(const CCLib::VerticesIndexes& vertIndexes, const CCVector3d& w, ccColor::Rgb& C);
+	//! Same as other 'interpolateColors' method with a set of 3 vertices indexes
+	bool interpolateColors(const CCLib::VerticesIndexes& vertIndexes, const CCVector3d& w, ccColor::Rgba& C);
 
 	//! Used internally by 'subdivide'
 	bool pushSubdivide(/*PointCoordinateType maxArea, */unsigned indexA, unsigned indexB, unsigned indexC);
@@ -423,6 +435,8 @@ protected:
 	//recursive equivalents of some of ccGenericMesh methods (applied to sub-meshes as well)
 	ccMesh_extended_call1(showNormals, bool, showNormals_extended)
 
+protected: //members
+
 	//! associated cloud (vertices)
 	ccGenericPointCloud* m_associatedCloud;
 
