@@ -61,19 +61,20 @@ const unsigned CC_MAX_NUMBER_OF_POINTS_PER_CLOUD = 2000000000; //we must keep it
 	- per-point visibility information (to hide/display subsets of points)
 	- other children objects (meshes, calibrated pictures, etc.)
 **/
-class QCC_DB_LIB_API ccPointCloud : public CCLib::PointCloudTpl<ccGenericPointCloud>
+class QCC_DB_LIB_API ccPointCloud : public CCLib::PointCloudTpl<ccGenericPointCloud, QString>
 {
 public:
 	//! Base class (shortcut)
-	using BaseClass = CCLib::PointCloudTpl<ccGenericPointCloud>;
+	using BaseClass = CCLib::PointCloudTpl<ccGenericPointCloud, QString>;
 
 	//! Default constructor
 	/** Creates an empty cloud without any feature. Each of them shoud be
 		specifically instantiated/created (once the points have been
 		added to this cloud, at least partially).
 		\param name cloud name (optional)
+		\param uniqueID unique ID (handle with care)
 	**/
-	ccPointCloud(QString name = QString()) throw();
+	ccPointCloud(QString name = QString(), unsigned uniqueID = ccUniqueIDGenerator::InvalidUniqueID) throw();
 
 	//! Default destructor
 	~ccPointCloud() override;
@@ -151,7 +152,7 @@ public: //features deletion/clearing
 	/** Prefer ccPointCloud::clear by default.
 		\warning DANGEROUS
 	**/
-	void unalloactePoints();
+	void unallocatePoints();
 
 	//! Erases the cloud colors
 	void unallocateColors();
@@ -179,7 +180,7 @@ public: //features allocation/resize
 	bool reserveThePointsTable(unsigned _numberOfPoints);
 
 	//! Reserves memory to store the RGB colors
-	/** Before adding colors to the cloud (with addRGBColor())
+	/** Before adding colors to the cloud (with addColor())
 		be sure to reserve the necessary amount of memory
 		with this method. This method reserves memory for as
 		many colors as the number of points in the cloud
@@ -455,7 +456,7 @@ public: //other methods
 	const ccColor::Rgb* geScalarValueColor(ScalarType d) const override;
 	const ccColor::Rgb* getPointScalarValueColor(unsigned pointIndex) const override;
 	ScalarType getPointDisplayedDistance(unsigned pointIndex) const override;
-	const ccColor::Rgb& getPointColor(unsigned pointIndex) const override;
+	const ccColor::Rgba& getPointColor(unsigned pointIndex) const override;
 	const CompressedNormType& getPointNormalIndex(unsigned pointIndex) const override;
 	const CCVector3& getPointNormal(unsigned pointIndex) const override;
 	CCLib::ReferenceCloud* crop(const ccBBox& box, bool inside = true) override;
@@ -490,7 +491,12 @@ public: //other methods
 	//! Sets a particular point color
 	/** WARNING: colors must be enabled.
 	**/
-	void setPointColor(unsigned pointIndex, const ccColor::Rgb& col);
+	void setPointColor(unsigned pointIndex, const ccColor::Rgba& col);
+
+	//! Sets a particular point color
+	/** WARNING: colors must be enabled.
+	**/
+	inline void setPointColor(unsigned pointIndex, const ccColor::Rgb& col) { setPointColor(pointIndex, ccColor::Rgba(col, ccColor::MAX)); }
 
 	//! Sets a particular point compressed normal
 	/** WARNING: normals must be enabled.
@@ -537,23 +543,29 @@ public: //other methods
 	**/
 	bool convertNormalToDipDirSFs(ccScalarField* dipSF, ccScalarField* dipDirSF);
 
-	//! Pushes an RGB color on stack
-	/** \param C RGB color
+	//! Pushes an RGBA color on stack
+	/** \param C RGBA color
 	**/
-	void addRGBColor(const ccColor::Rgb& C);
+	void addColor(const ccColor::Rgba& C);
+
+	//! Pushes an RGB color on stack
+	/** Will be converted to an RGBA color automatically.
+		\param C RGB color
+	**/
+	inline void addColor(const ccColor::Rgb& C) { addColor(ccColor::Rgba(C, ccColor::MAX)); }
 
 	//! Pushes an RGB color on stack (shortcut)
 	/** \param r red component
 		\param g green component
 		\param b blue component
 	**/
-	inline void addRGBColor(ColorCompType r, ColorCompType g, ColorCompType b) { addRGBColor(ccColor::Rgb(r, g, b)); }
+	inline void addColor(ColorCompType r, ColorCompType g, ColorCompType b, ColorCompType a = ccColor::MAX) { addColor(ccColor::Rgba(r, g, b, a)); }
 
 	//! Pushes a grey color on stack (shortcut)
 	/** Shortcut: color is converted to RGB(g, g, g)
 		\param g grey component
 	**/
-	inline void addGreyColor(ColorCompType g) { addRGBColor(ccColor::Rgb(g, g, g)); }
+	inline void addGreyColor(ColorCompType g) { addColor(ccColor::Rgba(g, g, g, ccColor::MAX)); }
 
 	//! Converts RGB to grey scale colors
 	/** \return success
@@ -566,9 +578,10 @@ public: //other methods
 		\param r red component
 		\param g green component
 		\param b blue component
+		\param a alpha component
 		\return success
 	**/
-	bool colorize(float r, float g, float b);
+	bool colorize(float r, float g, float b, float a = 1.0f);
 
 	//! Assigns color to points proportionnaly to their 'height'
 	/** Height is defined wrt to the specified dimension (heightDim).
@@ -588,26 +601,34 @@ public: //other methods
 	**/
 	bool setRGBColorByBanding(unsigned char dim, double freq);
 
-	//! Sets RGB colors with current scalar field (values & parameters)
+	//! Converts current scalar field (values & display parameters) to RGB colors
 	/** \return success
 	**/
-	bool setRGBColorWithCurrentScalarField(bool mixWithExistingColor = false);
+	bool convertCurrentScalarFieldToColors(bool mixWithExistingColor = false);
 
 	//! Set a unique color for the whole cloud (shortcut)
 	/** Color array is automatically allocated if necessary.
 		\param r red component
 		\param g green component
 		\param b blue component
+		\param a alpha component
 		\return success
 	**/
-	inline bool setRGBColor(ColorCompType r, ColorCompType g, ColorCompType b) { return setRGBColor(ccColor::Rgb(r, g, b)); }
+	inline bool setColor(ColorCompType r, ColorCompType g, ColorCompType b, ColorCompType a = ccColor::MAX) { return setColor(ccColor::Rgba(r, g, b, a)); }
 
-	//! Set a unique color for the whole cloud
+	//! Set a unique color for the whole cloud (RGB version)
 	/** Color array is automatically allocated if necessary.
-		\param col RGB color (size: 3)
+		\param col RGB color
 		\return success
 	**/
-	bool setRGBColor(const ccColor::Rgb& col);
+	inline bool setColor(const ccColor::Rgb& col) { return setColor(ccColor::Rgba(col, ccColor::MAX)); }
+
+	//! Set a unique color for the whole cloud (RGBA version)
+	/** Color array is automatically allocated if necessary.
+		\param col RGBA color
+		\return success
+	**/
+	bool setColor(const ccColor::Rgba& col);
 
 	//! Inverts normals (if any)
 	void invertNormals();
@@ -633,41 +654,40 @@ public: //other methods
 	**/
 	void hidePointsByScalarValue(ScalarType minVal, ScalarType maxVal);
 
-	//! Unrolls the cloud and its normals on a cylinder
+	enum UnrollMode { CYLINDER = 0, CONE = 1, STRAIGHTENED_CONE = 2, STRAIGHTENED_CONE2 = 3 };
+
+	struct UnrollBaseParams
+	{
+		PointCoordinateType radius;	//!< unrolling cylinder radius (or cone base radius)
+		unsigned char axisDim;		//!< unrolling cylinder/cone axis (X=0, Y=1 or Z=2)
+	};
+	struct UnrollCylinderParams : public UnrollBaseParams
+	{
+		CCVector3 center;			//! A point belonging to the cylinder axis
+	};
+	struct UnrollConeParams : public UnrollBaseParams
+	{
+		CCVector3 apex;				//! Cone apex
+		double coneAngle_deg;		//! Cone aperture angle (in degrees)
+	};
+
+	//! Unrolls the cloud and its normals on a cylinder or a cone
 	/** This method is redundant with the "developCloudOnCylinder" method of CCLib,
 		apart that it can also handle the cloud normals.
-		\param radius unrolling cylinder radius
-		\param coneAxisDim dimension along which the cylinder axis is aligned (X=0, Y=1, Z=2)
-		\param center a point belonging to the cylinder axis (automatically computed if not specified)
+		\param mode unrolling mode
+		\param params unrolling parameters (must match the unrolling mode)
 		\param exportDeviationSF to export the deviation fro the ideal cone as a scalar field
+		\param startAngle_deg start angle (in degrees) - 0 corresponds to +X (east)
+		\param stopAngle_deg stop angle (in degrees)
 		\param progressCb for progress notification
 		\return the unrolled point cloud
 		**/
-	ccPointCloud* unrollOnCylinder(	PointCoordinateType radius,
-									unsigned char coneAxisDim,
-									CCVector3* center = nullptr,
-									bool exportDeviationSF = false,
-									double startAngle_deg = 0.0,
-									double stopAngle_deg = 360.0,
-									CCLib::GenericProgressCallback* progressCb = nullptr) const;
-
-	//! Unrolls the cloud (and its normals) on a cone
-	/** \param coneAngle_deg cone apex angle (between 0 and 180 degrees)
-		\param coneApex cone apex 3D position
-		\param coneAxisDim dimension along which the cone axis is aligned (X=0, Y=1, Z=2)
-		\param developStraightenedCone if true, this method will unroll a straightened version of the cone (as a cylinder)
-		\param baseRadius unrolling straightened cone base radius (necessary if developStraightenedCone is true)
-		\param exportDeviationSF to export the deviation fro the ideal cone as a scalar field
-		\param progressCb for progress notification
-		\return the unrolled point cloud
-	**/
-	ccPointCloud* unrollOnCone(	double coneAngle_deg,
-								const CCVector3& coneApex,
-								unsigned char coneAxisDim,
-								bool developStraightenedCone,
-								PointCoordinateType baseRadius,
-								bool exportDeviationSF = false,
-								CCLib::GenericProgressCallback* progressCb = nullptr) const;
+	ccPointCloud* unroll(	UnrollMode mode,
+							UnrollBaseParams* params,
+							bool exportDeviationSF = false,
+							double startAngle_deg = 0.0,
+							double stopAngle_deg = 360.0,
+							CCLib::GenericProgressCallback* progressCb = nullptr) const;
 
 	//! Adds associated SF color ramp info to current GL context
 	void addColorRampInfo(CC_DRAW_CONTEXT& context);
@@ -679,8 +699,8 @@ public: //other methods
 	**/
 	int addScalarField(ccScalarField* sf);
 
-	//! Returns pointer on RGB colors table
-	ColorsTableType* rgbColors() const { return m_rgbColors; }
+	//! Returns pointer on RGBA colors table
+	RGBAColorsTableType* rgbaColors() const { return m_rgbaColors; }
 
 	//! Returns pointer on compressed normals indexes table
 	NormsIndexesTableType* normals() const { return m_normals; }
@@ -709,13 +729,16 @@ public: //other methods
 	//! Exports the specified coordinate dimension(s) to scalar field(s)
 	bool exportCoordToSF(bool exportDims[3]);
 
+	//! Exports the specified normal dimension(s) to scalar field(s)
+	bool exportNormalToSF(bool exportDims[3]);
+
 protected:
 
 	//inherited from ccHObject
 	void drawMeOnly(CC_DRAW_CONTEXT& context) override;
 	void applyGLTransformation(const ccGLMatrix& trans) override;
 	bool toFile_MeOnly(QFile& out) const override;
-	bool fromFile_MeOnly(QFile& in, short dataVersion, int flags) override;
+	bool fromFile_MeOnly(QFile& in, short dataVersion, int flags, LoadedIDMap& oldToNewIDMap) override;
 	void notifyGeometryUpdate() override;
 
 	//inherited from PointCloud
@@ -724,7 +747,7 @@ protected:
 	void swapPoints(unsigned firstIndex, unsigned secondIndex) override;
 
 	//! Colors
-	ColorsTableType* m_rgbColors;
+	RGBAColorsTableType* m_rgbaColors;
 
 	//! Normals (compressed)
 	NormsIndexesTableType* m_normals;
