@@ -21,12 +21,12 @@
 #include "ccPolyline.h"
 
 //Local
-#include "ccPointCloud.h"
 #include "ccCone.h"
+#include "ccPointCloud.h"
 
-ccPolyline::ccPolyline(GenericIndexedCloudPersist* associatedCloud)
+ccPolyline::ccPolyline(GenericIndexedCloudPersist* associatedCloud, unsigned uniqueID/*=ccUniqueIDGenerator::InvalidUniqueID*/)
 	: Polyline(associatedCloud)
-	, ccShiftedObject("Polyline")
+	, ccShiftedObject("Polyline", uniqueID)
 {
 	set2DMode(false);
 	setForeground(true);
@@ -47,10 +47,10 @@ ccPolyline::ccPolyline(GenericIndexedCloudPersist* associatedCloud)
 }
 
 ccPolyline::ccPolyline(const ccPolyline& poly)
-	: Polyline(0)
+	: Polyline(nullptr)
 	, ccShiftedObject(poly)
 {
-	ccPointCloud* clone = 0;
+	ccPointCloud* clone = nullptr;
 	initWith(clone, poly);
 }
 
@@ -160,7 +160,7 @@ void ccPolyline::applyGLTransformation(const ccGLMatrix& trans)
 }
 
 //unit arrow
-static QSharedPointer<ccCone> c_unitArrow(0);
+static QSharedPointer<ccCone> c_unitArrow(nullptr);
 
 void ccPolyline::drawMeOnly(CC_DRAW_CONTEXT& context)
 {
@@ -196,98 +196,95 @@ void ccPolyline::drawMeOnly(CC_DRAW_CONTEXT& context)
 		glFunc->glPushName(getUniqueIDForDisplay());
 
 	if (isColorOverriden())
-		ccGL::Color3v(glFunc, getTempColor().rgb);
+		ccGL::Color4v(glFunc, getTempColor().rgba);
 	else if (colorsShown())
 		ccGL::Color3v(glFunc, m_rgbColor.rgb);
 
 	//display polyline
-	if (vertCount > 1)
+	if (m_width != 0)
 	{
-		if (m_width != 0)
-		{
-			glFunc->glPushAttrib(GL_LINE_BIT);
-			glFunc->glLineWidth(static_cast<GLfloat>(m_width));
-		}
+		glFunc->glPushAttrib(GL_LINE_BIT);
+		glFunc->glLineWidth(static_cast<GLfloat>(m_width));
+	}
 
-		//DGM: we do the 'GL_LINE_LOOP' manually as I have a strange bug
-		//on one on my graphic card with this mode!
-		//glBegin(m_isClosed ? GL_LINE_LOOP : GL_LINE_STRIP);
-		glFunc->glBegin(GL_LINE_STRIP);
-		for (unsigned i = 0; i < vertCount; ++i)
-		{
-			ccGL::Vertex3v(glFunc, getPoint(i)->u);
-		}
-		if (m_isClosed)
-		{
-			ccGL::Vertex3v(glFunc, getPoint(0)->u);
-		}
-		glFunc->glEnd();
+	//DGM: we do the 'GL_LINE_LOOP' manually as I have a strange bug
+	//on one on my graphic card with this mode!
+	//glBegin(m_isClosed ? GL_LINE_LOOP : GL_LINE_STRIP);
+	glFunc->glBegin(GL_LINE_STRIP);
+	for (unsigned i = 0; i < vertCount; ++i)
+	{
+		ccGL::Vertex3v(glFunc, getPoint(i)->u);
+	}
+	if (m_isClosed)
+	{
+		ccGL::Vertex3v(glFunc, getPoint(0)->u);
+	}
+	glFunc->glEnd();
 
-		//display arrow
-		if (m_showArrow && m_arrowIndex < vertCount && (m_arrowIndex > 0 || m_isClosed))
-		{
-			const CCVector3* P0 = getPoint(m_arrowIndex == 0 ? vertCount - 1 : m_arrowIndex - 1);
-			const CCVector3* P1 = getPoint(m_arrowIndex);
-			//direction of the last polyline chunk
-			CCVector3 u = *P1 - *P0;
-			u.normalize();
+	//display arrow
+	if (m_showArrow && m_arrowIndex < vertCount && (m_arrowIndex > 0 || m_isClosed))
+	{
+		const CCVector3* P0 = getPoint(m_arrowIndex == 0 ? vertCount - 1 : m_arrowIndex - 1);
+		const CCVector3* P1 = getPoint(m_arrowIndex);
+		//direction of the last polyline chunk
+		CCVector3 u = *P1 - *P0;
+		u.normalize();
 
-			if (m_mode2D)
+		if (m_mode2D)
+		{
+			u *= -m_arrowLength;
+			static const PointCoordinateType s_defaultArrowAngle = static_cast<PointCoordinateType>(15.0 * CC_DEG_TO_RAD);
+			static const PointCoordinateType cost = cos(s_defaultArrowAngle);
+			static const PointCoordinateType sint = sin(s_defaultArrowAngle);
+			CCVector3 A(cost * u.x - sint * u.y, sint * u.x + cost * u.y, 0);
+			CCVector3 B(cost * u.x + sint * u.y, -sint * u.x + cost * u.y, 0);
+			glFunc->glBegin(GL_POLYGON);
+			ccGL::Vertex3v(glFunc, (A + *P1).u);
+			ccGL::Vertex3v(glFunc, (B + *P1).u);
+			ccGL::Vertex3v(glFunc, (*P1).u);
+			glFunc->glEnd();
+		}
+		else
+		{
+			if (!c_unitArrow)
 			{
-				u *= -m_arrowLength;
-				static const PointCoordinateType s_defaultArrowAngle = static_cast<PointCoordinateType>(15.0 * CC_DEG_TO_RAD);
-				static const PointCoordinateType cost = cos(s_defaultArrowAngle);
-				static const PointCoordinateType sint = sin(s_defaultArrowAngle);
-				CCVector3 A(cost * u.x - sint * u.y, sint * u.x + cost * u.y, 0);
-				CCVector3 B(cost * u.x + sint * u.y, -sint * u.x + cost * u.y, 0);
-				glFunc->glBegin(GL_POLYGON);
-				ccGL::Vertex3v(glFunc, (A + *P1).u);
-				ccGL::Vertex3v(glFunc, (B + *P1).u);
-				ccGL::Vertex3v(glFunc, (*P1).u);
-				glFunc->glEnd();
+				c_unitArrow = QSharedPointer<ccCone>(new ccCone(0.5, 0.0, 1.0));
+				c_unitArrow->showColors(true);
+				c_unitArrow->showNormals(false);
+				c_unitArrow->setVisible(true);
+				c_unitArrow->setEnabled(true);
 			}
+			if (colorsShown())
+				c_unitArrow->setTempColor(m_rgbColor);
 			else
-			{
-				if (!c_unitArrow)
-				{
-					c_unitArrow = QSharedPointer<ccCone>(new ccCone(0.5, 0.0, 1.0));
-					c_unitArrow->showColors(true);
-					c_unitArrow->showNormals(false);
-					c_unitArrow->setVisible(true);
-					c_unitArrow->setEnabled(true);
-				}
-				if (colorsShown())
-					c_unitArrow->setTempColor(m_rgbColor);
-				else
-					c_unitArrow->setTempColor(context.pointsDefaultCol);
-				//build-up unit arrow own 'context'
-				CC_DRAW_CONTEXT markerContext = context;
-				markerContext.drawingFlags &= (~CC_DRAW_ENTITY_NAMES); //we must remove the 'push name flag' so that the sphere doesn't push its own!
-				markerContext.display = 0;
+				c_unitArrow->setTempColor(context.pointsDefaultCol);
+			//build-up unit arrow own 'context'
+			CC_DRAW_CONTEXT markerContext = context;
+			markerContext.drawingFlags &= (~CC_DRAW_ENTITY_NAMES); //we must remove the 'push name flag' so that the sphere doesn't push its own!
+			markerContext.display = nullptr;
 
-				glFunc->glMatrixMode(GL_MODELVIEW);
-				glFunc->glPushMatrix();
-				ccGL::Translate(glFunc, P1->x, P1->y, P1->z);
-				ccGLMatrix rotMat = ccGLMatrix::FromToRotation(u, CCVector3(0, 0, PC_ONE));
-				glFunc->glMultMatrixf(rotMat.inverse().data());
-				glFunc->glScalef(m_arrowLength, m_arrowLength, m_arrowLength);
-				ccGL::Translate(glFunc, 0.0, 0.0, -0.5);
-				c_unitArrow->draw(markerContext);
-				glFunc->glPopMatrix();
-			}
+			glFunc->glMatrixMode(GL_MODELVIEW);
+			glFunc->glPushMatrix();
+			ccGL::Translate(glFunc, P1->x, P1->y, P1->z);
+			ccGLMatrix rotMat = ccGLMatrix::FromToRotation(u, CCVector3(0, 0, PC_ONE));
+			glFunc->glMultMatrixf(rotMat.inverse().data());
+			glFunc->glScalef(m_arrowLength, m_arrowLength, m_arrowLength);
+			ccGL::Translate(glFunc, 0.0, 0.0, -0.5);
+			c_unitArrow->draw(markerContext);
+			glFunc->glPopMatrix();
 		}
+	}
 
-		if (m_width != 0)
-		{
-			glFunc->glPopAttrib();
-		}
+	if (m_width != 0)
+	{
+		glFunc->glPopAttrib();
 	}
 
 	//display vertices
 	if (m_showVertices)
 	{
 		glFunc->glPushAttrib(GL_POINT_BIT);
-		glFunc->glPointSize((GLfloat)m_vertMarkWidth);
+		glFunc->glPointSize(static_cast<GLfloat>(m_vertMarkWidth));
 
 		glFunc->glBegin(GL_POINTS);
 		for (unsigned i = 0; i < vertCount; ++i)
@@ -364,9 +361,9 @@ bool ccPolyline::toFile_MeOnly(QFile& out) const
 	return true;
 }
 
-bool ccPolyline::fromFile_MeOnly(QFile& in, short dataVersion, int flags)
+bool ccPolyline::fromFile_MeOnly(QFile& in, short dataVersion, int flags, LoadedIDMap& oldToNewIDMap)
 {
-	if (!ccHObject::fromFile_MeOnly(in, dataVersion, flags))
+	if (!ccHObject::fromFile_MeOnly(in, dataVersion, flags, oldToNewIDMap))
 		return false;
 
 	if (dataVersion < 28)
@@ -675,12 +672,12 @@ ccPointCloud* ccPolyline::samplePoints(	bool densityBased,
 		if (isColorOverriden())
 		{
 			//we use the default 'temporary' color
-			cloud->setRGBColor(getTempColor());
+			cloud->setColor(getTempColor());
 		}
 		else if (colorsShown())
 		{
 			//we use the default color
-			cloud->setRGBColor(m_rgbColor);
+			cloud->setColor(m_rgbColor);
 		}
 	}
 
