@@ -104,7 +104,8 @@ public:
 		INTERACT_SIG_LB_CLICKED  = 256,      //left button clicked
 		INTERACT_SIG_MOUSE_MOVED = 512,      //mouse moved (only if a button is clicked)
 		INTERACT_SIG_BUTTON_RELEASED = 1024, //mouse button released
-		INTERACT_SEND_ALL_SIGNALS = INTERACT_SIG_RB_CLICKED | INTERACT_SIG_LB_CLICKED | INTERACT_SIG_MOUSE_MOVED | INTERACT_SIG_BUTTON_RELEASED,
+		INTERACT_SIG_MB_CLICKED  = 2048,     //middle button clicked
+		INTERACT_SEND_ALL_SIGNALS = INTERACT_SIG_RB_CLICKED | INTERACT_SIG_LB_CLICKED | INTERACT_SIG_MB_CLICKED| INTERACT_SIG_MOUSE_MOVED | INTERACT_SIG_BUTTON_RELEASED,
 	};
 	Q_DECLARE_FLAGS(INTERACTION_FLAGS, INTERACTION_FLAG)
 
@@ -173,8 +174,8 @@ public:
 	void refresh(bool only2D = false) override;
 	void invalidateViewport() override;
 	void deprecate3DLayer() override;
-	void display3DLabel(const QString& str, const CCVector3& pos3D, const unsigned char* rgbColor = nullptr, const QFont& font = QFont()) override;
-	void displayText(QString text, int x, int y, unsigned char align = ALIGN_DEFAULT, float bkgAlpha = 0.0f, const unsigned char* rgbColor = nullptr, const QFont* font = nullptr) override;
+	void display3DLabel(const QString& str, const CCVector3& pos3D, const ccColor::Rgba* color = nullptr, const QFont& font = QFont()) override;
+	void displayText(QString text, int x, int y, unsigned char align = ALIGN_DEFAULT, float bkgAlpha = 0.0f, const ccColor::Rgba* color = nullptr, const QFont* font = nullptr) override;
 	QFont getTextDisplayFont() const override; //takes rendering zoom into account!
 	QFont getLabelDisplayFont() const override; //takes rendering zoom into account!
 	const ccViewportParameters& getViewportParameters() const override { return m_viewportParams; }
@@ -373,9 +374,9 @@ public:
 	virtual void getContext(CC_DRAW_CONTEXT& context);
 
 	//! Minimum point size
-	static const float MIN_POINT_SIZE_F;
+	static constexpr float MIN_POINT_SIZE_F = 1.0f;
 	//! Maximum point size
-	static const float MAX_POINT_SIZE_F;
+	static constexpr float MAX_POINT_SIZE_F = 16.0f;
 
 	//! Sets point size
 	/** \param size point size (between MIN_POINT_SIZE_F and MAX_POINT_SIZE_F)
@@ -383,14 +384,14 @@ public:
 	virtual void setPointSize(float size, bool silent = false);
 	
 	//! Minimum line width
-	static const float MIN_LINE_WIDTH_F;
+	static constexpr float MIN_LINE_WIDTH_F = 1.0f;
 	//! Maximum line width
-	static const float MAX_LINE_WIDTH_F;
+	static constexpr float MAX_LINE_WIDTH_F = 16.0f;
 
 	//! Sets line width
 	/** \param width lines width (between MIN_LINE_WIDTH_F and MAX_LINE_WIDTH_F)
 	**/
-	virtual void setLineWidth(float width);
+	virtual void setLineWidth(float width, bool silent = false);
 
 	//! Returns current font size
 	virtual int getFontPointSize() const;
@@ -575,15 +576,17 @@ public: //stereo mode
 							RED_CYAN = 3,
 							CYAN_RED = 4,
 							NVIDIA_VISION = 5,
-							OCULUS = 6
+							OCULUS = 6,
+							GENERIC_STEREO_DISPLAY = 7
 		};
 
 		//! Whether stereo-mode is 'analgyph' or real stereo mode
 		inline bool isAnaglyph() const { return glassType <= 4; }
-		
-		bool autoFocal;
-		double focalDist;
-		double eyeSepFactor;
+
+		int screenWidth_mm;
+		int screenDistance_mm;
+		int eyeSeparation_mm;
+		int stereoStrength;
 		GlassType glassType;
 	};
 
@@ -678,8 +681,9 @@ signals:
 		\param x mouse cursor x position
 		\param y mouse cursor y position
 		\param P the picked point
+		\param uvw barycentric coordinates of the point (if picked on a mesh)
 	**/
-	void itemPicked(ccHObject* entity, unsigned subEntityID, int x, int y, const CCVector3& P);
+	void itemPicked(ccHObject* entity, unsigned subEntityID, int x, int y, const CCVector3& P, const CCVector3d& uvw);
 
 	//! Signal emitted when an item is picked (FAST_PICKING mode only)
 	/** \param entity entity
@@ -730,14 +734,14 @@ signals:
 	**/
 	void rotation(const ccGLMatrixd& rotMat);
 
-	//! Signal emitted when the left mouse button is cliked on the window
+	//! Signal emitted when the left mouse button is clicked on the window
 	/** See INTERACT_SIG_LB_CLICKED.
 		Arguments correspond to the clicked point coordinates (x,y) in
 		pixels relative to the window corner!
 	**/
 	void leftButtonClicked(int x, int y);
 
-	//! Signal emitted when the right mouse button is cliked on the window
+	//! Signal emitted when the right mouse button is clicked on the window
 	/** See INTERACT_SIG_RB_CLICKED.
 		Arguments correspond to the clicked point coordinates (x,y) in
 		pixels relative to the window corner!
@@ -771,6 +775,13 @@ signals:
 	//! Signal emitted when the exclusive fullscreen is toggled
 	void exclusiveFullScreenToggled(bool exclusive);
 
+	//! Signal emitted when the middle mouse button is clicked on the window
+	/** See INTERACT_SIG_MB_CLICKED.
+		Arguments correspond to the clicked point coordinates (x,y) in
+		pixels relative to the window corner!
+	**/
+	void middleButtonClicked(int x, int y);
+
 protected: //rendering
 
 	//Default OpenGL functions set
@@ -817,38 +828,7 @@ protected: //rendering
 	};
 
 	//! Rendering params
-	struct RenderingParams
-	{
-		RenderingParams()
-			: passIndex(0)
-			, passCount(1)
-			, drawBackground(true)
-			, clearDepthLayer(true)
-			, clearColorLayer(true)
-			, draw3DPass(true)
-			, useFBO(false)
-			, draw3DCross(false)
-			, drawForeground(true)
-		{}
-
-		unsigned char passIndex;
-		unsigned char passCount;
-
-		//2D background
-		bool drawBackground;
-		bool clearDepthLayer;
-		bool clearColorLayer;
-
-		//3D central layer
-		bool draw3DPass;
-		bool useFBO;
-		bool draw3DCross;
-		//! Next LOD state
-		LODState nextLODState;
-
-		//2D foreground
-		bool drawForeground;
-	};
+	struct RenderingParams;
 
 	//! Full rendering pass (drawBackground + draw3D + drawForeground)
 	void fullRenderingPass(CC_DRAW_CONTEXT& context, RenderingParams& params);
@@ -927,20 +907,7 @@ protected: //other methods
 	ccGLMatrixd computeModelViewMatrix(const CCVector3d& cameraCenter) const;
 
 	//! Optional output metrics (from computeProjectionMatrix)
-	struct ProjectionMetrics
-	{
-		ProjectionMetrics()
-			: zNear(0.0)
-			, zFar(0.0)
-			, cameraToBBCenterDist(0.0)
-			, bbHalfDiag(0.0)
-		{}
-		
-		double zNear;
-		double zFar;
-		double cameraToBBCenterDist;
-		double bbHalfDiag;
-	};
+	struct ProjectionMetrics;
 
 	//! Computes the projection matrix
 	ccGLMatrixd computeProjectionMatrix(	const CCVector3d& cameraCenter,
@@ -963,33 +930,7 @@ protected: //other methods
 	void drawPivot();
 
 	//! Picking parameters
-	struct PickingParameters
-	{
-		//! Default constructor
-		PickingParameters(	PICKING_MODE _mode = NO_PICKING,
-							int _centerX = 0,
-							int _centerY = 0,
-							int _pickWidth = 5,
-							int _pickHeight = 5,
-							bool _pickInSceneDB = true,
-							bool _pickInLocalDB = true)
-			: mode(_mode)
-			, centerX(_centerX)
-			, centerY(_centerY)
-			, pickWidth(_pickWidth)
-			, pickHeight(_pickHeight)
-			, pickInSceneDB(_pickInSceneDB)
-			, pickInLocalDB(_pickInLocalDB)
-		{}
-
-		PICKING_MODE mode;
-		int centerX;
-		int centerY;
-		int pickWidth;
-		int pickHeight;
-		bool pickInSceneDB;
-		bool pickInLocalDB;
-	};
+	struct PickingParameters;
 
 	//! Starts picking process
 	/** \param params picking parameters
@@ -1007,6 +948,7 @@ protected: //other methods
 								ccHObject* pickedEntity,
 								int pickedItemIndex,
 								const CCVector3* nearestPoint = nullptr,
+								const CCVector3d* nearestPointBC = nullptr, //barycentric coordinates
 								const std::unordered_set<int>* selectedIDs = nullptr);
 	
 	//! Updates currently active items list (m_activeItems)
@@ -1378,9 +1320,6 @@ protected: //members
 	//! Whether the pivot point is automatically picked at the center of the screen (when possible)
 	bool m_autoPickPivotAtCenter;
 
-	//! Candidate pivot point (will be used when the mouse is released)
-	CCVector3d m_autoPivotCandidate;
-
 	//! Deferred picking
 	QTimer m_deferredPickingTimer;
 
