@@ -31,6 +31,7 @@
 #include <QLabel>
 #include <QVBoxLayout>
 
+#include <numeric>
 
 void ccRenderingTools::ShowDepthBuffer(ccGBLSensor* sensor, QWidget* parent/*=0*/, unsigned maxDim/*=1024*/)
 {
@@ -44,8 +45,8 @@ void ccRenderingTools::ShowDepthBuffer(ccGBLSensor* sensor, QWidget* parent/*=0*
 	}
 
 	//determine min and max depths
-	ScalarType minDist = 0.0f;
-	ScalarType maxDist = 0.0f;
+	PointCoordinateType minDist = 0.0f;
+	PointCoordinateType maxDist = 0.0f;
 	{
 		const PointCoordinateType* _zBuff = depthBuffer.zBuff.data();
 		double sumDist = 0.0;
@@ -76,7 +77,7 @@ void ccRenderingTools::ShowDepthBuffer(ccGBLSensor* sensor, QWidget* parent/*=0*
 			double avg = sumDist / count;
 			double stdDev = sqrt(fabs(sumDist2 / count - avg*avg));
 			//for better dynamics
-			maxDist = std::min(maxDist, static_cast<ScalarType>(avg + 1.0 * stdDev));
+			maxDist = std::min(maxDist, static_cast<PointCoordinateType>(avg + 1.0 * stdDev));
 		}
 	}
 
@@ -190,8 +191,8 @@ void ConvertToLogScale(ScalarType& dispMin, ScalarType& dispMax)
 {
 	ScalarType absDispMin = (dispMax < 0 ? std::min(-dispMax, -dispMin) : std::max<ScalarType>(dispMin, 0)); 
 	ScalarType absDispMax = std::max(std::abs(dispMin), std::abs(dispMax));
-	dispMin = std::log10(std::max(absDispMin, FLT_EPSILON));
-	dispMax = std::log10(std::max(absDispMax, FLT_EPSILON));
+	dispMin = std::log10(std::max(absDispMin, std::numeric_limits<ScalarType>::epsilon()));
+	dispMax = std::log10(std::max(absDispMax, std::numeric_limits<ScalarType>::epsilon()));
 }
 
 void ccRenderingTools::DrawColorRamp(const CC_DRAW_CONTEXT& context)
@@ -406,11 +407,11 @@ void ccRenderingTools::DrawColorRamp(const CC_DRAW_CONTEXT& context, const ccSca
 					if (customLabels)
 					{
 						assert(sf->getColorScale() && !sf->getColorScale()->isRelative());
-						col = sf->getColorScale()->getColorByValue(value, &ccColor::lightGrey);
+						col = sf->getColorScale()->getColorByValue(value, &ccColor::lightGreyRGB);
 					}
 					else
 					{
-						col = &ccColor::lightGrey;
+						col = &ccColor::lightGreyRGB;
 					}
 				}
 				assert(col);
@@ -450,7 +451,7 @@ void ccRenderingTools::DrawColorRamp(const CC_DRAW_CONTEXT& context, const ccSca
 			if (logScale)
 				value = exp(value*c_log10);
 			const ccColor::Rgb* col = sf->getColor(static_cast<ScalarType>(value));
-			glFunc->glColor3ubv(col ? col->rgb : ccColor::lightGrey.rgb);
+			glFunc->glColor3ubv(col ? col->rgb : ccColor::lightGreyRGB.rgb);
 			glFunc->glBegin(GL_POLYGON);
 			glFunc->glVertex2i(x,y);
 			glFunc->glVertex2i(x+scaleWidth,y);
