@@ -29,6 +29,7 @@
 
 //qCC_db
 #include <cc2DLabel.h>
+#include <ccHObjectCaster.h>
 #include <ccLog.h>
 #include <ccPointCloud.h>
 #include <ccProgressDialog.h>
@@ -69,14 +70,18 @@ AsciiOpenDlg* AsciiFilter::GetOpenDialog(QWidget* parentWidget/*=0*/)
 	return s_openDialog;
 }
 
-bool AsciiFilter::canLoadExtension(const QString& upperCaseExt) const
+
+AsciiFilter::AsciiFilter()
+	: FileIOFilter( {
+					"_ASCII Filter",
+					2.0f,	// priority
+					QStringList{ "txt", "asc", "neu", "xyz", "pts", "csv" },
+					"asc",
+					QStringList{ GetFileFilter() },
+					QStringList{ GetFileFilter() },
+					Import | Export | BuiltIn
+					} )
 {
-	return (	upperCaseExt == "ASC"
-			||	upperCaseExt == "TXT"
-			||	upperCaseExt == "XYZ"
-			||	upperCaseExt == "NEU"
-			||	upperCaseExt == "PTS"
-			||	upperCaseExt == "CSV");
 }
 
 bool AsciiFilter::canSave(CC_CLASS_ENUM type, bool& multiple, bool& exclusive) const
@@ -211,7 +216,7 @@ CC_FILE_ERROR AsciiFilter::saveToFile(ccHObject* entity, const QString& filename
 	//output precision
 	const int s_coordPrecision = saveDialog->coordsPrecision();
 	const int s_sfPrecision = saveDialog->sfPrecision(); 
-	const int s_nPrecision = 2+sizeof(PointCoordinateType);
+	const int s_nPrecision = 2 + sizeof(PointCoordinateType);
 
 	//other parameters
 	bool saveColumnsHeader = saveDialog->saveColumnsNamesHeader();
@@ -219,6 +224,7 @@ CC_FILE_ERROR AsciiFilter::saveToFile(ccHObject* entity, const QString& filename
 	bool swapColorAndSFs = saveDialog->swapColorAndSF();
 	QChar separator(saveDialog->getSeparator());
 	bool saveFloatColors = saveDialog->saveFloatColors();
+	bool saveAlphaChannel = saveDialog->saveAlphaChannel();
 
 	if (saveColumnsHeader)
 	{
@@ -237,6 +243,11 @@ CC_FILE_ERROR AsciiFilter::saveToFile(ccHObject* entity, const QString& filename
 			header.append(saveFloatColors ? AsciiHeaderColumns::Gf() : AsciiHeaderColumns::G());
 			header.append(separator);
 			header.append(saveFloatColors ? AsciiHeaderColumns::Bf() : AsciiHeaderColumns::B());
+			if (saveAlphaChannel)
+			{
+				header.append(separator);
+				header.append(saveFloatColors ? AsciiHeaderColumns::Af() : AsciiHeaderColumns::A());
+			}
 		}
 
 		if (writeSF)
@@ -259,6 +270,11 @@ CC_FILE_ERROR AsciiFilter::saveToFile(ccHObject* entity, const QString& filename
 			header.append(saveFloatColors ? AsciiHeaderColumns::Gf() : AsciiHeaderColumns::G());
 			header.append(separator);
 			header.append(saveFloatColors ? AsciiHeaderColumns::Bf() : AsciiHeaderColumns::B());
+			if (saveAlphaChannel)
+			{
+				header.append(separator);
+				header.append(saveFloatColors ? AsciiHeaderColumns::Af() : AsciiHeaderColumns::A());
+			}
 		}
 
 		if (writeNorms)
@@ -298,7 +314,7 @@ CC_FILE_ERROR AsciiFilter::saveToFile(ccHObject* entity, const QString& filename
 		if (writeColors)
 		{
 			//add rgb color
-			const ccColor::Rgb& col = cloud->getPointColor(i);
+			const ccColor::Rgba& col = cloud->getPointColor(i);
 			if (saveFloatColors)
 			{
 				colorLine.append(separator);
@@ -307,6 +323,11 @@ CC_FILE_ERROR AsciiFilter::saveToFile(ccHObject* entity, const QString& filename
 				colorLine.append(QString::number(static_cast<double>(col.g) / ccColor::MAX));
 				colorLine.append(separator);
 				colorLine.append(QString::number(static_cast<double>(col.b) / ccColor::MAX));
+				if (saveAlphaChannel)
+				{
+					colorLine.append(separator);
+					colorLine.append(QString::number(static_cast<double>(col.a) / ccColor::MAX));
+				}
 			}
 			else
 			{
@@ -316,10 +337,17 @@ CC_FILE_ERROR AsciiFilter::saveToFile(ccHObject* entity, const QString& filename
 				colorLine.append(QString::number(col.g));
 				colorLine.append(separator);
 				colorLine.append(QString::number(col.b));
+				if (saveAlphaChannel)
+				{
+					colorLine.append(separator);
+					colorLine.append(QString::number(col.a));
+				}
 			}
 
 			if (!swapColorAndSFs)
+			{
 				line.append(colorLine);
+			}
 		}
 
 		if (writeSF)
@@ -428,6 +456,7 @@ CC_FILE_ERROR AsciiFilter::loadFile(const QString& filename,
 
 	AsciiOpenDlg::Sequence openSequence = openDialog->getOpenSequence();
 	char separator = static_cast<char>(openDialog->getSeparator());
+	bool commaAsDecimal = openDialog->useCommaAsDecimal();
 	unsigned maxCloudSize = openDialog->getMaxCloudSize();
 	unsigned skipLineCount = openDialog->getSkippedLinesCount();
 	bool showLabelsIn2D = openDialog->showLabelsIn2D();
@@ -441,6 +470,7 @@ CC_FILE_ERROR AsciiFilter::loadFile(const QString& filename,
 											container,
 											openSequence,
 											separator,
+											commaAsDecimal,
 											approximateNumberOfLines,
 											fileSize,
 											maxCloudSize,
@@ -452,7 +482,7 @@ CC_FILE_ERROR AsciiFilter::loadFile(const QString& filename,
 struct cloudAttributesDescriptor
 {
 	ccPointCloud* cloud;
-	static const unsigned c_attribCount = 13;
+	static const unsigned c_attribCount = 14;
 	union
 	{
 		struct{	int xCoordIndex;
@@ -464,6 +494,7 @@ struct cloudAttributesDescriptor
 				int redIndex;
 				int greenIndex;
 				int blueIndex;
+				int alphaIndex;
 				int iRgbaIndex;
 				int fRgbaIndex;
 				int greyIndex;
@@ -475,7 +506,7 @@ struct cloudAttributesDescriptor
 	std::vector<CCLib::ScalarField*> scalarFields;
 	bool hasNorms;
 	bool hasRGBColors;
-	bool hasFloatRGBColors[3];
+	bool hasFloatRGBColors[4];
 
 	cloudAttributesDescriptor()
 	{
@@ -491,7 +522,7 @@ struct cloudAttributesDescriptor
 		}
 		hasNorms = false;
 		hasRGBColors = false;
-		hasFloatRGBColors[0] = hasFloatRGBColors[1] = hasFloatRGBColors[2] = false;
+		hasFloatRGBColors[0] = hasFloatRGBColors[1] = hasFloatRGBColors[2] = hasFloatRGBColors[3] = false;
 		
 		scalarIndexes.clear();
 		scalarFields.clear();
@@ -520,7 +551,6 @@ void clearStructure(cloudAttributesDescriptor &cloudDesc)
 cloudAttributesDescriptor prepareCloud(	const AsciiOpenDlg::Sequence &openSequence,
 										unsigned numberOfPoints,
 										int& maxIndex,
-										char separator,
 										unsigned step = 1)
 {
 	ccPointCloud* cloud = new ccPointCloud();
@@ -604,7 +634,7 @@ cloudAttributesDescriptor prepareCloud(	const AsciiOpenDlg::Sequence &openSequen
 				}
 				else
 				{
-					sfName.replace('_',separator);
+					sfName.replace('_', ' ');
 				}
 
 				ccScalarField* sf = new ccScalarField(qPrintable(sfName));
@@ -664,6 +694,20 @@ cloudAttributesDescriptor prepareCloud(	const AsciiOpenDlg::Sequence &openSequen
 				ccLog::Warning("Failed to allocate memory for colors! (skipped)");
 			}
 			break;
+		case ASCII_OPEN_DLG_Af:
+			cloudDesc.hasFloatRGBColors[3] = true;
+		case ASCII_OPEN_DLG_A:
+			if (cloud->reserveTheRGBTable())
+			{
+				cloudDesc.alphaIndex = i;
+				cloudDesc.hasRGBColors = true;
+				cloud->showColors(true);
+			}
+			else
+			{
+				ccLog::Warning("Failed to allocate memory for colors! (skipped)");
+			}
+			break;
 		case ASCII_OPEN_DLG_RGB32i:
 		case ASCII_OPEN_DLG_RGB32f:
 			if (cloud->reserveTheRGBTable())
@@ -713,6 +757,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 															ccHObject& container,
 															const AsciiOpenDlg::Sequence& openSequence,
 															char separator,
+															bool commaAsDecimal,
 															unsigned approximateNumberOfLines,
 															qint64 fileSize,
 															unsigned maxCloudSize,
@@ -728,7 +773,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 
 	//we initialize the loading accelerator structure and point cloud
 	int maxPartIndex = -1;
-	cloudAttributesDescriptor cloudDesc = prepareCloud(openSequence, cloudChunkSize, maxPartIndex, separator, chunkRank);
+	cloudAttributesDescriptor cloudDesc = prepareCloud(openSequence, cloudChunkSize, maxPartIndex, chunkRank);
 
 	if (!cloudDesc.cloud)
 	{
@@ -775,7 +820,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 	CCVector3d P(0, 0, 0);
 	CCVector3d Pshift(0, 0, 0);
 	CCVector3 N(0, 0, 0);
-	ccColor::Rgb col;
+	ccColor::Rgba col(0, 0, 0, 255);
 	bool preserveCoordinateShift = true;
 
 	//other useful variables
@@ -784,6 +829,8 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 
 	CC_FILE_ERROR result = CC_FERR_NO_ERROR;
 
+	QLocale locale(commaAsDecimal ? QLocale::French : QLocale::English);
+
 	//main process
 	unsigned nextLimit = /*cloudChunkPos+*/cloudChunkSize;
 	while (true)
@@ -856,7 +903,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 				//and create new one
 				cloudChunkPos = pointsRead;
 				cloudChunkSize = std::min(maxCloudSize, approximateNumberOfLines - cloudChunkPos);
-				cloudDesc = prepareCloud(openSequence, cloudChunkSize, maxPartIndex, separator, ++chunkRank);
+				cloudDesc = prepareCloud(openSequence, cloudChunkSize, maxPartIndex, ++chunkRank);
 				if (!cloudDesc.cloud)
 				{
 					ccLog::Error("Not enough memory! Process stopped ...");
@@ -879,7 +926,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 		}
 
 		//we split current line
-		QStringList parts = currentLine.split(separator, QString::SkipEmptyParts);
+		QStringList parts = currentLine.simplified().split(separator, QString::SkipEmptyParts);
 
 		int nParts = parts.size();
 		if (nParts > maxPartIndex) //fake loop for easy break
@@ -891,7 +938,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 				bool ok = true;
 				if (cloudDesc.xCoordIndex >= 0)
 				{
-					P.x = parts[cloudDesc.xCoordIndex].toDouble(&ok);
+					P.x = locale.toDouble(parts[cloudDesc.xCoordIndex], &ok);
 					if (!ok)
 					{
 						break;
@@ -899,7 +946,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 				}
 				if (cloudDesc.yCoordIndex >= 0)
 				{
-					P.y = parts[cloudDesc.yCoordIndex].toDouble(&ok);
+					P.y = locale.toDouble(parts[cloudDesc.yCoordIndex], &ok);
 					if (!ok)
 					{
 						break;
@@ -907,7 +954,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 				}
 				if (cloudDesc.zCoordIndex >= 0)
 				{
-					P.z = parts[cloudDesc.zCoordIndex].toDouble(&ok);
+					P.z = locale.toDouble(parts[cloudDesc.zCoordIndex], &ok);
 					if (!ok)
 					{
 						break;
@@ -943,11 +990,11 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 			if (cloudDesc.hasNorms)
 			{
 				if (cloudDesc.xNormIndex >= 0)
-					N.x = static_cast<PointCoordinateType>(parts[cloudDesc.xNormIndex].toDouble());
+					N.x = static_cast<PointCoordinateType>(locale.toDouble(parts[cloudDesc.xNormIndex]));
 				if (cloudDesc.yNormIndex >= 0)
-					N.y = static_cast<PointCoordinateType>(parts[cloudDesc.yNormIndex].toDouble());
+					N.y = static_cast<PointCoordinateType>(locale.toDouble(parts[cloudDesc.yNormIndex]));
 				if (cloudDesc.zNormIndex >= 0)
-					N.z = static_cast<PointCoordinateType>(parts[cloudDesc.zNormIndex].toDouble());
+					N.z = static_cast<PointCoordinateType>(locale.toDouble(parts[cloudDesc.zNormIndex]));
 				cloudDesc.cloud->addNorm(N);
 			}
 
@@ -956,44 +1003,52 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 			{
 				if (cloudDesc.iRgbaIndex >= 0)
 				{
-					const uint32_t rgb = parts[cloudDesc.iRgbaIndex].toInt();
-					col.r = ((rgb >> 16) & 0x0000ff);
-					col.g = ((rgb >>  8) & 0x0000ff);
-					col.b = ((rgb      ) & 0x0000ff);
+					const uint32_t rgba = parts[cloudDesc.iRgbaIndex].toInt();
+					col.a = ((rgba >> 24) & 0x0000ff);
+					col.r = ((rgba >> 16) & 0x0000ff);
+					col.g = ((rgba >>  8) & 0x0000ff);
+					col.b = ((rgba      ) & 0x0000ff);
 
 				}
 				else if (cloudDesc.fRgbaIndex >= 0)
 				{
-					const float rgbf = parts[cloudDesc.fRgbaIndex].toFloat();
-					const uint32_t rgb = *(reinterpret_cast<const uint32_t *>(&rgbf));
-					col.r = ((rgb >> 16) & 0x0000ff);
-					col.g = ((rgb >>  8) & 0x0000ff);
-					col.b = ((rgb      ) & 0x0000ff);
+					const float rgbaf = locale.toFloat(parts[cloudDesc.fRgbaIndex]);
+					const uint32_t rgba = *(reinterpret_cast<const uint32_t *>(&rgbaf));
+					col.a = ((rgba >> 24) & 0x0000ff);
+					col.r = ((rgba >> 16) & 0x0000ff);
+					col.g = ((rgba >>  8) & 0x0000ff);
+					col.b = ((rgba      ) & 0x0000ff);
 				}
 				else
 				{
 					if (cloudDesc.redIndex >= 0)
 					{
 						float multiplier = cloudDesc.hasFloatRGBColors[0] ? static_cast<float>(ccColor::MAX) : 1.0f;
-						col.r = static_cast<ColorCompType>(parts[cloudDesc.redIndex].toFloat() * multiplier);
+						col.r = static_cast<ColorCompType>(locale.toFloat(parts[cloudDesc.redIndex]) * multiplier);
 					}
 					if (cloudDesc.greenIndex >= 0)
 					{
 						float multiplier = cloudDesc.hasFloatRGBColors[1] ? static_cast<float>(ccColor::MAX) : 1.0f;
-						col.g = static_cast<ColorCompType>(parts[cloudDesc.greenIndex].toFloat() * multiplier);
+						col.g = static_cast<ColorCompType>(locale.toFloat(parts[cloudDesc.greenIndex]) * multiplier);
 					}
 					if (cloudDesc.blueIndex >= 0)
 					{
 						float multiplier = cloudDesc.hasFloatRGBColors[2] ? static_cast<float>(ccColor::MAX) : 1.0f;
-						col.b = static_cast<ColorCompType>(parts[cloudDesc.blueIndex].toFloat() * multiplier);
+						col.b = static_cast<ColorCompType>(locale.toFloat(parts[cloudDesc.blueIndex]) * multiplier);
+					}
+					if (cloudDesc.alphaIndex >= 0)
+					{
+						float multiplier = cloudDesc.hasFloatRGBColors[3] ? static_cast<float>(ccColor::MAX) : 1.0f;
+						col.a = static_cast<ColorCompType>(locale.toFloat(parts[cloudDesc.alphaIndex]) * multiplier);
 					}
 				}
-				cloudDesc.cloud->addRGBColor(col);
+				cloudDesc.cloud->addColor(col);
 			}
 			else if (cloudDesc.greyIndex >= 0)
 			{
-				col.r = col.r = col.b = static_cast<ColorCompType>(parts[cloudDesc.greyIndex].toInt());
-				cloudDesc.cloud->addRGBColor(col);
+				col.r = col.g = col.b = static_cast<ColorCompType>(parts[cloudDesc.greyIndex].toInt());
+				col.a = ccColor::MAX;
+				cloudDesc.cloud->addColor(col);
 			}
 
 			//Scalar distance
@@ -1001,7 +1056,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 			{
 				for (size_t j = 0; j < cloudDesc.scalarIndexes.size(); ++j)
 				{
-					D = static_cast<ScalarType>(parts[cloudDesc.scalarIndexes[j]].toDouble());
+					D = static_cast<ScalarType>(locale.toDouble(parts[cloudDesc.scalarIndexes[j]]));
 					cloudDesc.scalarFields[j]->emplace_back(D);
 				}
 			}
@@ -1010,7 +1065,7 @@ CC_FILE_ERROR AsciiFilter::loadCloudFromFormatedAsciiFile(	const QString& filena
 			if (cloudDesc.labelIndex >= 0)
 			{
 				cc2DLabel* label = new cc2DLabel();
-				label->addPoint(cloudDesc.cloud, cloudDesc.cloud->size() - 1);
+				label->addPickedPoint(cloudDesc.cloud, cloudDesc.cloud->size() - 1);
 				label->setName(parts[cloudDesc.labelIndex]);
 				label->setDisplayedIn2D(showLabelsIn2D);
 				label->displayPointLegend(!showLabelsIn2D);
