@@ -18,41 +18,52 @@
 #include "BinFilter.h"
 
 //Qt
-#include <QMessageBox>
 #include <QApplication>
 #include <QFileInfo>
+#include <QMessageBox>
 #include <QtConcurrentRun>
 
 //qCC_db
+#include <cc2DLabel.h>
+#include <ccCameraSensor.h>
+#include <ccFacet.h>
 #include <ccFlags.h>
 #include <ccGenericPointCloud.h>
-#include <ccPointCloud.h>
-#include <ccProgressDialog.h>
-#include <ccMesh.h>
-#include <ccSubMesh.h>
-#include <ccPolyline.h>
-#include <ccMaterialSet.h>
-#include <cc2DLabel.h>
-#include <ccFacet.h>
-#include <ccSensor.h>
-#include <ccCameraSensor.h>
+#include <ccHObjectCaster.h>
 #include <ccImage.h>
+#include <ccMaterialSet.h>
+#include <ccMesh.h>
+#include <ccPointCloud.h>
+#include <ccPolyline.h>
+#include <ccProgressDialog.h>
 #include <ccScalarField.h>
+#include <ccSensor.h>
+#include <ccSubMesh.h>
 
 //system
+#include <cassert>
+#include <cstring>
 #include <unordered_set>
-#include <assert.h>
-#include <string.h>
+
 #if defined(CC_WINDOWS)
 #include <windows.h>
 #else
-#include <time.h>
+#include <ctime>
 #include <unistd.h>
 #endif
 
-bool BinFilter::canLoadExtension(const QString& upperCaseExt) const
+
+BinFilter::BinFilter()
+	: FileIOFilter( {
+					"_CloudCompare BIN Filter",
+					1.0f,	// priority
+					QStringList{ "bin" },
+					"bin",
+					QStringList{ GetFileFilter() },
+					QStringList{ GetFileFilter() },
+					Import | Export | BuiltIn
+					} )	
 {
-	return (upperCaseExt == "BIN");
 }
 
 bool BinFilter::canSave(CC_CLASS_ENUM type, bool& multiple, bool& exclusive) const
@@ -72,6 +83,7 @@ bool BinFilter::canSave(CC_CLASS_ENUM type, bool& multiple, bool& exclusive) con
 	case CC_TYPES::NORMALS_ARRAY:
 	case CC_TYPES::NORMAL_INDEXES_ARRAY:
 	case CC_TYPES::RGB_COLOR_ARRAY:
+	case CC_TYPES::RGBA_COLOR_ARRAY:
 	case CC_TYPES::TEX_COORDS_ARRAY:
 	case CC_TYPES::LABEL_2D:
 	case CC_TYPES::TRANS_BUFFER:
@@ -133,9 +145,9 @@ static int ReadEntityHeader(QFile& in, unsigned &numberOfPoints, HeaderFlags& he
 	return 0;
 }
 
-static QFile* s_file = 0;
+static QFile* s_file = nullptr;
 static int s_flags = 0;
-static ccHObject* s_container = 0;
+static ccHObject* s_container = nullptr;
 
 CC_FILE_ERROR _LoadFileV2()
 {
@@ -156,7 +168,7 @@ CC_FILE_ERROR BinFilter::saveToFile(ccHObject* root, const QString& filename, co
 	if (!out.open(QIODevice::WriteOnly))
 		return CC_FERR_WRITING;
 
-	QScopedPointer<ccProgressDialog> pDlg(0);
+	QScopedPointer<ccProgressDialog> pDlg(nullptr);
 	if (parameters.parentWidget)
 	{
 		pDlg.reset(new ccProgressDialog(false, parameters.parentWidget));
@@ -187,8 +199,8 @@ CC_FILE_ERROR BinFilter::saveToFile(ccHObject* root, const QString& filename, co
 		QApplication::processEvents();
 	}
 	
-	s_file = 0;
-	s_container = 0;
+	s_file = nullptr;
+	s_container = nullptr;
 
 	CC_FILE_ERROR result = future.result();
 
@@ -215,7 +227,7 @@ CC_FILE_ERROR BinFilter::SaveFileV2(QFile& out, ccHObject* object)
 		if (sizeof(ScalarType) == 4)
 			flags |= static_cast<char>(ccSerializableObject::DF_SCALAR_VAL_32_BITS);
 		assert(flags <= 8);
-		firstBytes[3] = 48+flags; //48 = ASCII("0")
+		firstBytes[3] = 48 + flags; //48 = ASCII("0")
 	}
 
 	if (out.write(firstBytes,4) < 0)
@@ -223,7 +235,7 @@ CC_FILE_ERROR BinFilter::SaveFileV2(QFile& out, ccHObject* object)
 
 	// Current BIN file version
 	uint32_t binVersion_u32 = static_cast<uint32_t>(ccObject::GetCurrentDBVersion());
-	if (out.write((char*)&binVersion_u32,4) < 0)
+	if (out.write((char*)&binVersion_u32, 4) < 0)
 		return CC_FERR_WRITING;
 
 	CC_FILE_ERROR result = CC_FERR_NO_ERROR;
@@ -274,10 +286,13 @@ CC_FILE_ERROR BinFilter::SaveFileV2(QFile& out, ccHObject* object)
 		else if (currentObject->isA(CC_TYPES::LABEL_2D))
 		{
 			cc2DLabel* label = static_cast<cc2DLabel*>(currentObject);
-			for (unsigned i=0;i<label->size();++i)
+			for (unsigned i = 0; i < label->size(); ++i)
 			{
-				const cc2DLabel::PickedPoint& pp = label->getPoint(i);
-				dependencies.insert(pp.cloud);
+				const cc2DLabel::PickedPoint& pp = label->getPickedPoint(i);
+				if (pp._cloud)
+					dependencies.insert(pp._cloud);
+				else if (pp._mesh)
+					dependencies.insert(pp._mesh);
 			}
 		}
 		else if (currentObject->isA(CC_TYPES::FACET))
@@ -303,14 +318,14 @@ CC_FILE_ERROR BinFilter::SaveFileV2(QFile& out, ccHObject* object)
 		{
 			if (!object->find((*it)->getUniqueID()))
 			{
-				ccLog::Warning(QString("[BIN] Dependency broken: entity '%1' must also be in selection in order to save '%2'").arg((*it)->getName(),currentObject->getName()));
+				ccLog::Warning(QString("[BIN] Dependency broken: entity '%1' must also be in selection in order to save '%2'").arg((*it)->getName(), currentObject->getName()));
 				result = CC_FERR_BROKEN_DEPENDENCY_ERROR;
 			}
 		}
 		//release some memory...
 		dependencies.clear();
 
-		for (unsigned i=0; i<currentObject->getChildrenNumber(); ++i)
+		for (unsigned i = 0; i < currentObject->getChildrenNumber(); ++i)
 			toCheck.push_back(currentObject->getChild(i));
 	}
 
@@ -358,18 +373,18 @@ CC_FILE_ERROR BinFilter::loadFile(const QString& filename, ccHObject& container,
 
 		//if (sizeof(PointCoordinateType) == 8 && strncmp((char*)&firstBytes,"CCB3",4) != 0)
 		//{
-		//	QMessageBox::information(0, QString("Wrong version"), QString("This file has been generated with the standard 'float' version!\nAt this time it cannot be read with the 'double' version."),QMessageBox::Ok);
+		//	QMessageBox::information(nullptr, QString("Wrong version"), QString("This file has been generated with the standard 'float' version!\nAt this time it cannot be read with the 'double' version."),QMessageBox::Ok);
 		//	return CC_FERR_WRONG_FILE_TYPE;
 		//}
 		//else if (sizeof(PointCoordinateType) == 4 && strncmp((char*)&firstBytes,"CCB2",4) != 0)
 		//{
-		//	QMessageBox::information(0, QString("Wrong version"), QString("This file has been generated with the new 'double' version!\nAt this time it cannot be read with the standard 'float' version."),QMessageBox::Ok);
+		//	QMessageBox::information(nullptr, QString("Wrong version"), QString("This file has been generated with the new 'double' version!\nAt this time it cannot be read with the standard 'float' version."),QMessageBox::Ok);
 		//	return CC_FERR_WRONG_FILE_TYPE;
 		//}
 
 		if (parameters.alwaysDisplayLoadDialog)
 		{
-			QScopedPointer<ccProgressDialog> pDlg(0);
+			QScopedPointer<ccProgressDialog> pDlg(nullptr);
 			if (parameters.parentWidget)
 			{
 				pDlg.reset(new ccProgressDialog(false, parameters.parentWidget));
@@ -401,8 +416,8 @@ CC_FILE_ERROR BinFilter::loadFile(const QString& filename, ccHObject& container,
 				QApplication::processEvents();
 			}
 	
-			s_file = 0;
-			s_container = 0;
+			s_file = nullptr;
+			s_container = nullptr;
 
 			return future.result();
 		}
@@ -418,73 +433,41 @@ inline bool Match(ccHObject* object, unsigned uniqueID, CC_CLASS_ENUM expectedTy
 	return object && object->getUniqueID() == uniqueID && object->isKindOf(expectedType);
 }
 
-ccHObject* FindRobust(ccHObject* root, ccHObject* source, unsigned uniqueID, CC_CLASS_ENUM expectedType)
+ccHObject* FindRobust(ccHObject* root, ccHObject* source, const ccObject::LoadedIDMap& oldToNewIDMap, unsigned oldUniqueID, CC_CLASS_ENUM expectedType)
 {
-	if (source)
+	ccObject::LoadedIDMap::const_iterator it = oldToNewIDMap.find(oldUniqueID);
+	while (it != oldToNewIDMap.end() && it.key() == oldUniqueID)
 	{
-		//1st test the parent
-		ccHObject* parent = source->getParent();
-		if (Match(parent,uniqueID,expectedType))
-			return parent;
+		unsigned uniqueID = it.value();
+		++it;
 
-		//now test the children
-		for (unsigned i=0; i<source->getChildrenNumber(); ++i)
+		if (source)
 		{
-			ccHObject* child = source->getChild(i);
-			if (Match(child,uniqueID,expectedType))
-				return child;
-		}
-	}
+			//1st test the parent
+			ccHObject* parent = source->getParent();
+			if (Match(parent, uniqueID, expectedType))
+				return parent;
 
-	//now test the whole DB
-	ccHObject* foundObject = 0;
-	{
-		ccHObject::Container hiddenEntities;
-		while (true)
-		{
-			ccHObject* object = root->find(uniqueID);
-			if (object)
+			//now test the children
+			for (unsigned i = 0; i < source->getChildrenNumber(); ++i)
 			{
-				//if we found an object, we must also test its type!
-				if (object->isKindOf(expectedType))
-				{
-					foundObject = object;
-					break;
-				}
-
-				/********* BIG UGLY RECOVERY TRICK *********/
-
-				//if the type doesn't match, we may be in front of a degenerate case :(
-				//we'll look if there's other entities with the same ID!!!
-				try
-				{
-					hiddenEntities.push_back(object);
-					//we temporarily 'hide' this entity by removing its unique ID
-					object->setUniqueID(0);
-				}
-				catch (const std::bad_alloc&)
-				{
-					//not enough memory?! Stop this process (anyway it's already a degenerate case ;)
-					break;
-				}
-
-				/********* BIG UGLY RECOVERY TRICK *********/
-			}
-			else
-			{
-				break;
+				ccHObject* child = source->getChild(i);
+				if (Match(child, uniqueID, expectedType))
+					return child;
 			}
 		}
-		//restore original IDs (if necessary)
-		while (!hiddenEntities.empty())
+
+		//now test the whole DB
+		ccHObject* object = root->find(uniqueID);
+		//if we've found an object, we must also test its type!
+		if (object && object->isKindOf(expectedType))
 		{
-			hiddenEntities.back()->setUniqueID(uniqueID);
-			hiddenEntities.pop_back();
+			return object;
 		}
 	}
 
 	//no entity found!
-	return foundObject;
+	return nullptr;
 }
 
 CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
@@ -502,25 +485,37 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 	QString scalarFormat = ((flags & ccSerializableObject::DF_SCALAR_VAL_32_BITS) ? "float" : "double");
 	ccLog::Print(QString("[BIN] Version %1.%2 (coords: %3 / scalar: %4)").arg(binVersion / 10).arg(binVersion % 10).arg(coordsFormat).arg(scalarFormat));
 
+	if (ccObject::GetCurrentDBVersion() < binVersion)
+	{
+		ccLog::Error("This version of CloudCompare is too old and can't load this file, sorry");
+		return CC_FERR_CONSOLE_ERROR;
+	}
+
 	//we keep track of the last unique ID before load
 	unsigned lastUniqueIDBeforeLoad = ccObject::GetLastUniqueID();
 
 	//we read first entity type
 	CC_CLASS_ENUM classID = ccObject::ReadClassIDFromFile(in, static_cast<short>(binVersion));
 	if (classID == CC_TYPES::OBJECT)
+	{
 		return CC_FERR_CONSOLE_ERROR;
+	}
 
+	//call the CC object factory
 	ccHObject* root = ccHObject::New(classID);
-
 	if (!root)
+	{
 		return CC_FERR_MALFORMED_FILE;
+	}
+
+	ccObject::LoadedIDMap oldToNewIDMap;
 
 	if (classID == CC_TYPES::CUSTOM_H_OBJECT)
 	{
 		// store seeking position
 		size_t original_pos = in.pos();
 		// we need to load it as plain ccCustomHobject
-		root->fromFileNoChildren(in, static_cast<short>(binVersion), flags); // this will load it
+		root->fromFileNoChildren(in, static_cast<short>(binVersion), flags, oldToNewIDMap); // this will load it
 		in.seek(original_pos); // reseek back the file
 
 		QString classId = root->getMetaData("class_name").toString();
@@ -534,10 +529,10 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 			return CC_FERR_FILE_WAS_WRITTEN_BY_UNKNOWN_PLUGIN;
 	}
 
-	if (!root->fromFile(in, static_cast<short>(binVersion), flags))
+	if (!root->fromFile(in, static_cast<short>(binVersion), flags, oldToNewIDMap))
 	{
-		//DGM: can't delete it, too dangerous (bad pointers ;)
-		//delete root;
+		//delete root; //DGM: can't delete it, too dangerous (bad pointers ;)
+		ccLog::Error(QString("Failed to read file (file position: %1 / %2").arg(in.pos()).arg(in.size()));
 		return CC_FERR_CONSOLE_ERROR;
 	}
 
@@ -545,7 +540,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 
 	//re-link objects (and check errors)
 	bool checkErrors = true;
-	ccHObject* orphans = new ccHObject("Orphans (CORRUPTED FILE)");;
+	ccHObject* orphans = new ccHObject("Orphans (CORRUPTED FILE)");
 	ccHObject::Container toCheck;
 	toCheck.push_back(root);
 	while (!toCheck.empty())
@@ -573,7 +568,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				intptr_t meshID = (intptr_t)subMesh->getAssociatedMesh();
 				if (meshID > 0)
 				{
-					ccHObject* mesh = FindRobust(root, subMesh, static_cast<unsigned>(meshID), CC_TYPES::MESH);
+					ccHObject* mesh = FindRobust(root, subMesh, oldToNewIDMap, meshID, CC_TYPES::MESH);
 					if (mesh)
 					{
 						subMesh->setAssociatedMesh(ccHObjectCaster::ToMesh(mesh), false); //'false' because previous mesh is not null (= real mesh ID)!!!
@@ -588,7 +583,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 						}
 						else
 						{
-							subMesh->setAssociatedMesh(0, false); //'false' because previous mesh is not null (= real mesh ID)!!!
+							subMesh->setAssociatedMesh(nullptr, false); //'false' because previous mesh is not null (= real mesh ID)!!!
 							//DGM: can't delete it, too dangerous (bad pointers ;)
 							//delete subMesh;
 							ccLog::Warning(QString("[BIN] Couldn't find associated mesh (ID=%1) for sub-mesh '%2' in the file!").arg(meshID).arg(subMesh->getName()));
@@ -606,7 +601,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				intptr_t cloudID = (intptr_t)mesh->getAssociatedCloud();
 				if (cloudID > 0)
 				{
-					ccHObject* cloud = FindRobust(root, mesh, static_cast<unsigned>(cloudID), CC_TYPES::POINT_CLOUD);
+					ccHObject* cloud = FindRobust(root, mesh, oldToNewIDMap, cloudID, CC_TYPES::POINT_CLOUD);
 					if (cloud)
 					{
 						mesh->setAssociatedCloud(ccHObjectCaster::ToGenericPointCloud(cloud));
@@ -614,9 +609,9 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 					else
 					{
 						//we have a problem here ;)
-						mesh->setAssociatedCloud(0);
+						mesh->setAssociatedCloud(nullptr);
 						if (mesh->getMaterialSet())
-							mesh->setMaterialSet(0, false);
+							mesh->setMaterialSet(nullptr, false);
 						//DGM: can't delete it, too dangerous (bad pointers ;)
 						//delete mesh;
 						if (mesh->getParent())
@@ -629,7 +624,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 						{
 							return CC_FERR_MALFORMED_FILE;
 						}
-						mesh = 0;
+						currentObject = mesh = nullptr;
 					}
 				}
 				else
@@ -649,21 +644,20 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 						else
 						{
 							//we'll try to live with that
-							currentObject = 0;
 							delete mesh;
 						}
 					}
-					mesh = 0;
+					currentObject = mesh = nullptr;
 				}
 
 				if (mesh)
 				{
 					//materials
-					ccHObject* materials = 0;
+					ccHObject* materials = nullptr;
 					intptr_t matSetID = (intptr_t)mesh->getMaterialSet();
 					if (matSetID > 0)
 					{
-						materials = FindRobust(root, mesh, static_cast<unsigned>(matSetID), CC_TYPES::MATERIAL_SET);
+						materials = FindRobust(root, mesh, oldToNewIDMap, matSetID, CC_TYPES::MATERIAL_SET);
 						if (materials)
 						{
 							mesh->setMaterialSet(static_cast<ccMaterialSet*>(materials), false);
@@ -671,7 +665,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 						else
 						{
 							//we have a (less severe) problem here ;)
-							mesh->setMaterialSet(0, false);
+							mesh->setMaterialSet(nullptr, false);
 							mesh->showMaterials(false);
 							ccLog::Warning(QString("[BIN] Couldn't find shared materials set (ID=%1) for mesh '%2' in the file!").arg(matSetID).arg(mesh->getName()));
 							result = CC_FERR_BROKEN_DEPENDENCY_ERROR;
@@ -679,15 +673,15 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 							//add it to the 'orphans' set
 							if (materials)
 								orphans->addChild(materials);
-							materials = 0;
+							materials = nullptr;
 						}
 					}
 					//per-triangle normals
-					ccHObject* triNormsTable = 0;
+					ccHObject* triNormsTable = nullptr;
 					intptr_t triNormsTableID = (intptr_t)mesh->getTriNormsTable();
 					if (triNormsTableID > 0)
 					{
-						triNormsTable = FindRobust(root, mesh, static_cast<unsigned>(triNormsTableID), CC_TYPES::NORMAL_INDEXES_ARRAY);
+						triNormsTable = FindRobust(root, mesh, oldToNewIDMap, triNormsTableID, CC_TYPES::NORMAL_INDEXES_ARRAY);
 						if (triNormsTable)
 						{
 							mesh->setTriNormsTable(static_cast<NormsIndexesTableType*>(triNormsTable), false);
@@ -695,7 +689,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 						else
 						{
 							//we have a (less severe) problem here ;)
-							mesh->setTriNormsTable(0, false);
+							mesh->setTriNormsTable(nullptr, false);
 							mesh->showTriNorms(false);
 							ccLog::Warning(QString("[BIN] Couldn't find shared normals (ID=%1) for mesh '%2' in the file!").arg(triNormsTableID).arg(mesh->getName()));
 							result = CC_FERR_BROKEN_DEPENDENCY_ERROR;
@@ -703,15 +697,15 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 							//add it to the 'orphans' set
 							if (triNormsTable)
 								orphans->addChild(triNormsTable);
-							triNormsTable = 0;
+							triNormsTable = nullptr;
 						}
 					}
 					//per-triangle texture coordinates
-					ccHObject* texCoordsTable = 0;
+					ccHObject* texCoordsTable = nullptr;
 					intptr_t texCoordArrayID = (intptr_t)mesh->getTexCoordinatesTable();
 					if (texCoordArrayID > 0)
 					{
-						texCoordsTable = FindRobust(root, mesh, static_cast<unsigned>(texCoordArrayID), CC_TYPES::TEX_COORDS_ARRAY);
+						texCoordsTable = FindRobust(root, mesh, oldToNewIDMap, texCoordArrayID, CC_TYPES::TEX_COORDS_ARRAY);
 						if (texCoordsTable)
 						{
 							mesh->setTexCoordinatesTable(static_cast<TextureCoordsContainer*>(texCoordsTable), false);
@@ -719,14 +713,14 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 						else
 						{
 							//we have a (less severe) problem here ;)
-							mesh->setTexCoordinatesTable(0, false);
+							mesh->setTexCoordinatesTable(nullptr, false);
 							ccLog::Warning(QString("[BIN] Couldn't find shared texture coordinates (ID=%1) for mesh '%2' in the file!").arg(texCoordArrayID).arg(mesh->getName()));
 							result = CC_FERR_BROKEN_DEPENDENCY_ERROR;
 
 							//add it to the 'orphans' set
 							if (texCoordsTable)
 								orphans->addChild(texCoordsTable);
-							texCoordsTable = 0;
+							texCoordsTable = nullptr;
 						}
 					}
 
@@ -764,12 +758,12 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 								}
 
 								//delete corrupted mesh
-								mesh->setMaterialSet(0, false);
-								mesh->setTriNormsTable(0, false);
-								mesh->setTexCoordinatesTable(0, false);
+								mesh->setMaterialSet(nullptr, false);
+								mesh->setTriNormsTable(nullptr, false);
+								mesh->setTexCoordinatesTable(nullptr, false);
 								if (mesh->getParent())
 									mesh->getParent()->removeChild(mesh);
-								mesh = 0;
+								currentObject = mesh = nullptr;
 
 								break;
 							}
@@ -782,7 +776,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 		{
 			ccPolyline* poly = ccHObjectCaster::ToPolyline(currentObject);
 			intptr_t cloudID = (intptr_t)poly->getAssociatedCloud();
-			ccHObject* cloud = FindRobust(root, poly, static_cast<unsigned>(cloudID), CC_TYPES::POINT_CLOUD);
+			ccHObject* cloud = FindRobust(root, poly, oldToNewIDMap, cloudID, CC_TYPES::POINT_CLOUD);
 			if (cloud)
 			{
 				poly->setAssociatedCloud(ccHObjectCaster::ToGenericPointCloud(cloud));
@@ -790,9 +784,10 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 			else
 			{
 				//we have a problem here ;)
-				poly->setAssociatedCloud(0);
+				poly->setAssociatedCloud(nullptr);
 				//DGM: can't delete it, too dangerous (bad pointers ;)
 				//delete root;
+				currentObject = nullptr;
 				ccLog::Warning(QString("[BIN] Couldn't find vertices (ID=%1) for polyline '%2' in the file!").arg(cloudID).arg(poly->getName()));
 				return CC_FERR_MALFORMED_FILE;
 			}
@@ -803,7 +798,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 			intptr_t bufferID = (intptr_t)sensor->getPositions();
 			if (bufferID > 0)
 			{
-				ccHObject* buffer = FindRobust(root, sensor, static_cast<unsigned>(bufferID), CC_TYPES::TRANS_BUFFER);
+				ccHObject* buffer = FindRobust(root, sensor, oldToNewIDMap, bufferID, CC_TYPES::TRANS_BUFFER);
 				if (buffer)
 				{
 					sensor->setPositions(ccHObjectCaster::ToTransBuffer(buffer));
@@ -811,11 +806,12 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				else
 				{
 					//we have a problem here ;)
-					sensor->setPositions(0);
+					sensor->setPositions(nullptr);
 
 					//DGM: can't delete it, too dangerous (bad pointers ;)
 					//delete root;
 
+					currentObject = nullptr;
 					ccLog::Warning(QString("[BIN] Couldn't find trans. buffer (ID=%1) for sensor '%2' in the file!").arg(bufferID).arg(sensor->getName()));
 
 					//positions are optional, so we can simply set them to nullptr and go ahead, we do not need to return.
@@ -828,27 +824,55 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 			cc2DLabel* label = ccHObjectCaster::To2DLabel(currentObject);
 			std::vector<cc2DLabel::PickedPoint> correctedPickedPoints;
 			//we must check all label 'points'!
-			for (unsigned i = 0; i < label->size(); ++i)
+			bool invalidLabel = false;
+			for (unsigned i = 0; !invalidLabel && i < label->size(); ++i)
 			{
-				const cc2DLabel::PickedPoint& pp = label->getPoint(i);
-				intptr_t cloudID = (intptr_t)pp.cloud;
-				ccHObject* cloud = FindRobust(root, label, static_cast<unsigned>(cloudID), CC_TYPES::POINT_CLOUD);
-				if (cloud)
+				const cc2DLabel::PickedPoint& pp = label->getPickedPoint(i);
+				if (pp._cloud)
 				{
-					ccGenericPointCloud* genCloud = ccHObjectCaster::ToGenericPointCloud(cloud);
-					assert(genCloud->size() > pp.index);
-					correctedPickedPoints.push_back(cc2DLabel::PickedPoint(genCloud, pp.index));
+					intptr_t cloudID = (intptr_t)pp._cloud;
+					ccHObject* cloud = FindRobust(root, label, oldToNewIDMap, cloudID, CC_TYPES::POINT_CLOUD);
+					if (cloud)
+					{
+						ccGenericPointCloud* genCloud = ccHObjectCaster::ToGenericPointCloud(cloud);
+						assert(genCloud->size() > pp.index);
+						correctedPickedPoints.push_back(cc2DLabel::PickedPoint(genCloud, pp.index, pp.entityCenterPoint));
+					}
+					else
+					{
+						//we have a problem here ;)
+						ccLog::Warning(QString("[BIN] Couldn't find cloud (ID=%1) associated to label ID=%2 in the file!").arg(cloudID).arg(label->getUniqueID())); //we can't call getName as it relies on the cloud/mesh pointers!
+						if (label->getParent())
+							label->getParent()->removeChild(label);
+						//DGM: can't delete it, too dangerous (bad pointers ;)
+						//delete label;
+						currentObject = label = nullptr;
+						invalidLabel = true;
+						break;
+					}
 				}
-				else
+				else if (pp._mesh)
 				{
-					//we have a problem here ;)
-					ccLog::Warning(QString("[BIN] Couldn't find cloud (ID=%1) associated to label '%2' in the file!").arg(cloudID).arg(label->getName()));
-					if (label->getParent())
-						label->getParent()->removeChild(label);
-					//DGM: can't delete it, too dangerous (bad pointers ;)
-					//delete label;
-					label = 0;
-					break;
+					intptr_t meshID = (intptr_t)pp._mesh;
+					ccHObject* mesh = FindRobust(root, label, oldToNewIDMap, meshID, CC_TYPES::MESH);
+					if (mesh)
+					{
+						ccGenericMesh* genMesh = ccHObjectCaster::ToGenericMesh(mesh);
+						assert(genMesh->size() > pp.index);
+						correctedPickedPoints.push_back(cc2DLabel::PickedPoint(genMesh, pp.index, pp.uv, pp.entityCenterPoint));
+					}
+					else
+					{
+						//we have a problem here ;)
+						ccLog::Warning(QString("[BIN] Couldn't find mesh (ID=%1) associated to label ID=%2 in the file!").arg(meshID).arg(label->getUniqueID())); //we can't call getName as it relies on the cloud/mesh pointers!
+						if (label->getParent())
+							label->getParent()->removeChild(label);
+						//DGM: can't delete it, too dangerous (bad pointers ;)
+						//delete label;
+						currentObject = label = nullptr;
+						invalidLabel = true;
+						break;
+					}
 				}
 			}
 
@@ -858,8 +882,13 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				bool visible = label->isVisible();
 				QString originalName(label->getRawName());
 				label->clear(true);
-				for (unsigned i = 0; i < correctedPickedPoints.size(); ++i)
-					label->addPoint(correctedPickedPoints[i].cloud, correctedPickedPoints[i].index);
+				for (const cc2DLabel::PickedPoint& cpp : correctedPickedPoints)
+				{
+					if (cpp._cloud)
+						label->addPickedPoint(cpp._cloud, cpp.index, cpp.entityCenterPoint);
+					else if (cpp._mesh)
+						label->addPickedPoint(cpp._mesh, cpp.index, cpp.uv, cpp.entityCenterPoint);
+				}
 				label->setVisible(visible);
 				label->setName(originalName);
 			}
@@ -873,7 +902,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				intptr_t cloudID = (intptr_t)facet->getOriginPoints();
 				if (cloudID > 0)
 				{
-					ccHObject* cloud = FindRobust(root, facet, static_cast<unsigned>(cloudID), CC_TYPES::POINT_CLOUD);
+					ccHObject* cloud = FindRobust(root, facet, oldToNewIDMap, cloudID, CC_TYPES::POINT_CLOUD);
 					if (cloud && cloud->isA(CC_TYPES::POINT_CLOUD))
 					{
 						facet->setOriginPoints(ccHObjectCaster::ToPointCloud(cloud));
@@ -881,7 +910,8 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 					else
 					{
 						//we have a problem here ;)
-						facet->setOriginPoints(0);
+						facet->setOriginPoints(nullptr);
+						currentObject = nullptr;
 						ccLog::Warning(QString("[BIN] Couldn't find origin points (ID=%1) for facet '%2' in the file!").arg(cloudID).arg(facet->getName()));
 					}
 				}
@@ -891,7 +921,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				intptr_t cloudID = (intptr_t)facet->getContourVertices();
 				if (cloudID > 0)
 				{
-					ccHObject* cloud = FindRobust(root, facet, static_cast<unsigned>(cloudID), CC_TYPES::POINT_CLOUD);
+					ccHObject* cloud = FindRobust(root, facet, oldToNewIDMap, cloudID, CC_TYPES::POINT_CLOUD);
 					if (cloud)
 					{
 						facet->setContourVertices(ccHObjectCaster::ToPointCloud(cloud));
@@ -899,7 +929,8 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 					else
 					{
 						//we have a problem here ;)
-						facet->setContourVertices(0);
+						facet->setContourVertices(nullptr);
+						currentObject = nullptr;
 						ccLog::Warning(QString("[BIN] Couldn't find contour points (ID=%1) for facet '%2' in the file!").arg(cloudID).arg(facet->getName()));
 					}
 				}
@@ -909,7 +940,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				intptr_t polyID = (intptr_t)facet->getContour();
 				if (polyID > 0)
 				{
-					ccHObject* poly = FindRobust(root, facet, static_cast<unsigned>(polyID), CC_TYPES::POLY_LINE);
+					ccHObject* poly = FindRobust(root, facet, oldToNewIDMap, polyID, CC_TYPES::POLY_LINE);
 					if (poly)
 					{
 						facet->setContour(ccHObjectCaster::ToPolyline(poly));
@@ -917,7 +948,8 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 					else
 					{
 						//we have a problem here ;)
-						facet->setContourVertices(0);
+						facet->setContourVertices(nullptr);
+						currentObject = nullptr;
 						ccLog::Warning(QString("[BIN] Couldn't find contour polyline (ID=%1) for facet '%2' in the file!").arg(polyID).arg(facet->getName()));
 					}
 				}
@@ -927,7 +959,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				intptr_t polyID = (intptr_t)facet->getPolygon();
 				if (polyID > 0)
 				{
-					ccHObject* poly = FindRobust(root, facet, static_cast<unsigned>(polyID), CC_TYPES::MESH);
+					ccHObject* poly = FindRobust(root, facet, oldToNewIDMap, polyID, CC_TYPES::MESH);
 					if (poly)
 					{
 						facet->setPolygon(ccHObjectCaster::ToMesh(poly));
@@ -935,7 +967,8 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 					else
 					{
 						//we have a problem here ;)
-						facet->setPolygon(0);
+						facet->setPolygon(nullptr);
+						currentObject = nullptr;
 						ccLog::Warning(QString("[BIN] Couldn't find polygon mesh (ID=%1) for facet '%2' in the file!").arg(polyID).arg(facet->getName()));
 					}
 				}
@@ -948,7 +981,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 			intptr_t sensorID = (intptr_t)image->getAssociatedSensor();
 			if (sensorID > 0)
 			{
-				ccHObject* sensor = FindRobust(root, image, static_cast<unsigned>(sensorID), CC_TYPES::CAMERA_SENSOR);
+				ccHObject* sensor = FindRobust(root, image, oldToNewIDMap, sensorID, CC_TYPES::CAMERA_SENSOR);
 				if (sensor)
 				{
 					image->setAssociatedSensor(ccHObjectCaster::ToCameraSensor(sensor));
@@ -956,11 +989,12 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 				else
 				{
 					//we have a problem here ;)
-					image->setAssociatedSensor(0);
+					image->setAssociatedSensor(nullptr);
 
 					//DGM: can't delete it, too dangerous (bad pointers ;)
 					//delete root;
 
+					currentObject = nullptr;
 					ccLog::Warning(QString("[BIN] Couldn't find camera sensor (ID=%1) for image '%2' in the file!").arg(sensorID).arg(image->getName()));
 					//return CC_FERR_MALFORMED_FILE;
 				}
@@ -986,57 +1020,12 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 		}
 	}
 
-	//check for unique IDs duplicate (yes it happens :-( )
-	{
-		std::unordered_set<unsigned> uniqueIDs;
-		unsigned maxUniqueID = root->findMaxUniqueID_recursive();
-		assert(toCheck.empty());
-		toCheck.push_back(root);
-		while (!toCheck.empty())
-		{
-			ccHObject* currentObject = toCheck.back();
-			toCheck.pop_back();
-
-			assert(currentObject);
-
-			//check that the ID is not already used (strangely it happens!)
-			unsigned uniqueID = currentObject->getUniqueID();
-			if (uniqueIDs.find(uniqueID) != uniqueIDs.end())
-			{
-				ccLog::Warning(QString("[BIN] Duplicate 'unique ID' found! (ID = %1)").arg(uniqueID));
-				currentObject->setUniqueID(++maxUniqueID);
-			}
-			else
-			{
-				uniqueIDs.insert(uniqueID);
-			}
-
-			for (unsigned i = 0; i < currentObject->getChildrenNumber(); ++i)
-			{
-				toCheck.push_back(currentObject->getChild(i));
-			}
-		}
-	}
-
-	//update 'unique IDs'
-	toCheck.push_back(root);
-	while (!toCheck.empty())
-	{
-		ccHObject* currentObject = toCheck.back();
-		toCheck.pop_back();
-
-		currentObject->setUniqueID(lastUniqueIDBeforeLoad + currentObject->getUniqueID());
-
-		for (unsigned i = 0; i < currentObject->getChildrenNumber(); ++i)
-			toCheck.push_back(currentObject->getChild(i));
-	}
-
 	if (root->isA(CC_TYPES::HIERARCHY_OBJECT))
 	{
 		//transfer children to container
 		root->transferChildren(container, true);
 		delete root;
-		root = 0;
+		root = nullptr;
 	}
 	else
 	{
@@ -1054,7 +1043,7 @@ CC_FILE_ERROR BinFilter::LoadFileV2(QFile& in, ccHObject& container, int flags)
 		else
 		{
 			delete orphans;
-			orphans = 0;
+			orphans = nullptr;
 		}
 	}
 
@@ -1067,7 +1056,7 @@ CC_FILE_ERROR BinFilter::LoadFileV1(QFile& in, ccHObject& container, unsigned nb
 
 	if (nbScansTotal > 99)
 	{
-		if (QMessageBox::question(0, QString("Oups"), QString("Hum, do you really expect %1 point clouds?").arg(nbScansTotal), QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
+		if (QMessageBox::question(nullptr, QString("Oups"), QString("Hum, do you really expect %1 point clouds?").arg(nbScansTotal), QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
 			return CC_FERR_WRONG_FILE_TYPE;
 	}
 	else if (nbScansTotal == 0)
@@ -1075,7 +1064,7 @@ CC_FILE_ERROR BinFilter::LoadFileV1(QFile& in, ccHObject& container, unsigned nb
 		return CC_FERR_NO_LOAD;
 	}
 
-	QScopedPointer<ccProgressDialog> pDlg(0);
+	QScopedPointer<ccProgressDialog> pDlg(nullptr);
 	if (parameters.parentWidget)
 	{
 		pDlg.reset(new ccProgressDialog(true, parameters.parentWidget));
@@ -1158,11 +1147,12 @@ CC_FILE_ERROR BinFilter::LoadFileV1(QFile& in, ccHObject& container, unsigned nb
 		
 		//Creation
 		ccPointCloud* loadedCloud = new ccPointCloud(cloudName);
+		CCLib::ScalarField* loadedCloudSF = nullptr;
 		if (!loadedCloud)
 			return CC_FERR_NOT_ENOUGH_MEMORY;
 
 		unsigned fileChunkPos = 0;
-		unsigned fileChunkSize = std::min(nbOfPoints,CC_MAX_NUMBER_OF_POINTS_PER_CLOUD);
+		unsigned fileChunkSize = std::min(nbOfPoints, CC_MAX_NUMBER_OF_POINTS_PER_CLOUD);
 
 		loadedCloud->reserveThePointsTable(fileChunkSize);
 		if (header.colors)
@@ -1176,31 +1166,46 @@ CC_FILE_ERROR BinFilter::LoadFileV1(QFile& in, ccHObject& container, unsigned nb
 			loadedCloud->showNormals(true);
 		}
 		if (header.scalarField)
-			loadedCloud->enableScalarField();
+		{
+			if (loadedCloud->enableScalarField())
+			{
+				loadedCloudSF = loadedCloud->getCurrentInScalarField();
+			}
+			else
+			{
+				ccLog::Warning(QString("Failed to allocate scalar field on cloud '%1'").arg(loadedCloud->getName()));
+			}
+		}
 
 		unsigned lineRead = 0;
-		int parts = 0;
+		unsigned parts = 0;
 
 		const ScalarType FORMER_HIDDEN_POINTS = static_cast<ScalarType>(-1.0);
 
-		//lecture du fichier
+		//read the file
 		for (unsigned i = 0; i < nbOfPoints; ++i)
 		{
 			if (lineRead == fileChunkPos + fileChunkSize)
 			{
-				if (header.scalarField)
+				if (loadedCloudSF)
 				{
-					loadedCloud->getCurrentInScalarField()->computeMinAndMax();
+					loadedCloudSF->computeMinAndMax();
+					loadedCloudSF = nullptr;
 				}
 
+				//create a new cloud
 				container.addChild(loadedCloud);
 				fileChunkPos = lineRead;
 				fileChunkSize = std::min(nbOfPoints - lineRead, CC_MAX_NUMBER_OF_POINTS_PER_CLOUD);
-				char partName[64];
+				char partName[sizeof(cloudName) + 16];
 				++parts;
-				sprintf(partName, "%s.part_%i", cloudName, parts);
+				sprintf(partName, "%s.part_%u", cloudName, parts);
 				loadedCloud = new ccPointCloud(partName);
-				loadedCloud->reserveThePointsTable(fileChunkSize);
+				if (!loadedCloud->reserveThePointsTable(fileChunkSize))
+				{
+					delete loadedCloud;
+					return CC_FERR_NOT_ENOUGH_MEMORY;
+				}
 
 				if (header.colors)
 				{
@@ -1213,7 +1218,16 @@ CC_FILE_ERROR BinFilter::LoadFileV1(QFile& in, ccHObject& container, unsigned nb
 					loadedCloud->showNormals(true);
 				}
 				if (header.scalarField)
-					loadedCloud->enableScalarField();
+				{
+					if (loadedCloud->enableScalarField())
+					{
+						loadedCloudSF = loadedCloud->getCurrentInScalarField();
+					}
+					else
+					{
+						ccLog::Warning(QString("Failed to allocate scalar field on cloud '%1'").arg(loadedCloud->getName()));
+					}
+				}
 			}
 
 			float Pf[3];
@@ -1232,7 +1246,7 @@ CC_FILE_ERROR BinFilter::LoadFileV1(QFile& in, ccHObject& container, unsigned nb
 					//Console::print("[BinFilter::loadModelFromBinaryFile] Error reading the %ith entity colors !\n",k);
 					return CC_FERR_READING;
 				}
-				loadedCloud->addRGBColor(C);
+				loadedCloud->addColor(C);
 			}
 
 			if (header.normals)
@@ -1254,8 +1268,11 @@ CC_FILE_ERROR BinFilter::LoadFileV1(QFile& in, ccHObject& container, unsigned nb
 					//Console::print("[BinFilter::loadModelFromBinaryFile] Error reading the %ith entity distance!\n",k);
 					return CC_FERR_READING;
 				}
-				ScalarType d = static_cast<ScalarType>(D);
-				loadedCloud->setPointScalarValue(i, d);
+				if (loadedCloudSF)
+				{
+					ScalarType d = static_cast<ScalarType>(D);
+					loadedCloudSF->addElement(d);
+				}
 			}
 
 			lineRead++;
@@ -1274,19 +1291,17 @@ CC_FILE_ERROR BinFilter::LoadFileV1(QFile& in, ccHObject& container, unsigned nb
 			QApplication::processEvents();
 		}
 
-		if (header.scalarField)
+		if (loadedCloudSF)
 		{
-			CCLib::ScalarField* sf = loadedCloud->getCurrentInScalarField();
-			assert(sf);
-			sf->setName(sfName);
+			loadedCloudSF->setName(sfName);
 
 			//replace HIDDEN_VALUES by NAN_VALUES
-			for (unsigned i = 0; i < sf->currentSize(); ++i)
+			for (unsigned i = 0; i < loadedCloudSF->currentSize(); ++i)
 			{
-				if (sf->getValue(i) == FORMER_HIDDEN_POINTS)
-					sf->setValue(i, NAN_VALUE);
+				if (loadedCloudSF->getValue(i) == FORMER_HIDDEN_POINTS)
+					loadedCloudSF->setValue(i, NAN_VALUE);
 			}
-			sf->computeMinAndMax();
+			loadedCloudSF->computeMinAndMax();
 
 			loadedCloud->setCurrentDisplayedScalarField(loadedCloud->getCurrentInScalarFieldIndex());
 			loadedCloud->showSF(true);
