@@ -22,35 +22,45 @@
 #include "PlyOpenDlg.h"
 
 //Qt
-#include <QImage>
 #include <QFileInfo>
+#include <QImage>
 #include <QMessageBox>
 #include <QPushButton>
 
 //qCC_db
+#include <ccHObjectCaster.h>
 #include <ccLog.h>
-#include <ccMesh.h>
-#include <ccPointCloud.h>
 #include <ccMaterial.h>
 #include <ccMaterialSet.h>
+#include <ccMesh.h>
+#include <ccPointCloud.h>
 #include <ccProgressDialog.h>
 #include <ccScalarField.h>
 
 //System
-#include <string.h>
-#include <assert.h>
+#include <cassert>
+#include <cstring>
 #if defined(CC_WINDOWS)
 #include <windows.h>
 #else
-#include <time.h>
+#include <ctime>
 #include <unistd.h>
 #endif
 
 using namespace CCLib;
 
-bool PlyFilter::canLoadExtension(const QString& upperCaseExt) const
-{
-	return (upperCaseExt == "PLY");
+
+PlyFilter::PlyFilter()
+	: FileIOFilter( {
+					"_PLY Filter",
+					7.0f,	// priority
+					QStringList{ "ply" },
+					"ply",
+					QStringList{ "PLY mesh (*.ply)" },
+					QStringList{ "PLY mesh (*.ply)" },
+					Import | Export | BuiltIn
+					} )
+{	
 }
 
 bool PlyFilter::canSave(CC_CLASS_ENUM type, bool& multiple, bool& exclusive) const
@@ -66,6 +76,11 @@ bool PlyFilter::canSave(CC_CLASS_ENUM type, bool& multiple, bool& exclusive) con
 }
 
 static e_ply_storage_mode s_defaultOutputFormat = PLY_DEFAULT;
+
+static void errorCallback(p_ply _ply, const char *message) {
+	ccLog::Error("[PLY] '%s'", message);
+}
+
 void PlyFilter::SetDefaultOutputFormat(e_ply_storage_mode format)
 {
 	s_defaultOutputFormat = format;
@@ -108,9 +123,9 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 	if (!vertices)
 		return CC_FERR_BAD_ENTITY_TYPE;
 
-	p_ply ply = ply_create(qPrintable(filename), storageType, nullptr, 0, nullptr);
+	p_ply ply = ply_create(qPrintable(filename), storageType, errorCallback, 0, nullptr);
 	if (!ply)
-		return CC_FERR_WRITING;
+		return CC_FERR_THIRD_PARTY_LIB_FAILURE;
 
 	//Has the cloud been recentered?
 	e_ply_type coordType = vertices->isShifted() || sizeof(PointCoordinateType) > 4 ? PLY_DOUBLE : PLY_FLOAT; //we use double coordinates for shifted vertices (i.e. >1e6)
@@ -127,7 +142,7 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 	}
 	else result = 0;
 
-	ccMaterial::CShared material(0);
+	ccMaterial::CShared material(nullptr);
 	if (mesh && mesh->hasMaterials())
 	{
 		//look for textures/materials in case there's no color
@@ -168,14 +183,14 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 					{
 						if (mesh->isA(CC_TYPES::MESH))
 						{
-							if (QMessageBox::question(	0,
+							if (QMessageBox::question(	nullptr,
 														"Multiple materials, one texture",
 														"This mesh has only one texture but multiple materials. PLY files can only handle one texture.\nShall we drop the materials (yes) or convert all materials and texture to per-vertex RGB colors? (no)",
 														QMessageBox::Yes | QMessageBox::No,
 														QMessageBox::Yes) == QMessageBox::No)
 							{
 								//we can forget the texture
-								material = ccMaterial::CShared(0);
+								material = ccMaterial::CShared(nullptr);
 								//try to convert materials to RGB
 								if (!static_cast<ccMesh*>(mesh)->convertMaterialsToVertexColors())
 								{
@@ -186,7 +201,7 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 						else if (mesh->isA(CC_TYPES::SUB_MESH))
 						{
 							//we can forget the texture
-							material = ccMaterial::CShared(0);
+							material = ccMaterial::CShared(nullptr);
 							ccLog::Warning("This sub-mesh has one texture and multiple materials. As this is a sub-mesh, we will ignore them...  you should convert the parent mesh textures/materials to RGB colors first");
 						}
 						else
@@ -204,7 +219,7 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 			{
 				assert(materials->size() != 0);
 				//we can forget the (first) texture (if any)
-				material = ccMaterial::CShared(0);
+				material = ccMaterial::CShared(nullptr);
 
 				if (mesh->hasColors())
 				{
@@ -215,7 +230,7 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 					if (mesh->isA(CC_TYPES::MESH))
 					{
 						//we ask the user if he wants to convert them to RGB
-						if (QMessageBox::question(	0,
+						if (QMessageBox::question(	nullptr,
 													"Multiple textures/materials",
 													"PLY files can't handle multiple textures/materials!\nDo you want to convert them to per-vertex RGB colors?",
 													QMessageBox::Yes | QMessageBox::No,
@@ -257,7 +272,7 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 			uniqueColor[1] = static_cast<ColorCompType>(diffuse.g * ccColor::MAX);
 			uniqueColor[2] = static_cast<ColorCompType>(diffuse.b * ccColor::MAX);
 			hasUniqueColor = true;
-			material = ccMaterial::CShared(0); //we can forget it!
+			material = ccMaterial::CShared(nullptr); //we can forget it!
 		}
 	}
 
@@ -348,7 +363,7 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 				if (!material->getTexture().mirrored().save(textureFilePath)) //mirrored --> see ccMaterial
 				{
 					ccLog::Warning(QString("[PLY] Failed to save texture in '%1'!").arg(textureFilePath));
-					material = ccMaterial::CShared(0);
+					material = ccMaterial::CShared(nullptr);
 				}
 				else
 				{
@@ -374,7 +389,7 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 	if (!result)
 	{
 		ply_close(ply);
-		return CC_FERR_WRITING;
+		return CC_FERR_THIRD_PARTY_LIB_FAILURE;
 	}
 
 	//save the point cloud (=vertices)
@@ -432,7 +447,9 @@ CC_FILE_ERROR PlyFilter::saveToFile(ccHObject* entity, QString filename, e_ply_s
 			if (material) //texture coordinates
 			{
 				ply_write(ply, 6.0);
-				TexCoords2D *tx1 = nullptr, *tx2 = nullptr, *tx3 = nullptr;
+				TexCoords2D *tx1 = nullptr;
+				TexCoords2D *tx2 = nullptr;
+				TexCoords2D *tx3 = nullptr;
 				mesh->getTriangleTexCoordinates(i, tx1, tx2, tx3);
 				ply_write(ply, tx1 ? tx1->tx : -1.0);
 				ply_write(ply, tx1 ? tx1->ty : -1.0);
@@ -571,7 +588,7 @@ static int rgb_cb(p_ply_argument argument)
 	e_ply_type type;
 	ply_get_property_info(prop, nullptr, &type, nullptr, nullptr);
 
-	static ccColor::Rgb s_color(0, 0, 0);
+	static ccColor::Rgba s_color(0, 0, 0, ccColor::MAX);
 
 	switch(type)
 	{
@@ -579,22 +596,22 @@ static int rgb_cb(p_ply_argument argument)
 	case PLY_DOUBLE:
 	case PLY_FLOAT32:
 	case PLY_FLOAT64:
-		s_color.rgb[flags & POS_MASK] = static_cast<ColorCompType>(std::min(std::max(0.0, ply_get_argument_value(argument)), 1.0) * ccColor::MAX);
+		s_color.rgba[flags & POS_MASK] = static_cast<ColorCompType>(std::min(std::max(0.0, ply_get_argument_value(argument)), 1.0) * ccColor::MAX);
 		break;
 	case PLY_INT8:
 	case PLY_UINT8:
 	case PLY_CHAR:
 	case PLY_UCHAR:
-		s_color.rgb[flags & POS_MASK] = static_cast<ColorCompType>(ply_get_argument_value(argument));
+		s_color.rgba[flags & POS_MASK] = static_cast<ColorCompType>(ply_get_argument_value(argument));
 		break;
 	default:
-		s_color.rgb[flags & POS_MASK] = static_cast<ColorCompType>(ply_get_argument_value(argument));
+		s_color.rgba[flags & POS_MASK] = static_cast<ColorCompType>(ply_get_argument_value(argument));
 		break;
 	}
 
 	if (flags & ELEM_EOL)
 	{
-		cloud->addRGBColor(s_color);
+		cloud->addColor(s_color); //TODO: handle alpha channel
 		++s_ColorCount;
 
 		if ((s_ColorCount % PROCESS_EVENTS_FREQ) == 0)
@@ -656,7 +673,7 @@ static int scalar_cb(p_ply_argument argument)
 		//skip the next pieces of data
 		return 1;
 	}
-	CCLib::ScalarField* sf = 0;
+	CCLib::ScalarField* sf = nullptr;
 	ply_get_argument_user_data(argument, (void**)(&sf), nullptr);
 
 	p_ply_element element;
@@ -680,7 +697,7 @@ static int face_cb(p_ply_argument argument)
 		//skip the next pieces of data
 		return 1;
 	}
-	ccMesh* mesh = 0;
+	ccMesh* mesh = nullptr;
 	ply_get_argument_user_data(argument, (void**)(&mesh), nullptr);
 	if (!mesh)
 	{
@@ -688,7 +705,8 @@ static int face_cb(p_ply_argument argument)
 		return 1;
 	}
 
-	long length, value_index;
+	long length = 0;
+	long value_index = 0;
 	ply_get_argument_property(argument, nullptr, &length, &value_index);
 	//unsupported polygon type!
 	if (length != 3 && length != 4)
@@ -770,7 +788,8 @@ static int texCoords_cb(p_ply_argument argument)
 		return 1;
 	}
 
-	long length, value_index;
+	long length = 0;
+	long value_index = 0;
 	ply_get_argument_property(argument, nullptr, &length, &value_index);
 	//unsupported/invalid coordinates!
 	if (length != 6 && length != 8)
@@ -788,7 +807,7 @@ static int texCoords_cb(p_ply_argument argument)
 
 	if (((value_index + 1) % 2) == 0)
 	{
-		TextureCoordsContainer* texCoords = 0;
+		TextureCoordsContainer* texCoords = nullptr;
 		ply_get_argument_user_data(argument, (void**)(&texCoords), nullptr);
 		assert(texCoords);
 		if (!texCoords)
@@ -825,7 +844,7 @@ static int texIndexes_cb(p_ply_argument argument)
 		s_maxTextureIndex = -1;
 	}
 
-	ccMesh::triangleMaterialIndexesSet* texIndexes = 0;
+	ccMesh::triangleMaterialIndexesSet* texIndexes = nullptr;
 	ply_get_argument_user_data(argument, (void**)(&texIndexes), nullptr);
 	assert(texIndexes);
 	if (!texIndexes)
@@ -870,16 +889,15 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 	/****************/
 
 	//open a PLY file for reading
-	p_ply ply = ply_open(qPrintable(filename), nullptr, 0, nullptr);
+	p_ply ply = ply_open(qPrintable(filename), errorCallback, 0, nullptr);
 	if (!ply)
-		return CC_FERR_READING;
-
+		return CC_FERR_THIRD_PARTY_LIB_FAILURE;
 	ccLog::PrintDebug(QString("[PLY] Opening file '%1' ...").arg(filename));
 
 	if (!ply_read_header(ply))
 	{
 		ply_close(ply);
-		return CC_FERR_WRONG_FILE_TYPE;
+		return CC_FERR_THIRD_PARTY_LIB_FAILURE;
 	}
 
 	//storage mode: little/big endian
@@ -892,9 +910,9 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 	//eventual texture files declared in the comments (keyword: TEXTUREFILE)
 	QStringList textureFileNames;
 	//texture coordinates
-	TextureCoordsContainer* texCoords = 0;
+	TextureCoordsContainer* texCoords = nullptr;
 	//texture indexes
-	ccMesh::triangleMaterialIndexesSet* texIndexes = 0;
+	ccMesh::triangleMaterialIndexesSet* texIndexes = nullptr;
 
 	/******************/
 	/***  Comments  ***/
@@ -944,7 +962,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 	{
 		//last read element
 		plyElement lastElement;
-		lastElement.elem = 0;
+		lastElement.elem = nullptr;
 		while ((lastElement.elem = ply_get_next_element(ply, lastElement.elem)))
 		{
 			//we get next element info
@@ -963,7 +981,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 
 			//last read property
 			plyProperty lastProperty;
-			lastProperty.prop = 0;
+			lastProperty.prop = nullptr;
 			lastProperty.elemIndex = 0;
 
 			while ((lastProperty.prop = ply_get_next_property(lastElement.elem, lastProperty.prop)))
@@ -1579,7 +1597,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 
 	/* MESH FACETS (TRI) */
 
-	ccMesh* mesh = 0;
+	ccMesh* mesh = nullptr;
 	unsigned numberOfFacets = 0;
 
 	if (facesIndex > 0)
@@ -1596,7 +1614,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 			ccLog::Error("Not enough memory to load facets (they will be ignored)!");
 			ccLog::Warning("[PLY] Mesh ignored!");
 			delete mesh;
-			mesh = 0;
+			mesh = nullptr;
 			numberOfFacets = 0;
 		}
 		else
@@ -1621,7 +1639,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 			ccLog::Error("Not enough memory to load texture coordinates (they will be ignored)!");
 			ccLog::Warning("[PLY] Texture coordinates ignored!");
 			texCoords->release();
-			texCoords = 0;
+			texCoords = nullptr;
 		}
 		else
 		{
@@ -1645,7 +1663,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 			ccLog::Error("Not enough memory to load texture indexes (they will be ignored)!");
 			ccLog::Warning("[PLY] Texture indexes ignored!");
 			texIndexes->release();
-			texIndexes = 0;
+			texIndexes = nullptr;
 		}
 		else
 		{
@@ -1654,7 +1672,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 		}
 	}
 
-	QScopedPointer<ccProgressDialog> pDlg(0);
+	QScopedPointer<ccProgressDialog> pDlg(nullptr);
 	if (parameters.parentWidget)
 	{
 		pDlg.reset(new ccProgressDialog(false, parameters.parentWidget));
@@ -1688,7 +1706,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 		if (mesh)
 			delete mesh;
 		delete cloud; 
-		return CC_FERR_READING;
+		return s_NotEnoughMemory ? CC_FERR_NOT_ENOUGH_MEMORY : CC_FERR_THIRD_PARTY_LIB_FAILURE;
 	}
 
 	//we check mesh
@@ -1705,7 +1723,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 				ccLog::Error("Mesh is empty!");
 			}
 			delete mesh;
-			mesh = 0;
+			mesh = nullptr;
 		}
 		else
 		{
@@ -1720,7 +1738,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 	{
 		ccLog::Error("Invalid texture coordinates! (they will be ignored)");
 		texCoords->release();
-		texCoords = 0;
+		texCoords = nullptr;
 	}
 
 	if (texIndexes)
@@ -1729,7 +1747,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 		{
 			ccLog::Error("No texture coordinates were loaded (texture indexes will be ignored)");
 			texIndexes->release();
-			texIndexes = 0;
+			texIndexes = nullptr;
 		}
 		else if (texIndexes->currentSize() < mesh->size())
 		{
@@ -1737,7 +1755,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 			{
 				ccLog::Error("Invalid texture indexes! (they will be ignored)");
 				texIndexes->release();
-				texIndexes = 0;
+				texIndexes = nullptr;
 			}
 			else
 			{
@@ -1746,7 +1764,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 				{
 					ccLog::Warning("Not enough memory to store texture indexes");
 					texIndexes->release();
-					texIndexes = 0;
+					texIndexes = nullptr;
 				}
 			}
 		}
@@ -1782,7 +1800,8 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 		mesh->shrinkToFit();
 
 		//check that vertex indices start at 0
-		unsigned minVertIndex = numberOfPoints, maxVertIndex = 0;
+		unsigned minVertIndex = numberOfPoints;
+		unsigned maxVertIndex = 0;
 		for (unsigned i = 0; i < s_triCount; ++i)
 		{
 			const CCLib::VerticesIndexes* tri = mesh->getTriangleVertIndexes(i);
@@ -1830,7 +1849,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 		//associated texture
 		if (texCoords)
 		{
-			ccMaterialSet* materials = 0;
+			ccMaterialSet* materials = nullptr;
 			if (!textureFileNames.isEmpty())
 			{
 				//try to load the materials
@@ -1860,7 +1879,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 				if (materials->empty())
 				{
 					materials->release();
-					materials = 0;
+					materials = nullptr;
 				}
 			}
 			else
@@ -1924,7 +1943,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 				{
 					ccLog::Warning("[PLY][Texture] Failed to reserve per-triangle texture coordinates! (not enough memory?)");
 					materials->release();
-					materials = 0;
+					materials = nullptr;
 				}
 			}
 
@@ -1935,7 +1954,7 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 				{
 					assert(!mesh->getTriangleMtlIndexesTable());
 					texIndexes->release();
-					texIndexes = 0;
+					texIndexes = nullptr;
 				}
 				else
 				{
@@ -1973,12 +1992,12 @@ CC_FILE_ERROR PlyFilter::loadFile(const QString& filename, const QString& inputT
 	if (texCoords)
 	{
 		texCoords->release();
-		texCoords = 0;
+		texCoords = nullptr;
 	}
 	if (texIndexes)
 	{
 		texIndexes->release();
-		texIndexes = 0;
+		texIndexes = nullptr;
 	}
 
 	return CC_FERR_NO_ERROR;
