@@ -20,32 +20,32 @@
 #include "RasterGridFilter.h"
 
 //qCC_db
-#include <ccPointCloud.h>
-#include <ccScalarField.h>
 #include <ccMesh.h>
 #include <ccPlane.h>
+#include <ccPointCloud.h>
+#include <ccScalarField.h>
 
 //GDAL
-#include <gdal_priv.h>
 #include <cpl_conv.h> // for CPLMalloc()
+#include <gdal_priv.h>
 
 //Qt
 #include <QMessageBox>
 
 //System
-#include <string.h> //for memset
+#include <cstring> //for memset
 
-bool RasterGridFilter::canLoadExtension(const QString& upperCaseExt) const
+RasterGridFilter::RasterGridFilter()
+	: FileIOFilter( {
+					"_Raster Grid Filter",
+					16.0f,	// priority
+					QStringList{ "tif", "tiff", "adf" },
+					"tif",
+					QStringList{ "RASTER grid (*.*)" },
+					QStringList(),
+					Import | BuiltIn
+					} )
 {
-	return (	upperCaseExt == "TIF"
-			||	upperCaseExt == "TIFF"
-			||	upperCaseExt == "ADF");
-}
-
-bool RasterGridFilter::canSave(CC_CLASS_ENUM type, bool& multiple, bool& exclusive) const
-{
-	//not supported yet
-	return false;
 }
 
 CC_FILE_ERROR RasterGridFilter::loadFile(const QString& filename, ccHObject& container, LoadParameters& parameters)
@@ -117,12 +117,15 @@ CC_FILE_ERROR RasterGridFilter::loadFile(const QString& filename, ccHObject& con
 			bool loadAsTexturedQuad = false;
 			if (colorBands >= 3)
 			{
-				loadAsTexturedQuad =
-					(QMessageBox::question(	parameters.parentWidget,
+				loadAsTexturedQuad = false;
+				if (parameters.parentWidget) //otherwise it means we are in command line mode --> no popup
+				{
+					loadAsTexturedQuad = (	QMessageBox::question(	parameters.parentWidget,
 											"Result type",
 											"Import raster as a cloud (yes) or a texture quad? (no)",
 											QMessageBox::Yes,
-											QMessageBox::No) == QMessageBox::No);
+											QMessageBox::No) == QMessageBox::No );
+				}
 			}
 
 			ccPointCloud* pc = new ccPointCloud();
@@ -352,7 +355,7 @@ CC_FILE_ERROR RasterGridFilter::loadFile(const QString& filename, ccHObject& con
 						else
 						{
 							//instantiate memory for RBG colors if necessary
-							if (!loadAsTexturedQuad && !pc->hasColors() && !pc->setRGBColor(ccColor::MAX, ccColor::MAX, ccColor::MAX))
+							if (!loadAsTexturedQuad && !pc->hasColors() && !pc->setColor(ccColor::white))
 							{
 								ccLog::Warning(QString("Failed to instantiate memory for storing color band '%1'!").arg(GDALGetColorInterpretationName(colorInterp)));
 							}
@@ -389,8 +392,7 @@ CC_FILE_ERROR RasterGridFilter::loadFile(const QString& filename, ccHObject& con
 											}
 											else
 											{
-												const ccColor::Rgb& origColor = pc->getPointColor(pointIndex);
-												C = ccColor::Rgba(origColor, ccColor::MAX);
+												C = pc->getPointColor(pointIndex);
 											}
 
 											switch (colorInterp)
@@ -459,22 +461,22 @@ CC_FILE_ERROR RasterGridFilter::loadFile(const QString& filename, ccHObject& con
 							//double* scanline = new double[nXSize];
 							memset(colValues, 0, sizeof(double)*nXSize);
 
-							for (int j=0; j<nYSize; ++j)
+							for (int j = 0; j < nYSize; ++j)
 							{
-								if (poBand->RasterIO( GF_Read, /*xOffset=*/0, /*yOffset=*/j, /*xSize=*/nXSize, /*ySize=*/1, /*buffer=*/colValues, /*bufferSizeX=*/nXSize, /*bufferSizeY=*/1, /*bufferType=*/GDT_Float64, /*x_offset=*/0, /*y_offset=*/0 ) != CE_None)
+								if (poBand->RasterIO(GF_Read, /*xOffset=*/0, /*yOffset=*/j, /*xSize=*/nXSize, /*ySize=*/1, /*buffer=*/colValues, /*bufferSizeX=*/nXSize, /*bufferSizeY=*/1, /*bufferType=*/GDT_Float64, /*x_offset=*/0, /*y_offset=*/0) != CE_None)
 								{
 									CPLFree(colValues);
 									delete pc;
 									return CC_FERR_READING;
 								}
 
-								for (int k=0; k<nXSize; ++k)
+								for (int k = 0; k < nXSize; ++k)
 								{
 									unsigned pointIndex = static_cast<unsigned>(k + j * rasterX);
 									if (pointIndex <= pc->size())
 									{
 										ScalarType s = static_cast<ScalarType>(colValues[k]);
-										sf->setValue(pointIndex,s);
+										sf->setValue(pointIndex, s);
 									}
 								}
 							}
@@ -508,14 +510,23 @@ CC_FILE_ERROR RasterGridFilter::loadFile(const QString& filename, ccHObject& con
 				else if (zInvalid != 0 && zInvalid < pc->size())
 				{
 					//shall we remove the points with invalid heights?
-					if (QMessageBox::question(0, "Remove NaN points?", "This raster has pixels with invalid heights. Shall we remove them?", QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
+					static bool s_alwaysRemoveInvalidHeights = false;
+					int result = QMessageBox::Yes;
+					if (parameters.parentWidget) //otherwise it means we are in command line mode --> no popup
 					{
+						result = (s_alwaysRemoveInvalidHeights ? QMessageBox::Yes : QMessageBox::question(0, "Remove NaN points?", "This raster has pixels with invalid heights. Shall we remove them?", QMessageBox::Yes, QMessageBox::YesToAll, QMessageBox::No));
+					}
+					if (result != QMessageBox::No)
+					{
+						if (result == QMessageBox::YesToAll)
+							s_alwaysRemoveInvalidHeights = true;
+
 						CCLib::ReferenceCloud validPoints(pc);
 						unsigned count = pc->size();
 						bool error = true;
-						if (validPoints.reserve(count-zInvalid))
+						if (validPoints.reserve(count - zInvalid))
 						{
-							for (unsigned i=0; i<count; ++i)
+							for (unsigned i = 0; i < count; ++i)
 							{
 								if (pc->getPoint(i)->z >= zMinMax[0])
 									validPoints.addPointIndex(i);
