@@ -27,6 +27,7 @@
 #include <ccPointCloud.h>
 #include <ccScalarField.h>
 #include <ccProgressDialog.h>
+#include <ccColorTypes.h>
 
 //qCC_gl
 #include <ccGLWidget.h>
@@ -40,9 +41,9 @@
 #include <assert.h>
 
 cc2Point5DimEditor::cc2Point5DimEditor()
-	: m_bbEditorDlg(0)
-	, m_glWindow(0)
-	, m_rasterCloud(0)
+	: m_bbEditorDlg(nullptr)
+	, m_glWindow(nullptr)
+	, m_rasterCloud(nullptr)
 {
 }
 
@@ -53,7 +54,7 @@ cc2Point5DimEditor::~cc2Point5DimEditor()
 		if (m_glWindow)
 			m_glWindow->removeFromOwnDB(m_rasterCloud);
 		delete m_rasterCloud;
-		m_rasterCloud = 0;
+		m_rasterCloud = nullptr;
 	}
 }
 
@@ -87,7 +88,7 @@ void cc2Point5DimEditor::create2DView(QFrame* parentFrame)
 {
 	if (!m_glWindow)
 	{
-		QWidget* glWidget = 0;
+		QWidget* glWidget = nullptr;
 		CreateGLWindow(m_glWindow, glWidget, false, true);
 		assert(m_glWindow && glWidget);
 		
@@ -134,7 +135,8 @@ bool cc2Point5DimEditor::getGridSize(unsigned& gridWidth, unsigned& gridHeight)
 
 QString cc2Point5DimEditor::getGridSizeAsString() const
 {
-	unsigned gridWidth = 0, gridHeight = 0;
+	unsigned gridWidth = 0;
+	unsigned gridHeight = 0;
 	if (!getGridSize(gridWidth, gridHeight))
 	{
 		return QObject::tr("invalid grid box");
