@@ -26,7 +26,7 @@ class ccApplication : public ccApplicationBase
 	Q_OBJECT
 
 public:
-	ccApplication( int &argc, char **argv );
+	ccApplication( int &argc, char **argv, bool isCommandLine );
 	
 protected:
 	bool event( QEvent *inEvent ) override;
