@@ -20,11 +20,13 @@
 //Local
 #include "ccBoundingBoxEditorDlg.h"
 #include "ccClippingBoxRepeatDlg.h"
-#include "ccContourExtractor.h"
 #include "ccCropTool.h"
 #include "ccGLWindow.h"
+#include "ccReservedIDs.h"
 #include "mainwindow.h"
 
+#include "db_tree/ccDBRoot.h"
+
 //qCC_db
 #include <ccClipBox.h>
 #include <ccPointCloud.h>
@@ -36,11 +38,8 @@
 //Last contour unique ID
 static std::vector<unsigned> s_lastContourUniqueIDs;
 
-//Contour extraction parameters
+//Contour extraction parameters (global)
 static double s_maxEdgeLength = -1.0;
-static bool s_splitContours = false;
-static bool s_multiPass = false;
-static double s_defaultGap = 0.0;
 
 // persistent map of the previous box used for each entity
 struct ccClipBoxParams
@@ -53,40 +52,40 @@ static QMap< unsigned, ccClipBoxParams > s_lastBoxParams;
 ccClippingBoxTool::ccClippingBoxTool(QWidget* parent)
 	: ccOverlayDialog(parent)
 	, Ui::ClippingBoxDlg()
-	, m_clipBox(0)
+	, m_clipBox(nullptr)
 {
 	setupUi(this);
 
-	connect(editBoxToolButton,				SIGNAL(clicked()),				this, SLOT(editBox()));
-	connect(extractContourToolButton,		SIGNAL(clicked()),				this, SLOT(extractContour()));
-	connect(removeLastContourToolButton,	SIGNAL(clicked()),				this, SLOT(removeLastContour()));
-	connect(exportButton,					SIGNAL(clicked()),				this, SLOT(exportSlice()));
-	connect(exportMultButton,				SIGNAL(clicked()),				this, SLOT(exportMultSlices()));
-	connect(resetButton,					SIGNAL(clicked()),				this, SLOT(reset()));
-	connect(restoreToolButton,				SIGNAL(clicked()),				this, SLOT(restoreLastBox()));
-	connect(closeButton,					SIGNAL(clicked()),				this, SLOT(closeDialog()));
+	connect(editBoxToolButton,				&QToolButton::clicked,				this, &ccClippingBoxTool::editBox);
+	connect(extractContourToolButton,		&QToolButton::clicked,				this, &ccClippingBoxTool::extractContour);
+	connect(removeLastContourToolButton,	&QToolButton::clicked,				this, &ccClippingBoxTool::removeLastContour);
+	connect(exportButton,					&QToolButton::clicked,				this, &ccClippingBoxTool::exportSlice);
+	connect(exportMultButton,				&QToolButton::clicked,				this, &ccClippingBoxTool::exportMultSlices);
+	connect(resetButton,					&QToolButton::clicked,				this, &ccClippingBoxTool::reset);
+	connect(restoreToolButton,				&QToolButton::clicked,				this, &ccClippingBoxTool::restoreLastBox);
+	connect(closeButton,					&QToolButton::clicked,				this, &ccClippingBoxTool::closeDialog);
 
-	connect(showBoxToolButton,				SIGNAL(toggled(bool)),			this, SLOT(toggleBox(bool)));
-	connect(showInteractorsToolButton,		SIGNAL(toggled(bool)),			this, SLOT(toggleInteractors(bool)));
+	connect(showBoxToolButton,				&QToolButton::toggled,				this, &ccClippingBoxTool::toggleBox);
+	connect(showInteractorsToolButton,		&QToolButton::toggled,				this, &ccClippingBoxTool::toggleInteractors);
 
-	connect(thickXDoubleSpinBox,			SIGNAL(valueChanged(double)),	this, SLOT(thicknessChanged(double)));
-	connect(thickYDoubleSpinBox,			SIGNAL(valueChanged(double)),	this, SLOT(thicknessChanged(double)));
-	connect(thickZDoubleSpinBox,			SIGNAL(valueChanged(double)),	this, SLOT(thicknessChanged(double)));
+	connect(thickXDoubleSpinBox,			static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),	this, &ccClippingBoxTool::thicknessChanged);
+	connect(thickYDoubleSpinBox,			static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),	this, &ccClippingBoxTool::thicknessChanged);
+	connect(thickZDoubleSpinBox,			static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),	this, &ccClippingBoxTool::thicknessChanged);
 
-	connect(minusXShiftToolButton,			SIGNAL(clicked()),				this, SLOT(shiftXMinus()));
-	connect(plusXShiftToolButton,			SIGNAL(clicked()),				this, SLOT(shiftXPlus()));
-	connect(minusYShiftToolButton,			SIGNAL(clicked()),				this, SLOT(shiftYMinus()));
-	connect(plusYShiftToolButton,			SIGNAL(clicked()),				this, SLOT(shiftYPlus()));
-	connect(minusZShiftToolButton,			SIGNAL(clicked()),				this, SLOT(shiftZMinus()));
-	connect(plusZShiftToolButton,			SIGNAL(clicked()),				this, SLOT(shiftZPlus()));
+	connect(minusXShiftToolButton,			&QToolButton::clicked,				this, &ccClippingBoxTool::shiftXMinus);
+	connect(plusXShiftToolButton,			&QToolButton::clicked,				this, &ccClippingBoxTool::shiftXPlus);
+	connect(minusYShiftToolButton,			&QToolButton::clicked,				this, &ccClippingBoxTool::shiftYMinus);
+	connect(plusYShiftToolButton,			&QToolButton::clicked,				this, &ccClippingBoxTool::shiftYPlus);
+	connect(minusZShiftToolButton,			&QToolButton::clicked,				this, &ccClippingBoxTool::shiftZMinus);
+	connect(plusZShiftToolButton,			&QToolButton::clicked,				this, &ccClippingBoxTool::shiftZPlus);
 
 	viewButtonsFrame->setEnabled(true);
-	connect(viewUpToolButton,				SIGNAL(clicked()),				this, SLOT(setTopView()));
-	connect(viewDownToolButton,				SIGNAL(clicked()),				this, SLOT(setBottomView()));
-	connect(viewFrontToolButton,			SIGNAL(clicked()),				this, SLOT(setFrontView()));
-	connect(viewBackToolButton,				SIGNAL(clicked()),				this, SLOT(setBackView()));
-	connect(viewLeftToolButton,				SIGNAL(clicked()),				this, SLOT(setLeftView()));
-	connect(viewRightToolButton,			SIGNAL(clicked()),				this, SLOT(setRightView()));
+	connect(viewUpToolButton,				&QToolButton::clicked,				this, &ccClippingBoxTool::setTopView);
+	connect(viewDownToolButton,				&QToolButton::clicked,				this, &ccClippingBoxTool::setBottomView);
+	connect(viewFrontToolButton,			&QToolButton::clicked,				this, &ccClippingBoxTool::setFrontView);
+	connect(viewBackToolButton,				&QToolButton::clicked,				this, &ccClippingBoxTool::setBackView);
+	connect(viewLeftToolButton,				&QToolButton::clicked,				this, &ccClippingBoxTool::setLeftView);
+	connect(viewRightToolButton,			&QToolButton::clicked,				this, &ccClippingBoxTool::setRightView);
 
 	s_maxEdgeLength = -1.0;
 	//s_lastContourUniqueIDs.clear();
@@ -97,7 +96,7 @@ ccClippingBoxTool::~ccClippingBoxTool()
 {
 	if (m_clipBox)
 		delete m_clipBox;
-	m_clipBox = 0;
+	m_clipBox = nullptr;
 }
 
 void ccClippingBoxTool::editBox()
@@ -109,6 +108,21 @@ void ccClippingBoxTool::editBox()
 	ccGLMatrix transformation;
 	m_clipBox->get(box, transformation);
 
+	//shift the box to its real center
+	{
+		CCVector3 C = box.getCenter();
+		CCVector3 realC = transformation * C;
+		box += (realC - C);
+
+		ccGLMatrix transMat;
+		transMat.setTranslation(-realC);
+		transformation.clearTranslation();
+		transMat = transformation * transMat;
+		transMat.setTranslation(transMat.getTranslationAsVec3D() + realC);
+		transformation = transMat;
+	}
+
+
 	ccBoundingBoxEditorDlg bbeDlg(this);
 	bbeDlg.setBaseBBox(box, false);
 	bbeDlg.showInclusionWarning(false);
@@ -125,11 +139,12 @@ void ccClippingBoxTool::editBox()
 		return;
 
 	box = bbeDlg.getBox();
-	m_clipBox->setBox(box);
 
 	//construct the local box orientation matrix
 	{
-		CCVector3 X, Y, Z;
+		CCVector3d X;
+		CCVector3d Y;
+		CCVector3d Z;
 		bbeDlg.getBoxAxes(X, Y, Z);
 		//make sure the vectors define an orthogonal basis
 		Z = X.cross(Y);
@@ -138,20 +153,22 @@ void ccClippingBoxTool::editBox()
 		X.normalize();
 		Y.normalize();
 		Z.normalize();
-		ccGLMatrix rotMat;
+		ccGLMatrixd rotMat;
 		rotMat.setColumn(0, X);
 		rotMat.setColumn(1, Y);
 		rotMat.setColumn(2, Z);
 
 		CCVector3 C = box.getCenter();
-		ccGLMatrix transMat;
+		ccGLMatrixd transMat;
 		transMat.setTranslation(-C);
-		transMat = rotMat.inverse() * transMat;
-		transMat.setTranslation(transMat.getTranslationAsVec3D() + C);
+		transMat = rotMat * transMat;
+		transMat.setTranslation(transMat.getTranslationAsVec3D() + CCVector3d::fromArray(C.u));
 
-		m_clipBox->setGLTransformation(transMat.inverse());
+		m_clipBox->setGLTransformation(ccGLMatrix(transMat.data()));
 	}
 
+	m_clipBox->setBox(box);
+
 	//onBoxModified(&box); //DGM: automatically called by 'm_clipBox'
 
 	if (m_associatedWin)
@@ -271,7 +288,7 @@ bool ccClippingBoxTool::linkWith(ccGLWindow* win)
 		m_associatedWin->removeFromOwnDB(m_clipBox);
 		m_clipBox->disconnect(this);
 		delete m_clipBox;
-		m_clipBox = 0;
+		m_clipBox = nullptr;
 		m_associatedWin->redraw();
 	}
 
@@ -284,11 +301,11 @@ bool ccClippingBoxTool::linkWith(ccGLWindow* win)
 	{
 		if (!m_clipBox)
 		{
-			m_clipBox = new ccClipBox();
+			m_clipBox = new ccClipBox(QString(), static_cast<unsigned>(ReservedIDs::CLIPPING_BOX));
 			m_clipBox->setVisible(true);
 			m_clipBox->setEnabled(true);
 			m_clipBox->setSelected(showInteractorsToolButton->isChecked());
-			connect(m_clipBox, SIGNAL(boxModified(const ccBBox*)), this, SLOT(onBoxModified(const ccBBox*)));
+			connect(m_clipBox, &ccClipBox::boxModified, this, &ccClippingBoxTool::onBoxModified);
 		}
 		m_associatedWin->addToOwnDB(m_clipBox);
 	}
@@ -374,7 +391,7 @@ ccHObject* GetSlice(ccHObject* obj, ccClipBox* clipBox, bool silent)
 	if (!obj)
 	{
 		assert(false);
-		return 0;
+		return nullptr;
 	}
 
 	if (obj->isKindOf(CC_TYPES::POINT_CLOUD))
@@ -392,7 +409,7 @@ ccHObject* GetSlice(ccHObject* obj, ccClipBox* clipBox, bool silent)
 			{
 				ccLog::Error("Not enough memory!");
 			}
-			return 0;
+			return nullptr;
 		}
 		clipBox->flagPointsInside(inputCloud, &selectionTable);
 		
@@ -411,7 +428,7 @@ ccHObject* GetSlice(ccHObject* obj, ccClipBox* clipBox, bool silent)
 	}
 	else if (obj->isKindOf(CC_TYPES::MESH))
 	{
-		const ccGLMatrix* _transformation = 0;
+		const ccGLMatrix* _transformation = nullptr;
 		ccGLMatrix transformation;
 		if (clipBox->isGLTransEnabled())
 		{
@@ -425,12 +442,12 @@ ccHObject* GetSlice(ccHObject* obj, ccClipBox* clipBox, bool silent)
 		{
 			if (!silent)
 				ccLog::Error("Failed to segment the mesh!");
-			return 0;
+			return nullptr;
 		}
 		return mesh;
 	}
 
-	return 0;
+	return nullptr;
 }
 
 void ccClippingBoxTool::exportSlice()
@@ -524,6 +541,7 @@ bool ccClippingBoxTool::ExtractSlicesAndContours
 	bool extractContours,
 	PointCoordinateType maxEdgeLength,
 	std::vector<ccPolyline*>& outputContours,
+	ccContourExtractor::ContourType contourType,
 	PointCoordinateType gap/*=0*/,
 	bool multiPass/*=false*/,
 	bool splitContours/*=false*/,
@@ -608,12 +626,14 @@ bool ccClippingBoxTool::ExtractSlicesAndContours
 					}
 				}
 
-				int indexMins[3], indexMaxs[3], gridDim[3];
+				int indexMins[3]{ 0, 0, 0 };
+				int indexMaxs[3]{ 0, 0, 0 };
+				int gridDim[3]{ 0, 0, 0 };
 				unsigned cellCount = ComputeGridDimensions(localBox, repeatDimensions, indexMins, indexMaxs, gridDim, gridOrigin, cellSizePlusGap);
 
 				//we'll potentially create up to one (ref.) cloud per input loud and per cell
 				std::vector<CCLib::ReferenceCloud*> refClouds;
-				refClouds.resize(cellCount * clouds.size(), 0);
+				refClouds.resize(cellCount * clouds.size(), nullptr);
 
 				if (progressDialog)
 				{
@@ -723,7 +743,7 @@ bool ccClippingBoxTool::ExtractSlicesAndContours
 										if (generateRandomColors)
 										{
 											ccColor::Rgb col = ccColor::Generator::Random();
-											if (!sliceCloud->setRGBColor(col))
+											if (!sliceCloud->setColor(col))
 											{
 												ccLog::Error("Not enough memory!");
 												error = true;
@@ -797,10 +817,12 @@ bool ccClippingBoxTool::ExtractSlicesAndContours
 					}
 				}
 
-				int indexMins[3], indexMaxs[3], gridDim[3];
+				int indexMins[3]{ 0, 0, 0 };
+				int indexMaxs[3]{ 0, 0, 0 };
+				int gridDim[3]{ 0, 0, 0 };
 				unsigned cellCount = ComputeGridDimensions(localBox, repeatDimensions, indexMins, indexMaxs, gridDim, gridOrigin, cellSizePlusGap);
 
-				const ccGLMatrix* _transformation = 0;
+				const ccGLMatrix* _transformation = nullptr;
 				ccGLMatrix transformation;
 				if (clipBox.isGLTransEnabled())
 				{
@@ -841,7 +863,7 @@ bool ccClippingBoxTool::ExtractSlicesAndContours
 										if (croppedVertices)
 										{
 											ccColor::Rgb col = ccColor::Generator::Random();
-											if (!croppedVertices->setRGBColor(col))
+											if (!croppedVertices->setColor(col))
 											{
 												ccLog::Error("Not enough memory!");
 												error = true;
@@ -903,15 +925,22 @@ bool ccClippingBoxTool::ExtractSlicesAndContours
 			}
 
 			//preferred dimension?
-			int preferredDim = -1;
-			if (repeatDimensionsSum == 1 && !projectOnBestFitPlane)
+			PointCoordinateType* preferredNormDir = nullptr;
+			PointCoordinateType* preferredUpDir = nullptr;
+			if (repeatDimensionsSum == 1)
 			{
 				for (int i = 0; i < 3; ++i)
+				{
 					if (repeatDimensions[i])
-						preferredDim = i;
+					{
+						ccGLMatrix invLocalTrans = localTrans.inverse();
+						if (!projectOnBestFitPlane) //otherwise the normal will be automatically computed
+							preferredNormDir = invLocalTrans.getColumn(i);
+						preferredUpDir = invLocalTrans.getColumn(i < 2 ? 2 : 0);
+						break;
+					}
+				}
 			}
-			ccGLMatrix invLocalTrans = localTrans.inverse();
-			PointCoordinateType* preferredOrientation = (preferredDim != -1 ? invLocalTrans.getColumn(preferredDim) : 0);
 
 			assert(cloudSliceCount <= outputSlices.size());
 
@@ -926,8 +955,10 @@ bool ccClippingBoxTool::ExtractSlicesAndContours
 					multiPass,
 					maxEdgeLength,
 					polys,
+					contourType,
 					splitContours,
-					preferredOrientation,
+					preferredNormDir,
+					preferredUpDir,
 					visualDebugMode))
 				{
 					if (!polys.empty())
@@ -1067,6 +1098,15 @@ void ccClippingBoxTool::extractSlicesAndContours(bool extractSlices, bool extrac
 	//random colors is only useful for mutliple slice/contour mode
 	repeatDlg.randomColorCheckBox->setEnabled(!singleContourMode);
 	
+	//Semi persistent contour extraction parameters
+	static bool s_splitContours = false;
+	static bool s_multiPass = false;
+	static double s_defaultGap = 0.0;
+	static int s_contourTypeIndex = 2; //full
+	static bool s_projectOnBestFitPlane = false;
+	static bool s_visualDebugMode = false;
+	static bool s_generateRandomColors = false;
+
 	//set default max edge length
 	if (s_maxEdgeLength < 0)
 		s_maxEdgeLength = m_clipBox->getBox().getDiagNorm() / 100.0;
@@ -1074,6 +1114,10 @@ void ccClippingBoxTool::extractSlicesAndContours(bool extractSlices, bool extrac
 	repeatDlg.splitContourCheckBox->setChecked(s_splitContours);
 	repeatDlg.multiPassCheckBox->setChecked(s_multiPass);
 	repeatDlg.gapDoubleSpinBox->setValue(s_defaultGap);
+	repeatDlg.contourTypeComboBox->setCurrentIndex(s_contourTypeIndex);
+	repeatDlg.projectOnBestFitCheckBox->setChecked(s_projectOnBestFitPlane);
+	repeatDlg.debugModeCheckBox->setChecked(s_visualDebugMode);
+	repeatDlg.randomColorCheckBox->setChecked(s_generateRandomColors);
 
 	if (!repeatDlg.exec())
 	{
@@ -1091,10 +1135,28 @@ void ccClippingBoxTool::extractSlicesAndContours(bool extractSlices, bool extrac
 	s_maxEdgeLength = repeatDlg.maxEdgeLengthDoubleSpinBox->value();
 	s_splitContours = repeatDlg.splitContourCheckBox->isChecked();
 	s_multiPass = repeatDlg.multiPassCheckBox->isChecked();
-	s_defaultGap = repeatDlg.gapDoubleSpinBox->value();
-	bool projectOnBestFitPlane = repeatDlg.projectOnBestFitCheckBox->isChecked();
-	bool visualDebugMode = repeatDlg.debugModeCheckBox->isChecked();
-	bool generateRandomColors = repeatDlg.randomColorCheckBox->isChecked();
+	s_contourTypeIndex = repeatDlg.contourTypeComboBox->currentIndex();
+	s_projectOnBestFitPlane = repeatDlg.projectOnBestFitCheckBox->isChecked();
+	s_visualDebugMode = repeatDlg.debugModeCheckBox->isChecked();
+	s_generateRandomColors = repeatDlg.randomColorCheckBox->isChecked();
+
+	ccContourExtractor::ContourType contourType = ccContourExtractor::ContourType::FULL;
+	switch (s_contourTypeIndex)
+	{
+	case 0:
+		contourType = ccContourExtractor::ContourType::LOWER;
+		break;
+	case 1:
+		contourType = ccContourExtractor::ContourType::UPPER;
+		break;
+	case 2:
+		contourType = ccContourExtractor::ContourType::FULL;
+		break;
+	default:
+		assert(false);
+		ccLog::Warning("Internal error: unhandled contour type");
+		break;
+	}
 
 	//whether to extract contours or not
 	if (!singleContourMode)
@@ -1118,12 +1180,13 @@ void ccClippingBoxTool::extractSlicesAndContours(bool extractSlices, bool extrac
 									extractContours,
 									static_cast<PointCoordinateType>(s_maxEdgeLength),
 									outputContours,
+									contourType,
 									static_cast<PointCoordinateType>(s_defaultGap),
 									s_multiPass,
 									s_splitContours,
-									projectOnBestFitPlane,
-									visualDebugMode,
-									generateRandomColors,
+									s_projectOnBestFitPlane,
+									s_visualDebugMode,
+									s_generateRandomColors,
 									&pDlg
 									))
 	{
@@ -1159,7 +1222,7 @@ void ccClippingBoxTool::extractSlicesAndContours(bool extractSlices, bool extrac
 			sliceGroup->addChild(slice);
 		}
 
-		QMessageBox::warning(0, "Process finished", QString("%1 slices have been generated.\n(you may have to close the tool and hide the initial cloud to see them...)").arg(sliceGroup->getChildrenNumber()));
+		QMessageBox::warning(nullptr, "Process finished", QString("%1 slices have been generated.\n(you may have to close the tool and hide the initial cloud to see them...)").arg(sliceGroup->getChildrenNumber()));
 		if (m_clipBox->getContainer().getFirstChild())
 		{
 			sliceGroup->setDisplay_recursive(m_clipBox->getContainer().getFirstChild()->getDisplay());
