@@ -59,14 +59,14 @@ ccComparisonDlg::ccComparisonDlg(	ccHObject* compEntity,
 	: QDialog(parent, Qt::Tool)
 	, Ui::ComparisonDialog()
 	, m_compEnt(compEntity)
-	, m_compCloud(0)
-	, m_compOctree(0)
+	, m_compCloud(nullptr)
+	, m_compOctree(nullptr)
 	, m_compOctreeIsPartial(false)
 	, m_compSFVisibility(false)
 	, m_refEnt(refEntity)
-	, m_refCloud(0)
-	, m_refMesh(0)
-	, m_refOctree(0)
+	, m_refCloud(nullptr)
+	, m_refMesh(nullptr)
+	, m_refOctree(nullptr)
 	, m_refOctreeIsPartial(false)
 	, m_refVisibility(false)
 	, m_compType(cpType)
@@ -129,24 +129,13 @@ ccComparisonDlg::ccComparisonDlg(	ccHObject* compEntity,
 		filterVisibilityCheckBox->setEnabled(m_refCloud && m_refCloud->isA(CC_TYPES::POINT_CLOUD) && static_cast<ccPointCloud*>(m_refCloud)->hasSensor());
 	}
 
-	connect(cancelButton,			SIGNAL(clicked()),					this,	SLOT(cancelAndExit()));
-	connect(okButton,				SIGNAL(clicked()),					this,	SLOT(applyAndExit()));
-	connect(computeButton,			SIGNAL(clicked()),					this,	SLOT(computeDistances()));
-	connect(histoButton,			SIGNAL(clicked()),					this,	SLOT(showHisto()));
-	connect(localModelComboBox,		SIGNAL(currentIndexChanged(int)),	this,	SLOT(locaModelChanged(int)));
-	connect(maxDistCheckBox,		SIGNAL(toggled(bool)),				this,	SLOT(maxDistUpdated()));
-	connect(maxSearchDistSpinBox,	SIGNAL(valueChanged(double)),		this,	SLOT(maxDistUpdated()));
-
-	//be sure to show the dialog before computing the approx distances
-	//(otherwise the progress bars appear anywhere!)
-	if (!m_noDisplay)
-	{
-		show();
-		QCoreApplication::processEvents();
-	}
-
-	//compute approximate results and unlock GUI
-	computeApproxDistances();
+	connect(cancelButton,			&QPushButton::clicked,					this,	&ccComparisonDlg::cancelAndExit);
+	connect(okButton,				&QPushButton::clicked,					this,	&ccComparisonDlg::applyAndExit);
+	connect(computeButton,			&QPushButton::clicked,					this,	&ccComparisonDlg::computeDistances);
+	connect(histoButton,			&QPushButton::clicked,					this,	&ccComparisonDlg::showHisto);
+	connect(maxDistCheckBox,		&QCheckBox::toggled,					this,	&ccComparisonDlg::maxDistUpdated);
+	connect(localModelComboBox, static_cast<void (QComboBox::*)(int)> (&QComboBox::currentIndexChanged),		this,	&ccComparisonDlg::locaModelChanged);
+	connect(maxSearchDistSpinBox, static_cast<void (QDoubleSpinBox::*)(double)> (&QDoubleSpinBox::valueChanged),this,	&ccComparisonDlg::maxDistUpdated);
 }
 
 ccComparisonDlg::~ccComparisonDlg()
@@ -402,7 +391,8 @@ bool ccComparisonDlg::computeApproxDistances()
 		ccLog::Print("[computeApproxDistances] Time: %3.2f s.", elapsedTime_ms / 1.0e3);
 
 		//display approx. dist. statistics
-		ScalarType mean,variance;
+		ScalarType mean;
+		ScalarType variance;
 		sf->computeMinAndMax();
 		sf->computeMeanAndVariance(mean,&variance);
 
@@ -454,7 +444,7 @@ bool ccComparisonDlg::computeApproxDistances()
 	}
 
 	computeButton->setEnabled(true);
-	preciseGroupBox->setEnabled(true);
+	preciseResultsTabWidget->setEnabled(true);
 	//we don't let the user leave with approximate distances!!!
 	okButton->setEnabled(false);
 
@@ -505,7 +495,7 @@ int ccComparisonDlg::determineBestOctreeLevel(double maxSearchDist)
 
 	//if the reference is a mesh
 	double meanTriangleSurface = 1.0;
-	CCLib::GenericIndexedMesh* mesh = 0;
+	CCLib::GenericIndexedMesh* mesh = nullptr;
 	if (!m_refOctree)
 	{
 		if (!m_refMesh)
@@ -836,7 +826,7 @@ bool ccComparisonDlg::computeDistances()
 			}
 			c2cParams.maxSearchDist = maxSearchDist;
 			c2cParams.multiThread = multiThread;
-			c2cParams.CPSet = 0;
+			c2cParams.CPSet = nullptr;
 		}
 		
 		result = CCLib::DistanceComputationTools::computeCloud2CloudDistance(	m_compCloud,
@@ -883,7 +873,8 @@ bool ccComparisonDlg::computeDistances()
 		ccLog::Print("[ComputeDistances] Time: %3.2f s.",static_cast<double>(elapsedTime_ms)/1.0e3);
 
 		//display some statics about the computed distances
-		ScalarType mean,variance;
+		ScalarType mean;
+		ScalarType variance;
 		sf->computeMinAndMax();
 		sf->computeMeanAndVariance(mean, &variance);
 		ccLog::Print("[ComputeDistances] Mean distance = %f / std deviation = %f",mean,sqrt(variance));
