@@ -38,7 +38,7 @@
 #endif
 
 //System
-#include <assert.h>
+#include <cassert>
 #include <cmath>
 #include <set>
 
@@ -49,28 +49,30 @@ enum HullPointFlags {	POINT_NOT_USED	= 0,
 						POINT_FROZEN	= 3,
 };
 
-typedef CCLib::PointProjectionTools::IndexedCCVector2 Vertex2D;
-typedef std::list<Vertex2D*>::iterator VertexIterator;
-typedef std::list<Vertex2D*>::const_iterator ConstVertexIterator;
+using Vertex2D = CCLib::PointProjectionTools::IndexedCCVector2;
+using VertexIterator = std::list<Vertex2D *>::iterator;
+using ConstVertexIterator = std::list<Vertex2D *>::const_iterator;
 
-struct Edge
+namespace 
 {
-	Edge() : nearestPointIndex(0), nearestPointSquareDist(-1.0f) {}
-	
-	Edge(const VertexIterator& A, unsigned _nearestPointIndex, float _nearestPointSquareDist)
-		: itA(A)
-		, nearestPointIndex(_nearestPointIndex)
-		, nearestPointSquareDist(_nearestPointSquareDist)
-	{}
-
-	//operator
-	inline bool operator< (const Edge& e) const { return nearestPointSquareDist < e.nearestPointSquareDist; }
-
-	VertexIterator itA;
-	unsigned nearestPointIndex;
-	float nearestPointSquareDist;
-};
-
+	struct Edge
+	{
+		Edge() : nearestPointIndex(0), nearestPointSquareDist(-1.0f) {}
+		
+		Edge(const VertexIterator& A, unsigned _nearestPointIndex, float _nearestPointSquareDist)
+			: itA(A)
+			, nearestPointIndex(_nearestPointIndex)
+			, nearestPointSquareDist(_nearestPointSquareDist)
+		{}
+		
+		//operator
+		inline bool operator< (const Edge& e) const { return nearestPointSquareDist < e.nearestPointSquareDist; }
+		
+		VertexIterator itA;
+		unsigned nearestPointIndex;
+		float nearestPointSquareDist;
+	};
+}
 
 //! Finds the nearest (available) point to an edge
 /** \return The nearest point distance (or -1 if no point was found!)
@@ -238,7 +240,8 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 	//hack: compute the theoretical 'minimal' edge length
 	PointCoordinateType minSquareEdgeLength = 0;
 	{
-		CCVector2 minP, maxP;
+		CCVector2 minP;
+		CCVector2 maxP;
 		for (size_t i=0; i<pointCount; ++i)
 		{
 			const Vertex2D& P = points[i];
@@ -337,26 +340,27 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 
 	//DEBUG MECHANISM
 	ccContourExtractorDlg debugDialog;
-	ccPointCloud* debugCloud = 0;
-	ccPolyline* debugContour = 0;
-	ccPointCloud* debugContourVertices = 0;
+	ccPointCloud* debugCloud = nullptr;
+	ccPolyline* debugContour = nullptr;
+	ccPointCloud* debugContourVertices = nullptr;
+	
 	if (enableVisualDebugMode)
 	{
 		debugDialog.init();
-		debugDialog.setGeometry(50,50,800,600);
+		debugDialog.setGeometry(50, 50, 800, 600);
 		debugDialog.show();
 
 		//create point cloud with all (2D) input points
 		{
 			debugCloud = new ccPointCloud;
 			debugCloud->reserve(pointCount);
-			for (size_t i=0; i<pointCount; ++i)
+			for (size_t i = 0; i < pointCount; ++i)
 			{
 				const Vertex2D& P = points[i];
-				debugCloud->addPoint(CCVector3(P.x,P.y,0));
+				debugCloud->addPoint(CCVector3(P.x, P.y, 0));
 			}
 			debugCloud->setPointSize(3);
-			debugDialog.addToDisplay(debugCloud,false); //the window will take care of deleting this entity!
+			debugDialog.addToDisplay(debugCloud, false); //the window will take care of deleting this entity!
 		}
 
 		//create polyline
@@ -371,13 +375,13 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 			for (VertexIterator itA = hullPoints.begin(); itA != hullPoints.end(); ++itA, ++index)
 			{
 				const Vertex2D* P = *itA;
-				debugContourVertices->addPoint(CCVector3(P->x,P->y,0));
+				debugContourVertices->addPoint(CCVector3(P->x, P->y, 0));
 				debugContour->addPointIndex(index/*(*itA)->index*/);
 			}
 			debugContour->setColor(ccColor::red);
 			debugContourVertices->setEnabled(false);
 			debugContour->setClosed(contourType == FULL);
-			debugDialog.addToDisplay(debugContour,false); //the window will take care of deleting this entity!
+			debugDialog.addToDisplay(debugContour, false); //the window will take care of deleting this entity!
 		}
 
 		//set zoom
@@ -414,7 +418,7 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 				--initEdgeCount;
 
 			VertexIterator itB = hullPoints.begin();
-			for (size_t i=0; i<initEdgeCount; ++i)
+			for (size_t i = 0; i < initEdgeCount; ++i)
 			{
 				VertexIterator itA = itB; ++itB;
 				if (itB == hullPoints.end())
@@ -468,14 +472,15 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 				assert(pointFlags[P.index] == POINT_NOT_USED); //we don't consider already used points!
 
 				//create labels
-				cc2DLabel* edgeLabel = 0;
-				cc2DLabel* label = 0;
+				cc2DLabel* edgeLabel = nullptr;
+				cc2DLabel* label = nullptr;
+				
 				if (enableVisualDebugMode && !debugDialog.isSkipped())
 				{
 					edgeLabel = new cc2DLabel("edge");
 					unsigned indexA = 0;
 					unsigned indexB = 0;
-					for (size_t i=0; i<pointCount; ++i)
+					for (size_t i = 0; i < pointCount; ++i)
 					{
 						const Vertex2D& P = points[i];
 						if (&P == *itA)
@@ -483,15 +488,15 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 						if (&P == *itB)
 							indexB = static_cast<unsigned>(i);
 					}
-					edgeLabel->addPoint(debugCloud,indexA);
-					edgeLabel->addPoint(debugCloud,indexB);
+					edgeLabel->addPickedPoint(debugCloud, indexA);
+					edgeLabel->addPickedPoint(debugCloud, indexB);
 					edgeLabel->setVisible(true);
 					edgeLabel->setDisplayedIn2D(false);
 					debugDialog.addToDisplay(edgeLabel);
 					debugDialog.refresh();
 
 					label = new cc2DLabel("nearest point");
-					label->addPoint(debugCloud,e.nearestPointIndex);
+					label->addPickedPoint(debugCloud, e.nearestPointIndex);
 					label->setVisible(true);
 					label->setSelected(true);
 					debugDialog.addToDisplay(label);
@@ -670,7 +675,7 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 					assert(enableVisualDebugMode);
 					debugDialog.removFromDisplay(label);
 					delete label;
-					label = 0;
+					label = nullptr;
 					//debugDialog.refresh();
 				}
 
@@ -679,7 +684,7 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 					assert(enableVisualDebugMode);
 					debugDialog.removFromDisplay(edgeLabel);
 					delete edgeLabel;
-					edgeLabel = 0;
+					edgeLabel = nullptr;
 					//debugDialog.refresh();
 				}
 			}
@@ -697,7 +702,7 @@ bool ccContourExtractor::ExtractConcaveHull2D(	std::vector<Vertex2D>& points,
 	return true;
 }
 
-typedef std::list<Vertex2D*> Hull2D;
+using Hull2D = std::list<Vertex2D *>;
 
 ccPolyline* ccContourExtractor::ExtractFlatContour(	CCLib::GenericIndexedCloudPersist* points,
 													bool allowMultiPass,
@@ -710,22 +715,38 @@ ccPolyline* ccContourExtractor::ExtractFlatContour(	CCLib::GenericIndexedCloudPe
 													double maxAngleDeg/*=0.0*/)
 {
 	assert(points);
+	
 	if (!points)
-		return 0;
+		return nullptr;
+	
 	unsigned ptsCount = points->size();
+	
 	if (ptsCount < 3)
-		return 0;
+		return nullptr;
 
 	CCLib::Neighbourhood Yk(points);
-	CCVector3 O, X, Y; //local base
-	bool useOXYasBase = false;
+	
+	//local base
+	CCVector3 O;
+	CCVector3 X;
+	CCVector3 Y;
+	
+	CCLib::Neighbourhood::InputVectorsUsage vectorsUsage = CCLib::Neighbourhood::None;
 
 	//we project the input points on a plane
 	std::vector<Vertex2D> points2D;
-	PointCoordinateType* planeEq = 0;
+	PointCoordinateType* planeEq = nullptr;
+	
+	if (preferredUpDir != nullptr)
+	{
+		Y = CCVector3(preferredUpDir);
+		vectorsUsage = CCLib::Neighbourhood::UseYAsUpDir;
+	}
+
 	//if the user has specified a default direction, we'll use it as 'projecting plane'
-	PointCoordinateType preferredPlaneEq[4] = {0, 0, 1, 0};
-	if (preferredNormDim != 0)
+	PointCoordinateType preferredPlaneEq[4] = { 0, 0, 1, 0 };
+
+	if (preferredNormDim != nullptr)
 	{
 		const CCVector3* G = points->getPoint(0); //any point through which the plane passes is ok
 		preferredPlaneEq[0] = preferredNormDim[0];
@@ -735,25 +756,27 @@ ccPolyline* ccContourExtractor::ExtractFlatContour(	CCLib::GenericIndexedCloudPe
 		preferredPlaneEq[3] = CCVector3::vdot(G->u, preferredPlaneEq);
 		planeEq = preferredPlaneEq;
 
-		if (preferredUpDir != 0)
+		if (preferredUpDir != nullptr)
 		{
 			O = *G;
-			Y = CCVector3(preferredUpDir);
+			//Y = CCVector3(preferredUpDir); //already done above
 			X = Y.cross(CCVector3(preferredNormDim));
-			useOXYasBase = true;
+			vectorsUsage = CCLib::Neighbourhood::UseOXYasBase;
 		}
 	}
 
-	if (!Yk.projectPointsOn2DPlane<Vertex2D>(points2D, planeEq, &O, &X, &Y, useOXYasBase))
+	if (!Yk.projectPointsOn2DPlane<Vertex2D>(points2D, planeEq, &O, &X, &Y, vectorsUsage))
 	{
 		ccLog::Warning("[ExtractFlatContour] Failed to project the points on the LS plane (not enough memory?)!");
-		return 0;
+		return nullptr;
 	}
 
 	//update the points indexes (not done by Neighbourhood::projectPointsOn2DPlane)
 	{
 		for (unsigned i = 0; i < ptsCount; ++i)
+		{
 			points2D[i].index = i;
+		}
 	}
 
 	//try to get the points on the convex/concave hull to build the contour and the polygon
@@ -767,7 +790,7 @@ ccPolyline* ccContourExtractor::ExtractFlatContour(	CCLib::GenericIndexedCloudPe
 								maxAngleDeg))
 	{
 		ccLog::Warning("[ExtractFlatContour] Failed to compute the convex hull of the input points!");
-		return 0;
+		return nullptr;
 	}
 
 	if (originalPointIndexes)
@@ -780,12 +803,14 @@ ccPolyline* ccContourExtractor::ExtractFlatContour(	CCLib::GenericIndexedCloudPe
 		{
 			//not enough memory
 			ccLog::Error("[ExtractFlatContour] Not enough memory!");
-			return 0;
+			return nullptr;
 		}
 
-		unsigned i=0;
+		unsigned i = 0;
 		for (Hull2D::const_iterator it = hullPoints.begin(); it != hullPoints.end(); ++it, ++i)
+		{
 			(*originalPointIndexes)[i] = (*it)->index;
+		}
 	}
 
 	unsigned hullPtsCount = static_cast<unsigned>(hullPoints.size());
@@ -796,14 +821,17 @@ ccPolyline* ccContourExtractor::ExtractFlatContour(	CCLib::GenericIndexedCloudPe
 		if (!contourVertices->reserve(hullPtsCount))
 		{
 			delete contourVertices;
-			contourVertices = 0;
+			contourVertices = nullptr;
 			ccLog::Error("[ExtractFlatContour] Not enough memory!");
-			return 0;
+			return nullptr;
 		}
 
 		//projection on the LS plane (in 3D)
 		for (Hull2D::const_iterator it = hullPoints.begin(); it != hullPoints.end(); ++it)
+		{
 			contourVertices->addPoint(O + X*(*it)->x + Y*(*it)->y);
+		}
+		
 		contourVertices->setName("vertices");
 		contourVertices->setEnabled(false);
 	}
@@ -821,7 +849,7 @@ ccPolyline* ccContourExtractor::ExtractFlatContour(	CCLib::GenericIndexedCloudPe
 	else
 	{
 		delete contourPolyline;
-		contourPolyline = 0;
+		contourPolyline = nullptr;
 		ccLog::Warning("[ExtractFlatContour] Not enough memory to create the contour polyline!");
 	}
 
@@ -832,14 +860,16 @@ bool ccContourExtractor::ExtractFlatContour(CCLib::GenericIndexedCloudPersist* p
 											bool allowMultiPass,
 											PointCoordinateType maxEdgeLength,
 											std::vector<ccPolyline*>& parts,
+											ContourType contourType/*=FULL*/,
 											bool allowSplitting/*=true*/,
-											const PointCoordinateType* preferredDim/*=0*/,
+											const PointCoordinateType* preferredNormDir/*=nullptr*/,
+											const PointCoordinateType* preferredUpDir/*=nullptr*/,
 											bool enableVisualDebugMode/*=false*/)
 {
 	parts.clear();
 
 	//extract whole contour
-	ccPolyline* basePoly = ExtractFlatContour(points, allowMultiPass, maxEdgeLength, preferredDim, 0, FULL, 0, enableVisualDebugMode);
+	ccPolyline* basePoly = ExtractFlatContour(points, allowMultiPass, maxEdgeLength, preferredNormDir, preferredUpDir, contourType, nullptr, enableVisualDebugMode);
 	if (!basePoly)
 	{
 		return false;
@@ -854,7 +884,7 @@ bool ccContourExtractor::ExtractFlatContour(CCLib::GenericIndexedCloudPersist* p
 	bool success = basePoly->split(maxEdgeLength, parts);
 
 	delete basePoly;
-	basePoly = 0;
+	basePoly = nullptr;
 
 	return success;
 }
