@@ -67,8 +67,9 @@
 #include "ccUtils.h"
 
 // This is included only for temporarily removing an object from the tree.
-//	TODO figure out a cleaner way to do this without having to include all of mainwindow.h
-#include "mainwindow.h"
+#include "ccMainAppInterface.h"
+
+#include <unordered_set>
 
 
 namespace ccEntityAction
@@ -97,7 +98,7 @@ namespace ccEntityAction
 	// Colours
 	bool setColor(ccHObject::Container selectedEntities, bool colorize, QWidget *parent)
 	{
-		QColor colour = QColorDialog::getColor(Qt::white, parent);
+		QColor colour = QColorDialog::getColor(Qt::white, parent, QString(), QColorDialog::ShowAlphaChannel);
 		
 		if (!colour.isValid())
 			return false;
@@ -138,11 +139,12 @@ namespace ccEntityAction
 				{
 					cloud->colorize(static_cast<float>(colour.redF()),
 									static_cast<float>(colour.greenF()),
-									static_cast<float>(colour.blueF()) );
+									static_cast<float>(colour.blueF()),
+									static_cast<float>(colour.alphaF()));
 				}
 				else
 				{
-					cloud->setRGBColor(	ccColor::FromQColor(colour) );
+					cloud->setColor(ccColor::FromQColora(colour));
 				}
 				cloud->showColors(true);
 				cloud->showSF(false); //just in case
@@ -239,7 +241,8 @@ namespace ccEntityAction
 		else if (ramp == ccColorGradientDlg::TwoColors)
 		{
 			colorScale = ccColorScale::Create("Temp scale");
-			QColor first,second;
+			QColor first;
+			QColor second;
 			dlg.getColors(first,second);
 			colorScale->insert(ccColorScaleElement(0.0, first), false);
 			colorScale->insert(ccColorScaleElement(1.0, second), true);
@@ -535,9 +538,6 @@ namespace ccEntityAction
 						continue;
 					}
 					
-					//ColorCompType C[3]={MAX_COLOR_COMP,MAX_COLOR_COMP,MAX_COLOR_COMP};
-					//mesh->getColorFromMaterial(triIndex,*P,C,withRGB);
-					//cloud->addRGBColor(C);
 					if (mesh->convertMaterialsToVertexColors())
 					{
 						mesh->showColors(true);
@@ -561,7 +561,8 @@ namespace ccEntityAction
 		QString defaultSFName("Intensity");
 
 		bool useCustomIntensityRange = false;
-		static double s_minI = 0.0, s_maxI = 1.0;
+		static double s_minI = 0.0;
+		static double s_maxI = 1.0;
 		if (QMessageBox::question(parent, "Intensity range", "Do you want to define the theoretical intensity range (yes)\nor use the actual one (no)?", QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
 		{
 			ccAskTwoDoubleValuesDlg atdvDlg("Min", "Max", -1000000.0, 1000000.0, s_minI, s_maxI, 3, "Theroetical intensity", parent);
@@ -907,7 +908,7 @@ namespace ccEntityAction
 				//if there is no displayed SF --> nothing to do!
 				if (pc->getCurrentDisplayedScalarField())
 				{
-					if (pc->setRGBColorWithCurrentScalarField(mixWithExistingColors))
+					if (pc->convertCurrentScalarFieldToColors(mixWithExistingColors))
 					{
 						ent->showColors(true);
 						ent->showSF(false); //just in case
@@ -938,7 +939,7 @@ namespace ccEntityAction
 			return false;
 		Q_ASSERT(s_randomColorsNumber > 1);
 		
-		ColorsTableType* randomColors = new ColorsTableType;
+		RGBAColorsTableType* randomColors = new RGBAColorsTableType;
 		if (!randomColors->reserveSafe(static_cast<unsigned>(s_randomColorsNumber)))
 		{
 			ccConsole::Error("Not enough memory!");
@@ -948,7 +949,7 @@ namespace ccEntityAction
 		//generate random colors
 		for (int i = 0; i < s_randomColorsNumber; ++i)
 		{
-			ccColor::Rgb col = ccColor::Generator::Random();
+			ccColor::Rgba col(ccColor::Generator::Random(), ccColor::MAX);
 			randomColors->addElement(col);
 		}
 		
@@ -1194,7 +1195,69 @@ namespace ccEntityAction
 		
 		return true;
 	}
-	
+
+	bool	exportNormalToSF(const ccHObject::Container& selectedEntities, QWidget* parent, bool* exportDimensions/*=nullptr*/)
+	{
+		bool exportDims[3] = { false, false, false };
+		if (exportDimensions)
+		{
+			exportDims[0] = exportDimensions[0];
+			exportDims[1] = exportDimensions[1];
+			exportDims[2] = exportDimensions[2];
+		}
+		else
+		{
+			//ask the user
+			ccExportCoordToSFDlg ectsDlg(parent);
+			ectsDlg.setWindowTitle(QObject::tr("Export normals to SF(s)"));
+
+			if (!ectsDlg.exec())
+			{
+				return false;
+			}
+
+			exportDims[0] = ectsDlg.exportX();
+			exportDims[1] = ectsDlg.exportY();
+			exportDims[2] = ectsDlg.exportZ();
+		}
+
+		if (!exportDims[0] && !exportDims[1] && !exportDims[2]) //nothing to do?!
+		{
+			return false;
+		}
+
+		//for each selected cloud (or vertices set)
+		for (ccHObject* entity : selectedEntities)
+		{
+			ccPointCloud* pc = ccHObjectCaster::ToPointCloud(entity);
+			if (pc == nullptr)
+			{
+				// TODO do something with error?
+				continue;
+			}
+
+			if (!pc->hasNormals())
+			{
+				ccLog::Warning(QString("Cloud '%1' has no normals").arg(pc->getName()));
+				continue;
+			}
+
+			if (!pc->exportNormalToSF(exportDims))
+			{
+				ccLog::Error("The process failed!");
+				return true; //true because we want the UI to be updated anyway
+			}
+
+			if (entity != pc)
+			{
+				entity->showSF(true); //for meshes
+			}
+			entity->prepareDisplayForRefresh_recursive();
+		}
+
+		return true;
+	}
+
 	bool	sfArithmetic(const ccHObject::Container &selectedEntities, QWidget *parent)
 	{
 		Q_ASSERT(!selectedEntities.empty());
@@ -1252,15 +1315,17 @@ namespace ccEntityAction
 		const bool exportR = dialog.getRStatus();
 		const bool exportG = dialog.getGStatus();
 		const bool exportB = dialog.getBStatus();
-		const bool exportC = dialog.getCompositeStatus();
+		const bool exportAlpha = dialog.getAlphaStatus();
+		const bool exportComposite = dialog.getCompositeStatus();
 		
 		for (const auto cloud : clouds)
 		{
-			std::vector<ccScalarField*> fields(4);
+			std::vector<ccScalarField*> fields(5, nullptr);
 			fields[0] = (exportR ? new ccScalarField(qPrintable(GetFirstAvailableSFName(cloud,"R"))) : nullptr);
 			fields[1] = (exportG ? new ccScalarField(qPrintable(GetFirstAvailableSFName(cloud,"G"))) : nullptr);
 			fields[2] = (exportB ? new ccScalarField(qPrintable(GetFirstAvailableSFName(cloud,"B"))) : nullptr);
-			fields[3] = (exportC ? new ccScalarField(qPrintable(GetFirstAvailableSFName(cloud,"Composite"))) : nullptr);
+			fields[3] = (exportAlpha ? new ccScalarField(qPrintable(GetFirstAvailableSFName(cloud, "Alpha"))) : nullptr);
+			fields[4] = (exportComposite ? new ccScalarField(qPrintable(GetFirstAvailableSFName(cloud,"Composite"))) : nullptr);
 			
 			//try to instantiate memory for each field
 			unsigned count = cloud->size();
@@ -1277,16 +1342,18 @@ namespace ccEntityAction
 			//export points
 			for (unsigned j = 0; j < cloud->size(); ++j)
 			{
-				const ccColor::Rgb& rgb = cloud->getPointColor(j);
+				const ccColor::Rgba& col = cloud->getPointColor(j);
 				
 				if (fields[0])
-					fields[0]->addElement(rgb.r);
+					fields[0]->addElement(col.r);
 				if (fields[1])
-					fields[1]->addElement(rgb.g);
+					fields[1]->addElement(col.g);
 				if (fields[2])
-					fields[2]->addElement(rgb.b);
+					fields[2]->addElement(col.b);
 				if (fields[3])
-					fields[3]->addElement(static_cast<ScalarType>(rgb.r + rgb.g + rgb.b) / 3);
+					fields[3]->addElement(col.a);
+				if (fields[4])
+					fields[4]->addElement(static_cast<ScalarType>(col.r + col.g + col.b) / 3);
 			}
 			
 			QString fieldsStr;
@@ -1385,6 +1452,8 @@ namespace ccEntityAction
 			return false;
 		}
 		
+		static const QString s_NormalScaleKey("Normal scale");
+
 		//look for clouds and meshes
 		std::vector<ccPointCloud*> clouds;
 		bool withScanGrid = false;
@@ -1401,19 +1470,40 @@ namespace ccEntityAction
 					ccPointCloud* cloud = static_cast<ccPointCloud*>(entity);
 					clouds.push_back(cloud);
 					
-					if (cloud->gridCount() > 0)
+					if (!withScanGrid)
 					{
-						withScanGrid = true;
+						withScanGrid = (cloud->gridCount() > 0);
 					}
-					for (unsigned i = 0; i < cloud->getChildrenNumber(); ++i)
+
+					if (!withSensor)
 					{
-						if (cloud->hasSensor())
+						for (unsigned i = 0; i < cloud->getChildrenNumber(); ++i)
 						{
-							withSensor = true;
+							if (cloud->hasSensor())
+							{
+								withSensor = true;
+								break; //no need to look anyfurther
+							}
 						}
 					}
 
-					if (defaultRadius == 0)
+					//does the cloud have a former radius value saved as meta-data?
+					if (cloud->hasMetaData(s_NormalScaleKey))
+					{
+						bool ok = false;
+						double formerRadius = cloud->getMetaData(s_NormalScaleKey).toDouble(&ok);
+						if (ok)
+						{
+							//remember the largest radius
+							defaultRadius = std::max(defaultRadius, static_cast<PointCoordinateType>(formerRadius));
+						}
+						else
+						{
+							assert(false);
+						}
+					}
+
+					if (defaultRadius == 0.0)
 					{
 						//default radius
 						defaultRadius = ccNormalVectors::GuessNaiveRadius(cloud);
@@ -1531,6 +1621,11 @@ namespace ccEntityAction
 					//compute normals with the octree
 					normalsAlreadyOriented = orientNormals && (preferredOrientation != ccNormalVectors::UNDEFINED);
 					result = cloud->computeNormalsWithOctree(model, orientNormals ? preferredOrientation : ccNormalVectors::UNDEFINED, defaultRadius, &pDlg);
+					if (result)
+					{
+						//save the normal computation radius as meta-data
+						cloud->setMetaData(s_NormalScaleKey, defaultRadius);
+					}
 				}
 				
 				//do we need to orient the normals? (this may have been already done if 'orientNormalsForThisCloud' is true)
@@ -1611,8 +1706,8 @@ namespace ccEntityAction
 				Q_ASSERT(mesh != nullptr);
 				
 				//we remove temporarily the mesh as its normals may be removed (and they can be a child object)
-				MainWindow* instance = dynamic_cast<MainWindow*>(parent);
-				MainWindow::ccHObjectContext objContext;
+				ccMainAppInterface* instance = dynamic_cast<ccMainAppInterface*>(parent);
+				ccMainAppInterface::ccHObjectContext objContext;
 				if (instance)
 					objContext = instance->removeObjectTemporarilyFromDBTree(mesh);
 				mesh->clearTriNormals();
@@ -1723,7 +1818,7 @@ namespace ccEntityAction
 		return true;
 	}
 	
-	bool	orientNormalsMST(const ccHObject::Container &selectedEntities, QWidget *parent)
+	bool	orientNormalsMST(const ccHObject::Container &selectedEntities, QWidget* parent = nullptr)
 	{
 		if (selectedEntities.empty())
 		{
@@ -1787,8 +1882,9 @@ namespace ccEntityAction
 	
 	bool	convertNormalsTo(const ccHObject::Container &selectedEntities, NORMAL_CONVERSION_DEST dest)
 	{
-		unsigned errorCount = 0;
-		
+		size_t errorCount = 0;
+		size_t successCount = 0;
+
 		size_t selNum = selectedEntities.size();
 		for (size_t i = 0; i < selNum; ++i)
 		{
@@ -1861,6 +1957,7 @@ namespace ccEntityAction
 								dipDirSF->setColorScale(dipDirScale);
 								ccCloud->setCurrentDisplayedScalarField(dipDirSFIndex); //dip dir. seems more interesting by default
 								ccCloud->showSF(true);
+								++successCount;
 							}
 							else
 							{
@@ -1896,7 +1993,7 @@ namespace ccEntityAction
 			ccConsole::Error("Error(s) occurred! (see console)");
 		}
 		
-		return true;
+		return (successCount != 0);
 	}
 	
 	//////////
@@ -1961,8 +2058,8 @@ namespace ccEntityAction
 		{
 			//we temporarily detach entity, as it may undergo
 			//"severe" modifications (octree deletion, etc.) --> see ccPointCloud::computeOctree
-			MainWindow* instance = dynamic_cast<MainWindow*>(parent);
-			MainWindow::ccHObjectContext objContext;
+			ccMainAppInterface* instance = dynamic_cast<ccMainAppInterface*>(parent);
+			ccMainAppInterface::ccHObjectContext objContext;
 			if (instance)
 				objContext = instance->removeObjectTemporarilyFromDBTree(cloud);
 
@@ -2049,8 +2146,8 @@ namespace ccEntityAction
 				{
 					mesh->showNormals(false);
 					
-					MainWindow* instance = dynamic_cast<MainWindow*>(parent);
-					MainWindow::ccHObjectContext objContext;
+					ccMainAppInterface* instance = dynamic_cast<ccMainAppInterface*>(parent);
+					ccMainAppInterface::ccHObjectContext objContext;
 					if (instance)
 						objContext = instance->removeObjectTemporarilyFromDBTree(mesh);
 					mesh->clearTriNormals();
@@ -2382,7 +2479,8 @@ namespace ccEntityAction
 					case 1: //WEIBULL
 					{
 						CCLib::WeibullDistribution* weibull = static_cast<CCLib::WeibullDistribution*>(distrib);
-						ScalarType a, b;
+						ScalarType a;
+						ScalarType b;
 						weibull->getParameters(a, b);
 						description = QString("a = %1 / b = %2 / shift = %3").arg(a, 0, 'f', precision).arg(b, 0, 'f', precision).arg(weibull->getValueShift(), 0, 'f', precision);
 						ccLog::Print(QString("[Distribution fitting] Additional Weibull distrib. parameters: mode = %1 / skewness = %2").arg(weibull->computeMode()).arg(weibull->computeSkewness()));
