@@ -20,6 +20,7 @@
 //Local
 #include "mainwindow.h"
 #include "ccItemSelectionDlg.h"
+#include "ccReservedIDs.h"
 
 //CCLib
 #include <ManualSegmentationTools.h>
@@ -50,8 +51,8 @@ ccGraphicalSegmentationTool::ccGraphicalSegmentationTool(QWidget* parent)
 	, Ui::GraphicalSegmentationDlg()
 	, m_somethingHasChanged(false)
 	, m_state(0)
-	, m_segmentationPoly(0)
-	, m_polyVertices(0)
+	, m_segmentationPoly(nullptr)
+	, m_polyVertices(nullptr)
 	, m_rectangularSelection(false)
 	, m_deleteHiddenParts(false)
 {
@@ -60,20 +61,20 @@ ccGraphicalSegmentationTool::ccGraphicalSegmentationTool(QWidget* parent)
 
 	setupUi(this);
 
-	connect(inButton,							SIGNAL(clicked()),		this,	SLOT(segmentIn()));
-	connect(outButton,							SIGNAL(clicked()),		this,	SLOT(segmentOut()));
-	connect(razButton,							SIGNAL(clicked()),		this,	SLOT(reset()));
-	connect(validButton,						SIGNAL(clicked()),		this,	SLOT(apply()));
-	connect(validAndDeleteButton,				SIGNAL(clicked()),		this,	SLOT(applyAndDelete()));
-	connect(cancelButton,						SIGNAL(clicked()),		this,	SLOT(cancel()));
-	connect(pauseButton,						SIGNAL(toggled(bool)),	this,	SLOT(pauseSegmentationMode(bool)));
+	connect(inButton,							&QToolButton::clicked,		this,	&ccGraphicalSegmentationTool::segmentIn);
+	connect(outButton,							&QToolButton::clicked,		this,	&ccGraphicalSegmentationTool::segmentOut);
+	connect(razButton,							&QToolButton::clicked,		this,	&ccGraphicalSegmentationTool::reset);
+	connect(validButton,						&QToolButton::clicked,		this,	&ccGraphicalSegmentationTool::apply);
+	connect(validAndDeleteButton,				&QToolButton::clicked,		this,	&ccGraphicalSegmentationTool::applyAndDelete);
+	connect(cancelButton,						&QToolButton::clicked,		this,	&ccGraphicalSegmentationTool::cancel);
+	connect(pauseButton,						&QToolButton::toggled,		this,	&ccGraphicalSegmentationTool::pauseSegmentationMode);
 
 	//selection modes
-	connect(actionSetPolylineSelection,			SIGNAL(triggered()),	this,	SLOT(doSetPolylineSelection()));
-	connect(actionSetRectangularSelection,		SIGNAL(triggered()),	this,	SLOT(doSetRectangularSelection()));
+	connect(actionSetPolylineSelection,			&QAction::triggered,	this,	&ccGraphicalSegmentationTool::doSetPolylineSelection);
+	connect(actionSetRectangularSelection,		&QAction::triggered,	this,	&ccGraphicalSegmentationTool::doSetRectangularSelection);
 	//import/export options
-	connect(actionUseExistingPolyline,			SIGNAL(triggered()),	this,	SLOT(doActionUseExistingPolyline()));
-	connect(actionExportSegmentationPolyline,	SIGNAL(triggered()),	this,	SLOT(doExportSegmentationPolyline()));
+	connect(actionUseExistingPolyline,			&QAction::triggered,	this,	&ccGraphicalSegmentationTool::doActionUseExistingPolyline);
+	connect(actionExportSegmentationPolyline,	&QAction::triggered,	this,	&ccGraphicalSegmentationTool::doExportSegmentationPolyline);
 
 	//add shortcuts
 	addOverridenShortcut(Qt::Key_Space);  //space bar for the "pause" button
@@ -83,7 +84,7 @@ ccGraphicalSegmentationTool::ccGraphicalSegmentationTool(QWidget* parent)
 	addOverridenShortcut(Qt::Key_Tab);    //tab key to switch between rectangular and polygonal selection modes
 	addOverridenShortcut(Qt::Key_I);      //'I' key for the "segment in" button
 	addOverridenShortcut(Qt::Key_O);      //'O' key for the "segment out" button
-	connect(this, SIGNAL(shortcutTriggered(int)), this, SLOT(onShortcutTriggered(int)));
+	connect(this, &ccOverlayDialog::shortcutTriggered, this, &ccGraphicalSegmentationTool::onShortcutTriggered);
 
 	QMenu* selectionModeMenu = new QMenu(this);
 	selectionModeMenu->addAction(actionSetPolylineSelection);
@@ -96,8 +97,8 @@ ccGraphicalSegmentationTool::ccGraphicalSegmentationTool(QWidget* parent)
 	importExportMenu->addAction(actionExportSegmentationPolyline);
 	loadSaveToolButton->setMenu(importExportMenu);
 
-	m_polyVertices = new ccPointCloud("vertices");
-	m_segmentationPoly = new ccPolyline(m_polyVertices);
+	m_polyVertices = new ccPointCloud("vertices", static_cast<unsigned>(ReservedIDs::INTERACTIVE_SEGMENTATION_TOOL_POLYLINE_VERTICES));
+	m_segmentationPoly = new ccPolyline(m_polyVertices, static_cast<unsigned>(ReservedIDs::INTERACTIVE_SEGMENTATION_TOOL_POLYLINE));
 	m_segmentationPoly->setForeground(true);
 	m_segmentationPoly->setColor(ccColor::green);
 	m_segmentationPoly->showColors(true);
@@ -122,11 +123,11 @@ ccGraphicalSegmentationTool::~ccGraphicalSegmentationTool()
 {
 	if (m_segmentationPoly)
 		delete m_segmentationPoly;
-	m_segmentationPoly = 0;
+	m_segmentationPoly = nullptr;
 
 	if (m_polyVertices)
 		delete m_polyVertices;
-	m_polyVertices = 0;
+	m_polyVertices = nullptr;
 }
 
 void ccGraphicalSegmentationTool::onShortcutTriggered(int key)
@@ -184,16 +185,16 @@ bool ccGraphicalSegmentationTool::linkWith(ccGLWindow* win)
 		oldWin->disconnect(this);
 		if (m_segmentationPoly)
 		{
-			m_segmentationPoly->setDisplay(0);
+			m_segmentationPoly->setDisplay(nullptr);
 		}
 	}
 	
 	if (m_associatedWin)
 	{
-		connect(m_associatedWin, SIGNAL(leftButtonClicked(int, int)), this, SLOT(addPointToPolyline(int, int)));
-		connect(m_associatedWin, SIGNAL(rightButtonClicked(int, int)), this, SLOT(closePolyLine(int, int)));
-		connect(m_associatedWin, SIGNAL(mouseMoved(int, int, Qt::MouseButtons)), this, SLOT(updatePolyLine(int, int, Qt::MouseButtons)));
-		connect(m_associatedWin, SIGNAL(buttonReleased()), this, SLOT(closeRectangle()));
+		connect(m_associatedWin, &ccGLWindow::leftButtonClicked,	this, &ccGraphicalSegmentationTool::addPointToPolyline);
+		connect(m_associatedWin, &ccGLWindow::rightButtonClicked,	this, &ccGraphicalSegmentationTool::closePolyLine);
+		connect(m_associatedWin, &ccGLWindow::mouseMoved,			this, &ccGraphicalSegmentationTool::updatePolyLine);
+		connect(m_associatedWin, &ccGLWindow::buttonReleased,		this, &ccGraphicalSegmentationTool::closeRectangle);
 
 		if (m_segmentationPoly)
 		{
@@ -629,6 +630,28 @@ void ccGraphicalSegmentationTool::segment(bool keepPointsInside)
 	const double half_w = camera.viewport[2] / 2.0;
 	const double half_h = camera.viewport[3] / 2.0;
 
+	//check if the polyline is totally inside the frustum or not
+	bool polyInsideFrustum = true;
+	{
+		int vertexCount = static_cast<int>(m_segmentationPoly->size());
+#if defined(_OPENMP)
+#pragma omp parallel for
+#endif
+		for (int i = 0; i < vertexCount; ++i)
+		{
+			const CCVector3* P = m_segmentationPoly->getPoint(i);
+
+			CCVector3d Q2D;
+			bool pointInFrustum = false;
+			camera.project(*P, Q2D, &pointInFrustum);
+
+			if (!pointInFrustum)
+			{
+				polyInsideFrustum = true;
+			}
+		}
+	}
+
 	//for each selected entity
 	for (QSet<ccHObject*>::const_iterator p = m_toSegment.constBegin(); p != m_toSegment.constEnd(); ++p)
 	{
@@ -638,25 +661,30 @@ void ccGraphicalSegmentationTool::segment(bool keepPointsInside)
 		ccGenericPointCloud::VisibilityTableType& visibilityArray = cloud->getTheVisibilityArray();
 		assert(!visibilityArray.empty());
 
-		unsigned cloudSize = cloud->size();
+		int cloudSize = static_cast<int>(cloud->size());
 
 		//we project each point and we check if it falls inside the segmentation polyline
 #if defined(_OPENMP)
 #pragma omp parallel for
 #endif
-		for (int i = 0; i < static_cast<int>(cloudSize); ++i)
+		for (int i = 0; i < cloudSize; ++i)
 		{
 			if (visibilityArray[i] == POINT_VISIBLE)
 			{
 				const CCVector3* P3D = cloud->getPoint(i);
 
 				CCVector3d Q2D;
-				camera.project(*P3D, Q2D);
+				bool pointInFrustum = false;
+				camera.project(*P3D, Q2D, &pointInFrustum);
 
-				CCVector2 P2D(	static_cast<PointCoordinateType>(Q2D.x-half_w),
-								static_cast<PointCoordinateType>(Q2D.y-half_h) );
-				
-				bool pointInside = CCLib::ManualSegmentationTools::isPointInsidePoly(P2D, m_segmentationPoly);
+				bool pointInside = false;
+				if (pointInFrustum || !polyInsideFrustum) //we can only skip the test if the polyline is fully inside the frustum
+				{
+					CCVector2 P2D(	static_cast<PointCoordinateType>(Q2D.x - half_w),
+									static_cast<PointCoordinateType>(Q2D.y - half_h));
+
+					pointInside = CCLib::ManualSegmentationTools::isPointInsidePoly(P2D, m_segmentationPoly);
+				}
 
 				visibilityArray[i] = (keepPointsInside != pointInside ? POINT_HIDDEN : POINT_VISIBLE);
 			}
