@@ -216,6 +216,15 @@ namespace ccLibAlgorithms
 			case CCLib::Neighbourhood::Verticality:
 				sfName = "Verticality";
 				break;
+			case CCLib::Neighbourhood::EigenValue1:
+				sfName = "1st eigenvalue";
+				break;
+			case CCLib::Neighbourhood::EigenValue2:
+				sfName = "2nd eigenvalue";
+				break;
+			case CCLib::Neighbourhood::EigenValue3:
+				sfName = "3rd eigenvalue";
+				break;
 			default:
 				assert(false);
 				ccLog::Error("Internal error: invalid sub option for Feature computation");
@@ -283,7 +292,7 @@ namespace ccLibAlgorithms
 			{
 				ccGenericPointCloud* cloud = ccHObjectCaster::ToGenericPointCloud(entities[i]);
 
-				ccPointCloud* pc = 0;
+				ccPointCloud* pc = nullptr;
 				int sfIdx = -1;
 				if (cloud->isA(CC_TYPES::POINT_CLOUD))
 				{
@@ -431,7 +440,7 @@ namespace ccLibAlgorithms
 		for (size_t i = 0; i < selNum; ++i)
 		{
 			//is the ith selected data is eligible for processing?
-			ccGenericPointCloud* cloud = 0;
+			ccGenericPointCloud* cloud = nullptr;
 			switch (algo)
 			{
 				case CCLIB_ALGO_SF_GRADIENT:
@@ -441,7 +450,7 @@ namespace ccLibAlgorithms
 					if (lockedVertices)
 					{
 						ccUtils::DisplayLockedVerticesWarning(entities[i]->getName(), selNum == 1);
-						cloud = 0;
+						cloud = nullptr;
 					}
 					if (cloud)
 					{
@@ -452,7 +461,7 @@ namespace ccLibAlgorithms
 							int outSfIdx = pc->getCurrentDisplayedScalarFieldIndex();
 							if (outSfIdx < 0)
 							{
-								cloud = 0;
+								cloud = nullptr;
 							}
 							else
 							{
@@ -463,7 +472,7 @@ namespace ccLibAlgorithms
 						}
 						else //if (!cloud->hasDisplayedScalarField()) //TODO: displayed but not necessarily set as OUTPUT!
 						{
-							cloud = 0;
+							cloud = nullptr;
 						}
 					}
 					break;
@@ -477,7 +486,7 @@ namespace ccLibAlgorithms
 			
 			if (cloud)
 			{
-				ccPointCloud* pc = 0;
+				ccPointCloud* pc = nullptr;
 				int sfIdx = -1;
 				if (cloud->isA(CC_TYPES::POINT_CLOUD))
 				{
@@ -640,7 +649,8 @@ namespace ccLibAlgorithms
 						{
 							const CCVector3* X = Yk.getLSPlaneX();
 							const CCVector3* O = Yk.getGravityCenter();
-							double minX = 0,maxX = 0;
+							double minX = 0;
+							double maxX = 0;
 							for (unsigned j=0; j<cloud->size(); ++j)
 							{
 								double x = (*cloud->getPoint(j) - *O).dot(*X);
