@@ -19,6 +19,7 @@
 
 //Local
 #include "mainwindow.h"
+#include "ccReservedIDs.h"
 
 //common
 #include <ccPickingHub.h>
@@ -63,10 +64,10 @@ ccTracePolylineTool::SegmentGLParams::SegmentGLParams(ccGenericGLDisplay* displa
 ccTracePolylineTool::ccTracePolylineTool(ccPickingHub* pickingHub, QWidget* parent)
 	: ccOverlayDialog(parent)
 	, Ui::TracePolyLineDlg()
-	, m_polyTip(0)
-	, m_polyTipVertices(0)
-	, m_poly3D(0)
-	, m_poly3DVertices(0)
+	, m_polyTip(nullptr)
+	, m_polyTipVertices(nullptr)
+	, m_poly3D(nullptr)
+	, m_poly3DVertices(nullptr)
 	, m_done(false)
 	, m_pickingHub(pickingHub)
 {
@@ -75,25 +76,25 @@ ccTracePolylineTool::ccTracePolylineTool(ccPickingHub* pickingHub, QWidget* pare
 	setupUi(this);
 	setWindowFlags(Qt::FramelessWindowHint | Qt::Tool);
 
-	connect(saveToolButton, SIGNAL(clicked()), this, SLOT(exportLine()));
-	connect(resetToolButton, SIGNAL(clicked()), this, SLOT(resetLine()));
-	connect(continueToolButton, SIGNAL(clicked()), this, SLOT(continueEdition()));
-	connect(validButton, SIGNAL(clicked()), this, SLOT(apply()));
-	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
-	connect(widthSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onWidthSizeChanged(int)));
+	connect(saveToolButton,		&QToolButton::clicked, this, &ccTracePolylineTool::exportLine);
+	connect(resetToolButton,	&QToolButton::clicked, this, &ccTracePolylineTool::resetLine);
+	connect(continueToolButton, &QToolButton::clicked, this, &ccTracePolylineTool::continueEdition);
+	connect(validButton,		&QToolButton::clicked, this, &ccTracePolylineTool::apply);
+	connect(cancelButton,		&QToolButton::clicked, this, &ccTracePolylineTool::cancel);
+	connect(widthSpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &ccTracePolylineTool::onWidthSizeChanged);
 
 	//add shortcuts
 	addOverridenShortcut(Qt::Key_Escape); //escape key for the "cancel" button
 	addOverridenShortcut(Qt::Key_Return); //return key for the "apply" button
-	connect(this, SIGNAL(shortcutTriggered(int)), this, SLOT(onShortcutTriggered(int)));
+	connect(this, &ccTracePolylineTool::shortcutTriggered, this, &ccTracePolylineTool::onShortcutTriggered);
 
-	m_polyTipVertices = new ccPointCloud("Tip vertices");
+	m_polyTipVertices = new ccPointCloud("Tip vertices", static_cast<unsigned>(ReservedIDs::TRACE_POLYLINE_TOOL_POLYLINE_TIP_VERTICES));
 	m_polyTipVertices->reserve(2);
 	m_polyTipVertices->addPoint(CCVector3(0, 0, 0));
 	m_polyTipVertices->addPoint(CCVector3(1, 1, 1));
 	m_polyTipVertices->setEnabled(false);
 
-	m_polyTip = new ccPolyline(m_polyTipVertices);
+	m_polyTip = new ccPolyline(m_polyTipVertices, static_cast<unsigned>(ReservedIDs::TRACE_POLYLINE_TOOL_POLYLINE_TIP));
 	m_polyTip->setForeground(true);
 	m_polyTip->setTempColor(ccColor::green);
 	m_polyTip->set2DMode(true);
@@ -143,13 +144,13 @@ ccPolyline* ccTracePolylineTool::polylineOverSampling(unsigned steps) const
 	if (!m_poly3D || !m_poly3DVertices || m_segmentParams.size() != m_poly3DVertices->size())
 	{
 		assert(false);
-		return 0;
+		return nullptr;
 	}
 
 	if (steps <= 1)
 	{
 		//nothing to do
-		return 0;
+		return nullptr;
 	}
 
 	ccHObject::Container clouds;
@@ -161,7 +162,7 @@ ccPolyline* ccTracePolylineTool::polylineOverSampling(unsigned steps) const
 	{
 		//no entity is currently displayed?!
 		assert(false);
-		return 0;
+		return nullptr;
 	}
 
 	unsigned n_verts = m_poly3DVertices->size();
@@ -177,7 +178,7 @@ ccPolyline* ccTracePolylineTool::polylineOverSampling(unsigned steps) const
 	{
 		ccLog::Warning("[ccTracePolylineTool::PolylineOverSampling] Not enough memory");
 		delete newPoly;
-		return 0;
+		return nullptr;
 	}
 	newVertices->importParametersFrom(m_poly3DVertices);
 	newVertices->setName(m_poly3DVertices->getName());
@@ -185,7 +186,7 @@ ccPolyline* ccTracePolylineTool::polylineOverSampling(unsigned steps) const
 	newPoly->importParametersFrom(*m_poly3D);
 	newPoly->setDisplay_recursive(m_poly3D->getDisplay());
 
-	QProgressDialog pDlg(QString("Oversampling"), "Cancel", 0, static_cast<int>(end_size), m_associatedWin ? m_associatedWin->asWidget() : 0);
+	QProgressDialog pDlg(QString("Oversampling"), "Cancel", 0, static_cast<int>(end_size), m_associatedWin ? m_associatedWin->asWidget() : nullptr);
 	pDlg.show();
 	QCoreApplication::processEvents();
 
@@ -194,13 +195,22 @@ ccPolyline* ccTracePolylineTool::polylineOverSampling(unsigned steps) const
 		const CCVector3* p1 = m_poly3DVertices->getPoint(i);
 		newVertices->addPoint(*p1);
 
+
 		unsigned i2 = (i + 1) % n_verts;
-		CCVector2d v = m_segmentParams[i2].clickPos - m_segmentParams[i].clickPos;
+		CCVector2d clickPos1/* = m_segmentParams[i].clickPos*/;
+		{
+			//we actually retro-project the 3D point in the second vertex camera frame so as to get a proper behavior
+			CCVector3d P2D;
+			m_segmentParams[i2].params.project(*p1, P2D);
+			clickPos1 = CCVector2d(P2D.x, P2D.y);
+		}
+
+		CCVector2d v = m_segmentParams[i2].clickPos - clickPos1;
 		v /= steps;
 
 		for (unsigned j = 1; j < steps; j++)
 		{
-			CCVector2d vj = m_segmentParams[i].clickPos + v * j;
+			CCVector2d vj = clickPos1 + v * j;
 
 			CCVector3 nearestPoint;
 			double nearestElementSquareDist = -1.0;
@@ -303,15 +313,13 @@ bool ccTracePolylineTool::linkWith(ccGLWindow* win)
 		oldWin->disconnect(this);
 
 		if (m_polyTip)
-			m_polyTip->setDisplay(0);
+			m_polyTip->setDisplay(nullptr);
 	}
 
 	if (m_associatedWin)
 	{
-		//connect(m_associatedWin, SIGNAL(itemPicked(ccHObject*, unsigned, int, int, const CCVector3&)), this, SLOT(handlePickedItem(ccHObject*, unsigned, int, int, const CCVector3&)));
-		//connect(m_associatedWin, SIGNAL(leftButtonClicked(int, int)), this, SLOT(addPointToPolyline(int, int)));
-		connect(m_associatedWin, SIGNAL(rightButtonClicked(int, int)), this, SLOT(closePolyLine(int, int)));
-		connect(m_associatedWin, SIGNAL(mouseMoved(int, int, Qt::MouseButtons)), this, SLOT(updatePolyLineTip(int, int, Qt::MouseButtons)));
+		connect(m_associatedWin, &ccGLWindow::rightButtonClicked, this, &ccTracePolylineTool::closePolyLine);
+		connect(m_associatedWin, &ccGLWindow::mouseMoved,		  this, &ccTracePolylineTool::updatePolyLineTip);
 	}
 
 	return true;
@@ -470,11 +478,11 @@ void ccTracePolylineTool::onItemPicked(const PickedItem& pi)
 	//if the 3D polyline doesn't exist yet, we create it
 	if (!m_poly3D || !m_poly3DVertices)
 	{
-		m_poly3DVertices = new ccPointCloud("Vertices");
+		m_poly3DVertices = new ccPointCloud("Vertices", static_cast<unsigned>(ReservedIDs::TRACE_POLYLINE_TOOL_POLYLINE_VERTICES));
 		m_poly3DVertices->setEnabled(false);
 		m_poly3DVertices->setDisplay(m_associatedWin);
 
-		m_poly3D = new ccPolyline(m_poly3DVertices);
+		m_poly3D = new ccPolyline(m_poly3DVertices, static_cast<unsigned>(ReservedIDs::TRACE_POLYLINE_TOOL_POLYLINE));
 		m_poly3D->setTempColor(ccColor::green);
 		m_poly3D->set2DMode(false);
 		m_poly3D->addChild(m_poly3DVertices);
@@ -589,8 +597,8 @@ void ccTracePolylineTool::restart(bool reset)
 			delete m_poly3D;
 			m_segmentParams.resize(0);
 			//delete m_poly3DVertices;
-			m_poly3D = 0;
-			m_poly3DVertices = 0;
+			m_poly3D = nullptr;
+			m_poly3DVertices = nullptr;
 		}
 		else
 		{
@@ -640,7 +648,7 @@ void ccTracePolylineTool::exportLine()
 		{
 			delete m_poly3D;
 			m_segmentParams.resize(0);
-			m_poly3DVertices = 0;
+			m_poly3DVertices = nullptr;
 			m_poly3D = poly;
 		}
 	}
@@ -656,9 +664,9 @@ void ccTracePolylineTool::exportLine()
 		assert(false);
 	}
 
-	m_poly3D = 0;
+	m_poly3D = nullptr;
 	m_segmentParams.resize(0);
-	m_poly3DVertices = 0;
+	m_poly3DVertices = nullptr;
 
 	resetLine(); //to update the GUI
 }
@@ -690,6 +698,6 @@ void ccTracePolylineTool::onWidthSizeChanged(int width)
 	
 	if (m_associatedWin)
 	{
-		m_associatedWin->redraw(m_poly3D == 0, false);
+		m_associatedWin->redraw(m_poly3D == nullptr, false);
 	}
 }
