@@ -29,7 +29,6 @@
 #include <ccPointCloud.h>
 #include <ccScalarField.h>
 #include <ccProgressDialog.h>
-#include <ccPolyline.h>
 #include <ccMesh.h>
 
 //qCC_gl
@@ -60,23 +59,23 @@ ccVolumeCalcTool::ccVolumeCalcTool(ccGenericPointCloud* cloud1, ccGenericPointCl
 {
 	setupUi(this);
 
-	connect(buttonBox,						SIGNAL(accepted()),					this,	SLOT(saveSettingsAndAccept()));
-	connect(buttonBox,						SIGNAL(rejected()),					this,	SLOT(reject()));
-	connect(gridStepDoubleSpinBox,			SIGNAL(valueChanged(double)),		this,	SLOT(updateGridInfo()));
-	connect(gridStepDoubleSpinBox,			SIGNAL(valueChanged(double)),		this,	SLOT(gridOptionChanged()));
-	connect(groundEmptyValueDoubleSpinBox,	SIGNAL(valueChanged(double)),		this,	SLOT(gridOptionChanged()));
-	connect(ceilEmptyValueDoubleSpinBox,	SIGNAL(valueChanged(double)),		this,	SLOT(gridOptionChanged()));
-	connect(projDimComboBox,				SIGNAL(currentIndexChanged(int)),	this,	SLOT(projectionDirChanged(int)));
-	connect(updatePushButton,				SIGNAL(clicked()),					this,	SLOT(updateGridAndDisplay()));
-	connect(heightProjectionComboBox,		SIGNAL(currentIndexChanged(int)),	this,	SLOT(gridOptionChanged()));
-	connect(fillGroundEmptyCellsComboBox,	SIGNAL(currentIndexChanged(int)),	this,	SLOT(groundFillEmptyCellStrategyChanged(int)));
-	connect(fillCeilEmptyCellsComboBox,		SIGNAL(currentIndexChanged(int)),	this,	SLOT(ceilFillEmptyCellStrategyChanged(int)));
-	connect(swapToolButton,					SIGNAL(clicked()),					this,	SLOT(swapRoles()));
-	connect(groundComboBox,					SIGNAL(currentIndexChanged(int)),	this,	SLOT(groundSourceChanged(int)));
-	connect(ceilComboBox,					SIGNAL(currentIndexChanged(int)),	this,	SLOT(ceilSourceChanged(int)));
-	connect(clipboardPushButton,			SIGNAL(clicked()),					this,	SLOT(exportToClipboard()));
-	connect(exportGridPushButton,			SIGNAL(clicked()),					this,	SLOT(exportGridAsCloud()));
-	connect(precisionSpinBox,				SIGNAL(valueChanged(int)),			this,	SLOT(setDisplayedNumberPrecision(int)));
+	connect(buttonBox,						&QDialogButtonBox::accepted,														this,	&ccVolumeCalcTool::saveSettingsAndAccept);
+	connect(buttonBox,						&QDialogButtonBox::rejected,														this,	&ccVolumeCalcTool::reject);
+	connect(gridStepDoubleSpinBox,			static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),		this,	&ccVolumeCalcTool::updateGridInfo);
+	connect(gridStepDoubleSpinBox,			static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),		this,	&ccVolumeCalcTool::gridOptionChanged);
+	connect(groundEmptyValueDoubleSpinBox,	static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),		this,	&ccVolumeCalcTool::gridOptionChanged);
+	connect(ceilEmptyValueDoubleSpinBox,	static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),		this,	&ccVolumeCalcTool::gridOptionChanged);
+	connect(projDimComboBox,				static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),				this,	&ccVolumeCalcTool::projectionDirChanged);
+	connect(updatePushButton,				&QPushButton::clicked,																this,	&ccVolumeCalcTool::updateGridAndDisplay);
+	connect(heightProjectionComboBox,		static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),				this,	&ccVolumeCalcTool::gridOptionChanged);
+	connect(fillGroundEmptyCellsComboBox,	static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),				this,	&ccVolumeCalcTool::groundFillEmptyCellStrategyChanged);
+	connect(fillCeilEmptyCellsComboBox,		static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),				this,	&ccVolumeCalcTool::ceilFillEmptyCellStrategyChanged);
+	connect(swapToolButton,					&QToolButton::clicked,																this,	&ccVolumeCalcTool::swapRoles);
+	connect(groundComboBox,					static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),				this,	&ccVolumeCalcTool::groundSourceChanged);
+	connect(ceilComboBox,					static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),				this,	&ccVolumeCalcTool::ceilSourceChanged);
+	connect(clipboardPushButton,			&QPushButton::clicked,																this,	&ccVolumeCalcTool::exportToClipboard);
+	connect(exportGridPushButton,			&QPushButton::clicked,																this,	&ccVolumeCalcTool::exportGridAsCloud);
+	connect(precisionSpinBox,				static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),						this,	&ccVolumeCalcTool::setDisplayedNumberPrecision);
 
 	if (m_cloud1 && !m_cloud2)
 	{
@@ -94,7 +93,7 @@ ccVolumeCalcTool::ccVolumeCalcTool(ccGenericPointCloud* cloud1, ccGenericPointCl
 	if (gridBBox.isValid())
 	{
 		createBoundingBoxEditor(gridBBox, this);
-		connect(editGridToolButton, SIGNAL(clicked()), this, SLOT(showGridBoxEditor()));
+		connect(editGridToolButton, &QToolButton::clicked, this, &ccVolumeCalcTool::showGridBoxEditor);
 	}
 	else
 	{
@@ -134,10 +133,6 @@ ccVolumeCalcTool::ccVolumeCalcTool(ccGenericPointCloud* cloud1, ccGenericPointCl
 	gridIsUpToDate(false);
 }
 
-ccVolumeCalcTool::~ccVolumeCalcTool()
-{
-}
-
 void ccVolumeCalcTool::setDisplayedNumberPrecision(int precision)
 {
 	//update window
@@ -341,7 +336,7 @@ ccPointCloud* ccVolumeCalcTool::ConvertGridToCloud(	ccRasterGrid& grid,
 	assert(gridBox.isValid());
 	assert(vertDim < 3);
 
-	ccPointCloud* rasterCloud = 0;
+	ccPointCloud* rasterCloud = nullptr;
 	try
 	{
 		//we only compute the default 'height' layer
@@ -353,7 +348,7 @@ ccPointCloud* ccVolumeCalcTool::ConvertGridToCloud(	ccRasterGrid& grid,
 			false,
 			false,
 			false,
-			0,
+			nullptr,
 			vertDim,
 			gridBox,
 			false,
@@ -377,7 +372,7 @@ ccPointCloud* ccVolumeCalcTool::ConvertGridToCloud(	ccRasterGrid& grid,
 		if (rasterCloud)
 		{
 			delete rasterCloud;
-			rasterCloud = 0;
+			rasterCloud = nullptr;
 		}
 	}
 
@@ -386,7 +381,7 @@ ccPointCloud* ccVolumeCalcTool::ConvertGridToCloud(	ccRasterGrid& grid,
 
 ccPointCloud* ccVolumeCalcTool::convertGridToCloud(bool exportToOriginalCS) const
 {
-	ccPointCloud* rasterCloud = 0;
+	ccPointCloud* rasterCloud = nullptr;
 	try
 	{
 		//we only compute the default 'height' layer
@@ -397,7 +392,7 @@ ccPointCloud* ccVolumeCalcTool::convertGridToCloud(bool exportToOriginalCS) cons
 																false,
 																false,
 																false,
-																0,
+																nullptr,
 																false,
 																std::numeric_limits<double>::quiet_NaN(),
 																exportToOriginalCS);
@@ -419,7 +414,7 @@ ccPointCloud* ccVolumeCalcTool::convertGridToCloud(bool exportToOriginalCS) cons
 		if (rasterCloud)
 		{
 			delete rasterCloud;
-			rasterCloud = 0;
+			rasterCloud = nullptr;
 		}
 	}
 
@@ -436,7 +431,7 @@ void ccVolumeCalcTool::updateGridAndDisplay()
 		{
 			m_glWindow->removeFromOwnDB(m_rasterCloud);
 			delete m_rasterCloud;
-			m_rasterCloud = 0;
+			m_rasterCloud = nullptr;
 		}
 
 		m_rasterCloud = convertGridToCloud(false);
@@ -563,7 +558,7 @@ bool ccVolumeCalcTool::ComputeVolume(	ccRasterGrid& grid,
 	}
 
 	//progress dialog
-	QScopedPointer<ccProgressDialog> pDlg(0);
+	QScopedPointer<ccProgressDialog> pDlg(nullptr);
 	if (parentWidget)
 	{
 		pDlg.reset(new ccProgressDialog(true, parentWidget));
@@ -773,7 +768,8 @@ bool ccVolumeCalcTool::updateGrid()
 		return false;
 	}
 
-	unsigned gridWidth = 0, gridHeight = 0;
+	unsigned gridWidth = 0;
+	unsigned gridHeight = 0;
 	if (!getGridSize(gridWidth, gridHeight))
 	{
 		return false;
@@ -784,7 +780,7 @@ bool ccVolumeCalcTool::updateGrid()
 	assert(gridStep != 0);
 
 	//ground
-	ccGenericPointCloud* groundCloud = 0;
+	ccGenericPointCloud* groundCloud = nullptr;
 	double groundHeight = 0;
 	switch (groundComboBox->currentIndex())
 	{
@@ -803,7 +799,7 @@ bool ccVolumeCalcTool::updateGrid()
 	}
 
 	//ceil
-	ccGenericPointCloud* ceilCloud = 0;
+	ccGenericPointCloud* ceilCloud = nullptr;
 	double ceilHeight = 0;
 	switch (ceilComboBox->currentIndex())
 	{
