@@ -29,9 +29,6 @@
 //CCLib
 #include <AutoSegmentationTools.h>
 
-//internal db
-#include "db_tree/ccDBRoot.h"
-
 class QAction;
 class QMdiArea;
 class QMdiSubWindow;
@@ -42,6 +39,7 @@ class cc3DMouseManager;
 class ccCameraParamEditDlg;
 class ccClippingBoxTool;
 class ccComparisonDlg;
+class ccDBRoot;
 class ccDrawableObject;
 class ccGamepadManager;
 class ccGLWindow;
@@ -59,6 +57,8 @@ class ccSectionExtractionTool;
 class ccStdPluginInterface;
 class ccTracePolylineTool;
 
+struct dbTreeSelectionInfo;
+
 namespace Ui {
 	class MainWindow;
 } 
@@ -170,7 +170,7 @@ public:
 	
 private slots:
 	//! Creates a new 3D GL sub-window
-	ccGLWindow* new3DView();
+	ccGLWindow* new3DView( bool allowEntitySelection );
 
 	//! Zooms in (current 3D view)
 	void zoomIn();
@@ -261,6 +261,7 @@ private slots:
 	void doActionInterpolateColors();
 	void doActionChangeColorLevels();
 	void doActionEnhanceRGBWithIntensities();
+	void doActionColorFromScalars();
 
 	void doActionSFGaussianFilter();
 	void doActionSFBilateralFilter();
@@ -313,6 +314,8 @@ private slots:
 	
 	void doActionCreatePlane();
 	void doActionEditPlane();
+	void doActionFlipPlane();
+	void doActionComparePlanes();
 
 	void doActionDeleteScanGrids();
 	void doActionSmoothMeshSF();
@@ -333,6 +336,7 @@ private slots:
 	void doActionFlagMeshVertices();
 	void doActionSmoothMeshLaplacian();
 	void doActionSubdivideMesh();
+	void doActionFlipMeshTriangles();
 	void doActionComputeCPS();
 	void doActionShowWaveDialog();
 	void doActionCompressFWFData();
@@ -361,6 +365,7 @@ private slots:
 	void doConvertPolylinesToMesh();
 	void doMeshTwoPolylines();
 	void doActionExportCoordToSF();
+	void doActionExportNormalToSF();
 	void doComputeBestFitBB();
 	void doActionCrop();
 
@@ -398,6 +403,7 @@ private slots:
 	//Entities comparison
 	void doActionCloudCloudDist();
 	void doActionCloudMeshDist();
+	void doActionCloudPrimitiveDist();
 	void deactivateComparisonMode(int);
 
 	//Point picking mechanism
@@ -431,6 +437,10 @@ private slots:
 	//! Creates a cloud with the (bounding-box) centers of all selected entities
 	void doActionCreateCloudFromEntCenters();
 
+	inline void doActionMoveBBCenterToOrigin()    { doActionFastRegistration(MoveBBCenterToOrigin); }
+	inline void doActionMoveBBMinCornerToOrigin() { doActionFastRegistration(MoveBBMinCornerToOrigin); }
+	inline void doActionMoveBBMaxCornerToOrigin() { doActionFastRegistration(MoveBBMaxCornerToOrigin); }
+
 private:
 	//! Shortcut: asks the user to select one cloud
 	/** \param defaultCloudEntity a cloud to select by default (optional)
@@ -439,8 +449,17 @@ private:
 	**/
 	ccPointCloud* askUserToSelectACloud(ccHObject* defaultCloudEntity = nullptr, QString inviteMessage = QString());
 
-	void	toggleSelectedEntitiesProperty( ccEntityAction::TOGGLE_PROPERTY property );
-	void	clearSelectedEntitiesProperty( ccEntityAction::CLEAR_PROPERTY property );
+	enum FastRegistrationMode
+	{
+		MoveBBCenterToOrigin,
+		MoveBBMinCornerToOrigin,
+		MoveBBMaxCornerToOrigin
+	};
+
+	void doActionFastRegistration(FastRegistrationMode mode);
+
+	void toggleSelectedEntitiesProperty( ccEntityAction::TOGGLE_PROPERTY property );
+	void clearSelectedEntitiesProperty( ccEntityAction::CLEAR_PROPERTY property );
 	
 	void setView( CC_VIEW_ORIENTATION view ) override;
 	
